/*! Built with IMPACT - impactjs.com */


// -----------------------------------------------------------------------------
// Impact Game Engine 1.24
// http://impactjs.com/
// -----------------------------------------------------------------------------


(function(window){ "use strict";

// -----------------------------------------------------------------------------
// Native Object extensions

Number.prototype.map = function(istart, istop, ostart, ostop) {
	return ostart + (ostop - ostart) * ((this - istart) / (istop - istart));
};

Number.prototype.limit = function(min, max) {
	return Math.min(max, Math.max(min, this));
};

Number.prototype.round = function(precision) {
	precision = Math.pow(10, precision || 0);
	return Math.round(this * precision) / precision;
};

Number.prototype.floor = function() {
	return Math.floor(this);
};

Number.prototype.ceil = function() {
	return Math.ceil(this);
};

Number.prototype.toInt = function() {
	return (this | 0);
};

Number.prototype.toRad = function() {
	return (this / 180) * Math.PI;
};

Number.prototype.toDeg = function() {
	return (this * 180) / Math.PI;
};

Object.defineProperty(Array.prototype, 'erase', {value: function(item) {
	for( var i = this.length; i--; ) {
		if( this[i] === item ) {
			this.splice(i, 1);
		}
	}
	return this;
}});

Object.defineProperty(Array.prototype, 'random', {value: function(item) {
	return this[ Math.floor(Math.random() * this.length) ];
}});

Function.prototype.bind = Function.prototype.bind || function (oThis) {
	if( typeof this !== "function" ) {
		throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");
	}

	var aArgs = Array.prototype.slice.call(arguments, 1),
		fToBind = this,
		fNOP = function () {},
		fBound = function () {
			return fToBind.apply(
				(this instanceof fNOP && oThis ? this : oThis),
				aArgs.concat(Array.prototype.slice.call(arguments))
			);
		};

	fNOP.prototype = this.prototype;
	fBound.prototype = new fNOP();

	return fBound;
};


// -----------------------------------------------------------------------------
// ig Namespace

window.ig = {
	game: null,
	debug: null,
	version: '1.24',
	global: window,
	modules: {},
	resources: [],
	ready: false,
	baked: false,
	nocache: '',
	ua: {},
	prefix: (window.ImpactPrefix || ''),
	lib: 'lib/',
	
	_current: null,
	_loadQueue: [],
	_waitForOnload: 0,
	
	
	$: function( selector ) {
		return selector.charAt(0) == '#'
			? document.getElementById( selector.substr(1) )
			: document.getElementsByTagName( selector );
	},
	
	
	$new: function( name ) {
		return document.createElement( name );
	},
	
	
	copy: function( object ) {
		if(
		   !object || typeof(object) != 'object' ||
		   object instanceof HTMLElement ||
		   object instanceof ig.Class
		) {
			return object;
		}
		else if( object instanceof Array ) {
			var c = [];
			for( var i = 0, l = object.length; i < l; i++) {
				c[i] = ig.copy(object[i]);
			}
			return c;
		}
		else {
			var c = {};
			for( var i in object ) {
				c[i] = ig.copy(object[i]);
			}
			return c;
		}
	},
	
	
	merge: function( original, extended ) {
		for( var key in extended ) {
			var ext = extended[key];
			if(
				typeof(ext) != 'object' ||
				ext instanceof HTMLElement ||
				ext instanceof ig.Class ||
				ext === null
			) {
				original[key] = ext;
			}
			else {
				if( !original[key] || typeof(original[key]) != 'object' ) {
					original[key] = (ext instanceof Array) ? [] : {};
				}
				ig.merge( original[key], ext );
			}
		}
		return original;
	},
	
	
	ksort: function( obj ) {
		if( !obj || typeof(obj) != 'object' ) {
			return [];
		}
		
		var keys = [], values = [];
		for( var i in obj ) {
			keys.push(i);
		}
		
		keys.sort();
		for( var i = 0; i < keys.length; i++ ) {
			values.push( obj[keys[i]] );
		}
		
		return values;
	},

	// Ah, yes. I love vendor prefixes. So much fun!
	setVendorAttribute: function( el, attr, val ) {
		var uc = attr.charAt(0).toUpperCase() + attr.substr(1);
		el[attr] = el['ms'+uc] = el['moz'+uc] = el['webkit'+uc] = el['o'+uc] = val;
	},


	getVendorAttribute: function( el, attr ) {
		var uc = attr.charAt(0).toUpperCase() + attr.substr(1);
		return el[attr] || el['ms'+uc] || el['moz'+uc] || el['webkit'+uc] || el['o'+uc];
	},


	normalizeVendorAttribute: function( el, attr ) {
		var prefixedVal = ig.getVendorAttribute( el, attr );
		if( !el[attr] && prefixedVal ) {
			el[attr] = prefixedVal;
		}
	},


	// This function normalizes getImageData to extract the real, actual
	// pixels from an image. The naive method recently failed on retina
	// devices with a backgingStoreRatio != 1
	getImagePixels: function( image, x, y, width, height ) {
		var canvas = ig.$new('canvas');
		canvas.width = image.width;
		canvas.height = image.height;
		var ctx = canvas.getContext('2d');
		
		// Try to draw pixels as accurately as possible
		ig.System.SCALE.CRISP(canvas, ctx);

		var ratio = ig.getVendorAttribute( ctx, 'backingStorePixelRatio' ) || 1;
		ig.normalizeVendorAttribute( ctx, 'getImageDataHD' );

		var realWidth = image.width / ratio,
			realHeight = image.height / ratio;

		canvas.width = Math.ceil( realWidth );
		canvas.height = Math.ceil( realHeight );

		ctx.drawImage( image, 0, 0, realWidth, realHeight );
		
		return (ratio === 1)
			? ctx.getImageData( x, y, width, height )
			: ctx.getImageDataHD( x, y, width, height );
	},

	
	module: function( name ) {
		if( ig._current ) {
			throw( "Module '"+ig._current.name+"' defines nothing" );
		}
		if( ig.modules[name] && ig.modules[name].body ) {
			throw( "Module '"+name+"' is already defined" );
		}
		
		ig._current = {name: name, requires: [], loaded: false, body: null};
		ig.modules[name] = ig._current;
		ig._loadQueue.push(ig._current);
		return ig;
	},
	
	
	requires: function() {
		ig._current.requires = Array.prototype.slice.call(arguments);
		return ig;
	},
	
	
	defines: function( body ) {
		ig._current.body = body;
		ig._current = null;
		ig._initDOMReady();
	},
	
	
	addResource: function( resource ) {
		ig.resources.push( resource );
	},
	
	
	setNocache: function( set ) {
		ig.nocache = set
			? '?' + Date.now()
			: '';
	},
	
	
	// Stubs for ig.Debug
	log: function() {},
	assert: function( condition, msg ) {},
	show: function( name, number ) {},
	mark: function( msg, color ) {},
	
	
	_loadScript: function( name, requiredFrom ) {
		ig.modules[name] = {name: name, requires:[], loaded: false, body: null};
		ig._waitForOnload++;
		
		var path = ig.prefix + ig.lib + name.replace(/\./g, '/') + '.js' + ig.nocache;
		var script = ig.$new('script');
		script.type = 'text/javascript';
		script.src = path;
		script.onload = function() {
			ig._waitForOnload--;
			ig._execModules();
		};
		script.onerror = function() {
			throw(
				'Failed to load module '+name+' at ' + path + ' ' +
				'required from ' + requiredFrom
			);
		};
		ig.$('head')[0].appendChild(script);
	},

	
	_execModules: function() {
		var modulesLoaded = false;
		for( var i = 0; i < ig._loadQueue.length; i++ ) {
			var m = ig._loadQueue[i];
			var dependenciesLoaded = true;
			
			for( var j = 0; j < m.requires.length; j++ ) {
				var name = m.requires[j];
				if( !ig.modules[name] ) {
					dependenciesLoaded = false;
					ig._loadScript( name, m.name );
				}
				else if( !ig.modules[name].loaded ) {
					dependenciesLoaded = false;
				}
			}
			
			if( dependenciesLoaded && m.body ) {
				ig._loadQueue.splice(i, 1);
				m.loaded = true;
				m.body();
				modulesLoaded = true;
				i--;
			}
		}
		
		if( modulesLoaded ) {
			ig._execModules();
		}
		
		// No modules executed, no more files to load but loadQueue not empty?
		// Must be some unresolved dependencies!
		else if( !ig.baked && ig._waitForOnload == 0 && ig._loadQueue.length != 0 ) {
			var unresolved = [];
			for( var i = 0; i < ig._loadQueue.length; i++ ) {
				
				// Which dependencies aren't loaded?
				var unloaded = [];
				var requires = ig._loadQueue[i].requires;
				for( var j = 0; j < requires.length; j++ ) {
					var m = ig.modules[ requires[j] ];
					if( !m || !m.loaded ) {
						unloaded.push( requires[j] );
					}
				}
				unresolved.push( ig._loadQueue[i].name + ' (requires: ' + unloaded.join(', ') + ')');
			}
			
			throw( 
				"Unresolved (or circular?) dependencies. " +
				"Most likely there's a name/path mismatch for one of the listed modules " +
				"or a previous syntax error prevents a module from loading:\n" +
				unresolved.join('\n')				
			);
		}
	},
	
	
	_DOMReady: function() {
		if( !ig.modules['dom.ready'].loaded ) {
			if ( !document.body ) {
				return setTimeout( ig._DOMReady, 13 );
			}
			ig.modules['dom.ready'].loaded = true;
			ig._waitForOnload--;
			ig._execModules();
		}
		return 0;
	},
	
	
	_boot: function() {
		if( document.location.href.match(/\?nocache/) ) {
			ig.setNocache( true );
		}
		
		// Probe user agent string
		ig.ua.pixelRatio = window.devicePixelRatio || 1;
		ig.ua.viewport = {
			width: window.innerWidth,
			height: window.innerHeight
		};
		ig.ua.screen = {
			width: window.screen.availWidth * ig.ua.pixelRatio,
			height: window.screen.availHeight * ig.ua.pixelRatio
		};
		
		ig.ua.iPhone = /iPhone/i.test(navigator.userAgent);
		ig.ua.iPhone4 = (ig.ua.iPhone && ig.ua.pixelRatio == 2);
		ig.ua.iPad = /iPad/i.test(navigator.userAgent);
		ig.ua.android = /android/i.test(navigator.userAgent);
		ig.ua.winPhone = /Windows Phone/i.test(navigator.userAgent);
		ig.ua.iOS = ig.ua.iPhone || ig.ua.iPad;
		ig.ua.mobile = ig.ua.iOS || ig.ua.android || ig.ua.winPhone || /mobile/i.test(navigator.userAgent);
		ig.ua.touchDevice = (('ontouchstart' in window) || (window.navigator.msMaxTouchPoints));
	},
	
	
	_initDOMReady: function() {
		if( ig.modules['dom.ready'] ) {
			ig._execModules();
			return;
		}
		
		ig._boot();
		
		
		ig.modules['dom.ready'] = { requires: [], loaded: false, body: null };
		ig._waitForOnload++;
		if ( document.readyState === 'complete' ) {
			ig._DOMReady();
		}
		else {
			document.addEventListener( 'DOMContentLoaded', ig._DOMReady, false );
			window.addEventListener( 'load', ig._DOMReady, false );
		}
	}
};


// -----------------------------------------------------------------------------
// Provide ig.setAnimation and ig.clearAnimation as a compatible way to use
// requestAnimationFrame if available or setInterval otherwise

// Use requestAnimationFrame if available
ig.normalizeVendorAttribute( window, 'requestAnimationFrame' );
if( window.requestAnimationFrame ) {
	var next = 1,
		anims = {};

	window.ig.setAnimation = function( callback, element ) {
		var current = next++;
		anims[current] = true;

		var animate = function() {
			if( !anims[current] ) { return; } // deleted?
			window.requestAnimationFrame( animate, element );
			callback();
		};
		window.requestAnimationFrame( animate, element );
		return current;
	};

	window.ig.clearAnimation = function( id ) {
		delete anims[id];
	};
}

// [set/clear]Interval fallback
else {
	window.ig.setAnimation = function( callback, element ) {
		return window.setInterval( callback, 1000/60 );
	};
	window.ig.clearAnimation = function( id ) {
		window.clearInterval( id );
	};
}


// -----------------------------------------------------------------------------
// Class object based on John Resigs code; inspired by base2 and Prototype
// http://ejohn.org/blog/simple-javascript-inheritance/

var initializing = false, fnTest = /xyz/.test(function(){xyz;}) ? /\bparent\b/ : /.*/;
var lastClassId = 0;

window.ig.Class = function(){};
var inject = function(prop) {	
	var proto = this.prototype;
	var parent = {};
	for( var name in prop ) {		
		if( 
			typeof(prop[name]) == "function" &&
			typeof(proto[name]) == "function" && 
			fnTest.test(prop[name])
		) {
			parent[name] = proto[name]; // save original function
			proto[name] = (function(name, fn){
				return function() {
					var tmp = this.parent;
					this.parent = parent[name];
					var ret = fn.apply(this, arguments);			 
					this.parent = tmp;
					return ret;
				};
			})( name, prop[name] );
		}
		else {
			proto[name] = prop[name];
		}
	}
};

window.ig.Class.extend = function(prop) {
	var parent = this.prototype;
 
	initializing = true;
	var prototype = new this();
	initializing = false;
 
	for( var name in prop ) {
		if( 
			typeof(prop[name]) == "function" &&
			typeof(parent[name]) == "function" && 
			fnTest.test(prop[name])
		) {
			prototype[name] = (function(name, fn){
				return function() {
					var tmp = this.parent;
					this.parent = parent[name];
					var ret = fn.apply(this, arguments);			 
					this.parent = tmp;
					return ret;
				};
			})( name, prop[name] );
		}
		else {
			prototype[name] = prop[name];
		}
	}
 
	function Class() {
		if( !initializing ) {
			
			// If this class has a staticInstantiate method, invoke it
			// and check if we got something back. If not, the normal
			// constructor (init) is called.
			if( this.staticInstantiate ) {
				var obj = this.staticInstantiate.apply(this, arguments);
				if( obj ) {
					return obj;
				}
			}
			for( var p in this ) {
				if( typeof(this[p]) == 'object' ) {
					this[p] = ig.copy(this[p]); // deep copy!
				}
			}
			if( this.init ) {
				this.init.apply(this, arguments);
			}
		}
		return this;
	}
	
	Class.prototype = prototype;
	Class.prototype.constructor = Class;
	Class.extend = window.ig.Class.extend;
	Class.inject = inject;
	Class.classId = prototype.classId = ++lastClassId;
	
	return Class;
};

// Merge the ImpactMixin - if present - into the 'ig' namespace. This gives other
// code the chance to modify 'ig' before it's doing any work.
if( window.ImpactMixin ) {
	ig.merge(ig, window.ImpactMixin);
}

})(window);



// -----------------------------------------------------------------------------
// The main() function creates the system, input, sound and game objects,
// creates a preloader and starts the run loop



// lib/impact/image.js
ig.baked=true;ig.module(
	'impact.image'
).defines(function(){ "use strict";

ig.Image = ig.Class.extend({
	data: null,
	width: 0,
	height: 0,
	loaded: false,
	failed: false,
	loadCallback: null,
	path: '',
	
	
	staticInstantiate: function( path ) {
		return ig.Image.cache[path] || null;
	},
	
	
	init: function( path ) {
		this.path = path;
		this.load();
	},
	
	
	load: function( loadCallback ) {
		if( this.loaded ) {
			if( loadCallback ) {
				loadCallback( this.path, true );
			}
			return;
		}
		else if( !this.loaded && ig.ready ) {
			this.loadCallback = loadCallback || null;
			
			this.data = new Image();
			this.data.onload = this.onload.bind(this);
			this.data.onerror = this.onerror.bind(this);
			this.data.src = ig.prefix + this.path + ig.nocache;
		}
		else {
			ig.addResource( this );
		}
		
		ig.Image.cache[this.path] = this;
	},
	
	
	reload: function() { 
		this.loaded = false;
		this.data = new Image();
		this.data.onload = this.onload.bind(this);
		this.data.src = this.path + '?' + Date.now();
	},
	
	
	onload: function( event ) {
		this.width = this.data.width;
		this.height = this.data.height;
		this.loaded = true;
		
		if( ig.system.scale != 1 ) {
			this.resize( ig.system.scale );
		}
		
		if( this.loadCallback ) {
			this.loadCallback( this.path, true );
		}
	},
	
	
	onerror: function( event ) {
		this.failed = true;
		
		if( this.loadCallback ) {
			this.loadCallback( this.path, false );
		}
	},
	
	
	resize: function( scale ) {
		// Nearest-Neighbor scaling
		
		// The original image is drawn into an offscreen canvas of the same size
		// and copied into another offscreen canvas with the new size. 
		// The scaled offscreen canvas becomes the image (data) of this object.
		
		var origPixels = ig.getImagePixels( this.data, 0, 0, this.width, this.height );
		
		var widthScaled = this.width * scale;
		var heightScaled = this.height * scale;

		var scaled = ig.$new('canvas');
		scaled.width = widthScaled;
		scaled.height = heightScaled;
		var scaledCtx = scaled.getContext('2d');
		var scaledPixels = scaledCtx.getImageData( 0, 0, widthScaled, heightScaled );
			
		for( var y = 0; y < heightScaled; y++ ) {
			for( var x = 0; x < widthScaled; x++ ) {
				var index = (Math.floor(y / scale) * this.width + Math.floor(x / scale)) * 4;
				var indexScaled = (y * widthScaled + x) * 4;
				scaledPixels.data[ indexScaled ] = origPixels.data[ index ];
				scaledPixels.data[ indexScaled+1 ] = origPixels.data[ index+1 ];
				scaledPixels.data[ indexScaled+2 ] = origPixels.data[ index+2 ];
				scaledPixels.data[ indexScaled+3 ] = origPixels.data[ index+3 ];
			}
		}
		scaledCtx.putImageData( scaledPixels, 0, 0 );
		this.data = scaled;
	},
	
	
	draw: function( targetX, targetY, sourceX, sourceY, width, height ) {
		if( !this.loaded ) { return; }
		
		var scale = ig.system.scale;
		sourceX = sourceX ? sourceX * scale : 0;
		sourceY = sourceY ? sourceY * scale : 0;
		width = (width ? width : this.width) * scale;
		height = (height ? height : this.height) * scale;
		
		ig.system.context.drawImage( 
			this.data, sourceX, sourceY, width, height,
			ig.system.getDrawPos(targetX), 
			ig.system.getDrawPos(targetY),
			width, height
		);
		
		ig.Image.drawCount++;
	},
	
	
	drawTile: function( targetX, targetY, tile, tileWidth, tileHeight, flipX, flipY ) {
		tileHeight = tileHeight ? tileHeight : tileWidth;
		
		if( !this.loaded || tileWidth > this.width || tileHeight > this.height ) { return; }
		
		var scale = ig.system.scale;
		var tileWidthScaled = Math.floor(tileWidth * scale);
		var tileHeightScaled = Math.floor(tileHeight * scale);
		
		var scaleX = flipX ? -1 : 1;
		var scaleY = flipY ? -1 : 1;
		
		if( flipX || flipY ) {
			ig.system.context.save();
			ig.system.context.scale( scaleX, scaleY );
		}
		ig.system.context.drawImage( 
			this.data, 
			( Math.floor(tile * tileWidth) % this.width ) * scale,
			( Math.floor(tile * tileWidth / this.width) * tileHeight ) * scale,
			tileWidthScaled,
			tileHeightScaled,
			ig.system.getDrawPos(targetX) * scaleX - (flipX ? tileWidthScaled : 0), 
			ig.system.getDrawPos(targetY) * scaleY - (flipY ? tileHeightScaled : 0),
			tileWidthScaled,
			tileHeightScaled
		);
		if( flipX || flipY ) {
			ig.system.context.restore();
		}
		
		ig.Image.drawCount++;
	}
});

ig.Image.drawCount = 0;
ig.Image.cache = {};
ig.Image.reloadCache = function() {
	for( var path in ig.Image.cache ) {
		ig.Image.cache[path].reload();
	}
};

});

// lib/impact/font.js
ig.baked=true;ig.module(
	'impact.font'
).requires(
	'impact.image'
).defines(function(){ "use strict";


ig.Font = ig.Image.extend({
	widthMap: [],
	indices: [],
	firstChar: 32,
	alpha: 1,
	letterSpacing: 1,
	lineSpacing: 0,
	
	
	onload: function( ev ) {
		this._loadMetrics( this.data );
		this.parent( ev );
	},


	widthForString: function( text ) {
		// Multiline?
		if( text.indexOf('\n') !== -1 ) {
			var lines = text.split( '\n' );
			var width = 0;
			for( var i = 0; i < lines.length; i++ ) {
				width = Math.max( width, this._widthForLine(lines[i]) );
			}
			return width;
		}
		else {
			return this._widthForLine( text );
		}
	},

	
	_widthForLine: function( text ) {
		var width = 0;
		for( var i = 0; i < text.length; i++ ) {
			width += this.widthMap[text.charCodeAt(i) - this.firstChar] + this.letterSpacing;
		}
		return width;
	},


	heightForString: function( text ) {
		return text.split('\n').length * (this.height + this.lineSpacing);
	},
	
	
	draw: function( text, x, y, align ) {
		if( typeof(text) != 'string' ) {
			text = text.toString();
		}
		
		// Multiline?
		if( text.indexOf('\n') !== -1 ) {
			var lines = text.split( '\n' );
			var lineHeight = this.height + this.lineSpacing;
			for( var i = 0; i < lines.length; i++ ) {
				this.draw( lines[i], x, y + i * lineHeight, align );
			}
			return;
		}
		
		if( align == ig.Font.ALIGN.RIGHT || align == ig.Font.ALIGN.CENTER ) {
			var width = this._widthForLine( text );
			x -= align == ig.Font.ALIGN.CENTER ? width/2 : width;
		}
		

		if( this.alpha !== 1 ) {
			ig.system.context.globalAlpha = this.alpha;
		}

		for( var i = 0; i < text.length; i++ ) {
			var c = text.charCodeAt(i);
			x += this._drawChar( c - this.firstChar, x, y );
		}

		if( this.alpha !== 1 ) {
			ig.system.context.globalAlpha = 1;
		}
		ig.Image.drawCount += text.length;
	},
	
	
	_drawChar: function( c, targetX, targetY ) {
		if( !this.loaded || c < 0 || c >= this.indices.length ) { return 0; }
		
		var scale = ig.system.scale;
		
		
		var charX = this.indices[c] * scale;
		var charY = 0;
		var charWidth = this.widthMap[c] * scale;
		var charHeight = (this.height-2) * scale;		
		
		ig.system.context.drawImage( 
			this.data,
			charX, charY,
			charWidth, charHeight,
			ig.system.getDrawPos(targetX), ig.system.getDrawPos(targetY),
			charWidth, charHeight
		);
		
		return this.widthMap[c] + this.letterSpacing;
	},
	
	
	_loadMetrics: function( image ) {
		// Draw the bottommost line of this font image into an offscreen canvas
		// and analyze it pixel by pixel.
		// A run of non-transparent pixels represents a character and its width
		
		this.height = image.height-1;
		this.widthMap = [];
		this.indices = [];
		
		var px = ig.getImagePixels( image, 0, image.height-1, image.width, 1 );
		
		var currentChar = 0;
		var currentWidth = 0;
		for( var x = 0; x < image.width; x++ ) {
			var index = x * 4 + 3; // alpha component of this pixel
			if( px.data[index] > 127 ) {
				currentWidth++;
			}
			else if( px.data[index] < 128 && currentWidth ) {
				this.widthMap.push( currentWidth );
				this.indices.push( x-currentWidth );
				currentChar++;
				currentWidth = 0;
			}
		}
		this.widthMap.push( currentWidth );
		this.indices.push( x-currentWidth );
	}
});


ig.Font.ALIGN = {
	LEFT: 0,
	RIGHT: 1,
	CENTER: 2
};

});


// lib/impact/sound.js
ig.baked=true;ig.module(
	'impact.sound'
).defines(function(){ "use strict";
	
ig.SoundManager = ig.Class.extend({
	clips: {},
	volume: 1,
	format: null,
	
	init: function() {
		// Quick sanity check if the Browser supports the Audio tag
		if( !ig.Sound.enabled || !window.Audio ) {
			ig.Sound.enabled = false;
			return;
		}
		
		// Probe sound formats and determine the file extension to load
		var probe = new Audio();
		for( var i = 0; i < ig.Sound.use.length; i++ ) {
			var format = ig.Sound.use[i];
			if( probe.canPlayType(format.mime) ) {
				this.format = format;
				break;
			}
		}
		
		// No compatible format found? -> Disable sound
		if( !this.format ) {
			ig.Sound.enabled = false;
		}

		// Create WebAudio Context
		if( ig.Sound.enabled && ig.Sound.useWebAudio ) {
			this.audioContext = new AudioContext();
			this.boundWebAudioUnlock = this.unlockWebAudio.bind(this);
			document.addEventListener('touchstart', this.boundWebAudioUnlock, false);

		}
	},
	
	unlockWebAudio: function() {
		document.removeEventListener('touchstart', this.boundWebAudioUnlock, false);
		
		// create empty buffer
		var buffer = this.audioContext.createBuffer(1, 1, 22050);
		var source = this.audioContext.createBufferSource();
		source.buffer = buffer;

		source.connect(this.audioContext.destination);
		source.start(0);
	},

	load: function( path, multiChannel, loadCallback ) {
		if( multiChannel && ig.Sound.useWebAudio ) {
			// Requested as Multichannel and we're using WebAudio?
			return this.loadWebAudio( path, multiChannel, loadCallback );
		}
		else {
			// Oldschool HTML5 Audio - always used for Music
			return this.loadHTML5Audio( path, multiChannel, loadCallback );
		}
	},

	loadWebAudio: function( path, multiChannel, loadCallback ) {
		// Path to the soundfile with the right extension (.ogg or .mp3)
		var realPath = ig.prefix + path.replace(/[^\.]+$/, this.format.ext) + ig.nocache;

		if( this.clips[path] ) {
			return this.clips[path];
		}

		var audioSource = new ig.Sound.WebAudioSource()
		this.clips[path] = audioSource;

		var request = new XMLHttpRequest();
		request.open('GET', realPath, true);
		request.responseType = 'arraybuffer';


		var that = this;
		request.onload = function(ev) {
			that.audioContext.decodeAudioData(request.response, 
				function(buffer) {
					audioSource.buffer = buffer;
					loadCallback( path, true, ev );
				}, 
				function(ev) {
					loadCallback( path, false, ev );
				}
			);
		};
		request.onerror = function(ev) {
			loadCallback( path, false, ev );
		};
		request.send();

		return audioSource;
	},
	
	loadHTML5Audio: function( path, multiChannel, loadCallback ) {
		
		// Path to the soundfile with the right extension (.ogg or .mp3)
		var realPath = ig.prefix + path.replace(/[^\.]+$/, this.format.ext) + ig.nocache;
		
		// Sound file already loaded?
		if( this.clips[path] ) {
			// Loaded as WebAudio, but now requested as HTML5 Audio? Probably Music?
			if( this.clips[path] instanceof ig.Sound.WebAudioSource ) {
				return this.clips[path];
			}
			
			// Only loaded as single channel and now requested as multichannel?
			if( multiChannel && this.clips[path].length < ig.Sound.channels ) {
				for( var i = this.clips[path].length; i < ig.Sound.channels; i++ ) {
					var a = new Audio( realPath );
					a.load();
					this.clips[path].push( a );
				}
			}
			return this.clips[path][0];
		}
		
		var clip = new Audio( realPath );
		if( loadCallback ) {
			
			// The canplaythrough event is dispatched when the browser determines
			// that the sound can be played without interuption, provided the
			// download rate doesn't change.
			// Mobile browsers stubbornly refuse to preload HTML5, so we simply
			// ignore the canplaythrough event and immediately "fake" a successful
			// load callback
			if( ig.ua.mobile ) {
				setTimeout(function(){
					loadCallback( path, true, null );
				}, 0);
			}
			else {
				clip.addEventListener( 'canplaythrough', function cb(ev){
					clip.removeEventListener('canplaythrough', cb, false);
					loadCallback( path, true, ev );
				}, false );

				clip.addEventListener( 'error', function(ev){
					loadCallback( path, false, ev );
				}, false);
			}
		}
		clip.preload = 'auto';
		clip.load();
		
		
		this.clips[path] = [clip];
		if( multiChannel ) {
			for( var i = 1; i < ig.Sound.channels; i++ ) {
				var a = new Audio(realPath);
				a.load();
				this.clips[path].push( a );
			}
		}
		
		return clip;
	},
	
	
	get: function( path ) {
		// Find and return a channel that is not currently playing	
		var channels = this.clips[path];

		// Is this a WebAudio source? We only ever have one for each Sound
		if( channels && channels instanceof ig.Sound.WebAudioSource ) {
			return channels;
		}

		// Oldschool HTML5 Audio - find a channel that's not currently 
		// playing or, if all are playing, rewind one
		for( var i = 0, clip; clip = channels[i++]; ) {
			if( clip.paused || clip.ended ) {
				if( clip.ended ) {
					clip.currentTime = 0;
				}
				return clip;
			}
		}
		
		// Still here? Pause and rewind the first channel
		channels[0].pause();
		channels[0].currentTime = 0;
		return channels[0];
	}
});



ig.Music = ig.Class.extend({
	tracks: [],
	namedTracks: {},
	currentTrack: null,
	currentIndex: 0,
	random: false,
	
	_volume: 1,
	_loop: false,
	_fadeInterval: 0,
	_fadeTimer: null,
	_endedCallbackBound: null,
	
	
	init: function() {
		this._endedCallbackBound = this._endedCallback.bind(this);
		
		Object.defineProperty(this,"volume", { 
			get: this.getVolume.bind(this),
			set: this.setVolume.bind(this)
		});
		
		Object.defineProperty(this,"loop", { 
			get: this.getLooping.bind(this),
			set: this.setLooping.bind(this)
		});
	},
	
	
	add: function( music, name ) {
		if( !ig.Sound.enabled ) {
			return;
		}
		
		var path = music instanceof ig.Sound ? music.path : music;
		
		var track = ig.soundManager.load(path, false);

		// Did we get a WebAudio Source? This is suboptimal; Music should be loaded
		// as HTML5 Audio so it can be streamed
		if( track instanceof ig.Sound.WebAudioSource ) {
			// Since this error will likely occour at game start, we stop the game
			// to not produce any more errors.
			ig.system.stopRunLoop();
			throw(
				"Sound '"+path+"' loaded as Multichannel but used for Music. " +
				"Set the multiChannel param to false when loading, e.g.: new ig.Sound(path, false)"
			);
		}

		track.loop = this._loop;
		track.volume = this._volume;
		track.addEventListener( 'ended', this._endedCallbackBound, false );
		this.tracks.push( track );
		
		if( name ) {
			this.namedTracks[name] = track;
		}
		
		if( !this.currentTrack ) {
			this.currentTrack = track;
		}
	},
	
	
	next: function() {
		if( !this.tracks.length ) { return; }
		
		this.stop();
		this.currentIndex = this.random
			? Math.floor(Math.random() * this.tracks.length)
			: (this.currentIndex + 1) % this.tracks.length;
		this.currentTrack = this.tracks[this.currentIndex];
		this.play();
	},
	
	
	pause: function() {
		if( !this.currentTrack ) { return; }
		this.currentTrack.pause();
	},
	
	
	stop: function() {
		if( !this.currentTrack ) { return; }
		this.currentTrack.pause();
		this.currentTrack.currentTime = 0;
	},
	
	
	play: function( name ) {
		// If a name was provided, stop playing the current track (if any)
		// and play the named track
		if( name && this.namedTracks[name] ) {
			var newTrack = this.namedTracks[name];
			if( newTrack != this.currentTrack ) {
				this.stop();
				this.currentTrack = newTrack;
			}
		}
		else if( !this.currentTrack ) { 
			return; 
		}
		this.currentTrack.play();
	},
	
		
	getLooping: function() {
		return this._loop;
	},
	
	
	setLooping: function( l ) {
		this._loop = l;
		for( var i in this.tracks ) {
			this.tracks[i].loop = l;
		}
	},	
		
	
	getVolume: function() {
		return this._volume;
	},
	
	
	setVolume: function( v ) {
		this._volume = v.limit(0,1);
		for( var i in this.tracks ) {
			this.tracks[i].volume = this._volume;
		}
	},
	
	
	fadeOut: function( time ) {
		if( !this.currentTrack ) { return; }
		
		clearInterval( this._fadeInterval );
		this.fadeTimer = new ig.Timer( time );
		this._fadeInterval = setInterval( this._fadeStep.bind(this), 50 );
	},
	
	
	_fadeStep: function() {
		var v = this.fadeTimer.delta()
			.map(-this.fadeTimer.target, 0, 1, 0)
			.limit( 0, 1 )
			* this._volume;
		
		if( v <= 0.01 ) {
			this.stop();
			this.currentTrack.volume = this._volume;
			clearInterval( this._fadeInterval );
		}
		else {
			this.currentTrack.volume = v;
		}
	},
	
	_endedCallback: function() {
		if( this._loop ) {
			this.play();
		}
		else {
			this.next();
		}
	}
});



ig.Sound = ig.Class.extend({
	path: '',
	volume: 1,
	currentClip: null,
	multiChannel: true,
	_loop: false,
	
	
	init: function( path, multiChannel ) {
		this.path = path;
		this.multiChannel = (multiChannel !== false);

		Object.defineProperty(this,"loop", { 
			get: this.getLooping.bind(this),
			set: this.setLooping.bind(this)
		});
		
		this.load();
	},

	getLooping: function() {
		return this._loop;
	},

	setLooping: function( loop ) {
		this._loop = loop;

		if( this.currentClip ) {
			this.currentClip.loop = loop;
		}
	},	
	
	load: function( loadCallback ) {
		if( !ig.Sound.enabled ) {
			if( loadCallback ) {
				loadCallback( this.path, true );
			}
			return;
		}
		
		if( ig.ready ) {
			ig.soundManager.load( this.path, this.multiChannel, loadCallback );
		}
		else {
			ig.addResource( this );
		}
	},
	
	
	play: function() {
		if( !ig.Sound.enabled ) {
			return;
		}
		
		this.currentClip = ig.soundManager.get( this.path );
		this.currentClip.loop = this._loop;
		this.currentClip.volume = ig.soundManager.volume * this.volume;
		this.currentClip.play();
	},
	
	
	stop: function() {
		if( this.currentClip ) {
			this.currentClip.pause();
			this.currentClip.currentTime = 0;
		}
	}
});


ig.Sound.WebAudioSource = ig.Class.extend({
	sources: [],
	gain: null,
	buffer: null,
	_loop: false,

	init: function() {
		this.gain = ig.soundManager.audioContext.createGain();
		this.gain.connect(ig.soundManager.audioContext.destination);

		Object.defineProperty(this,"loop", { 
			get: this.getLooping.bind(this),
			set: this.setLooping.bind(this)
		});

		Object.defineProperty(this,"volume", { 
			get: this.getVolume.bind(this),
			set: this.setVolume.bind(this)
		});
	},

	play: function() {
		if( !this.buffer ) { return; }
		var source = ig.soundManager.audioContext.createBufferSource();
		source.buffer = this.buffer;
		source.connect(this.gain); 
		source.loop = this._loop;

		// Add this new source to our sources array and remove it again
		// later when it has finished playing.
		var that = this;
		this.sources.push(source);
		source.onended = function(){ that.sources.erase(source); }

		source.start(0);
	},

	pause: function() {
		for( var i = 0; i < this.sources.length; i++ ) {
			try{
				this.sources[i].stop();
			} catch(err){}
		}
	},

	getLooping: function() {
		return this._loop;
	},

	setLooping: function( loop ) {
		this._loop = loop;

		for( var i = 0; i < this.sources.length; i++ ) {
			this.sources[i].loop = loop;
		}
	},

	getVolume: function() {
		return this.gain.gain.value;
	},

	setVolume: function( volume ) {
		this.gain.gain.value = volume;
	}
});


ig.Sound.FORMAT = {
	MP3: {ext: 'mp3', mime: 'audio/mpeg'},
	M4A: {ext: 'm4a', mime: 'audio/mp4; codecs=mp4a'},
	OGG: {ext: 'ogg', mime: 'audio/ogg; codecs=vorbis'},
	WEBM: {ext: 'webm', mime: 'audio/webm; codecs=vorbis'},
	CAF: {ext: 'caf', mime: 'audio/x-caf'}
};
ig.Sound.use = [ig.Sound.FORMAT.OGG, ig.Sound.FORMAT.MP3];
ig.Sound.channels = 4;
ig.Sound.enabled = true;

ig.normalizeVendorAttribute(window, 'AudioContext');
ig.Sound.useWebAudio = !!window.AudioContext;

});

// lib/impact/loader.js
ig.baked=true;ig.module(
	'impact.loader'
).requires(
	'impact.image',
	'impact.font',
	'impact.sound'
).defines(function(){ "use strict";

ig.Loader = ig.Class.extend({
	resources: [],
	
	gameClass: null,
	status: 0,
	done: false,
	
	_unloaded: [],
	_drawStatus: 0,
	_intervalId: 0,
	_loadCallbackBound: null,
	
	
	init: function( gameClass, resources ) {
		this.gameClass = gameClass;
		this.resources = resources;
		this._loadCallbackBound = this._loadCallback.bind(this);
		
		for( var i = 0; i < this.resources.length; i++ ) {
			this._unloaded.push( this.resources[i].path );
		}
	},
	
	
	load: function() {
		ig.system.clear( '#000' );
		
		if( !this.resources.length ) {
			this.end();
			return;
		}

		for( var i = 0; i < this.resources.length; i++ ) {
			this.loadResource( this.resources[i] );
		}
		this._intervalId = setInterval( this.draw.bind(this), 16 );
	},
	
	
	loadResource: function( res ) {
		res.load( this._loadCallbackBound );
	},
	
	
	end: function() {
		if( this.done ) { return; }
		
		this.done = true;
		clearInterval( this._intervalId );
		ig.system.setGame( this.gameClass );
	},
	
	
	draw: function() {
		this._drawStatus += (this.status - this._drawStatus)/5;
		var s = ig.system.scale;
		var w = ig.system.width * 0.6;
		var h = ig.system.height * 0.1;
		var x = ig.system.width * 0.5-w/2;
		var y = ig.system.height * 0.5-h/2;
		
		ig.system.context.fillStyle = '#000';
		ig.system.context.fillRect( 0, 0, 480, 320 );
		
		ig.system.context.fillStyle = '#fff';
		ig.system.context.fillRect( x*s, y*s, w*s, h*s );
		
		ig.system.context.fillStyle = '#000';
		ig.system.context.fillRect( x*s+s, y*s+s, w*s-s-s, h*s-s-s );
		
		ig.system.context.fillStyle = '#fff';
		ig.system.context.fillRect( x*s, y*s, w*s*this._drawStatus, h*s );
	},
	
	
	_loadCallback: function( path, status ) {
		if( status ) {
			this._unloaded.erase( path );
		}
		else {
			throw( 'Failed to load resource: ' + path );
		}
		
		this.status = 1 - (this._unloaded.length / this.resources.length);
		if( this._unloaded.length == 0 ) { // all done?
			setTimeout( this.end.bind(this), 250 );
		}
	}
});

});

// lib/impact/timer.js
ig.baked=true;ig.module(
	'impact.timer'
).defines(function(){ "use strict";

ig.Timer = ig.Class.extend({
	target: 0,
	base: 0,
	last: 0,
	pausedAt: 0,
	
	init: function( seconds ) {
		this.base = ig.Timer.time;
		this.last = ig.Timer.time;
		
		this.target = seconds || 0;
	},
	
	
	set: function( seconds ) {
		this.target = seconds || 0;
		this.base = ig.Timer.time;
		this.pausedAt = 0;
	},
	
	
	reset: function() {
		this.base = ig.Timer.time;
		this.pausedAt = 0;
	},
	
	
	tick: function() {
		var delta = ig.Timer.time - this.last;
		this.last = ig.Timer.time;
		return (this.pausedAt ? 0 : delta);
	},
	
	
	delta: function() {
		return (this.pausedAt || ig.Timer.time) - this.base - this.target;
	},


	pause: function() {
		if( !this.pausedAt ) {
			this.pausedAt = ig.Timer.time;
		}
	},


	unpause: function() {
		if( this.pausedAt ) {
			this.base += ig.Timer.time - this.pausedAt;
			this.pausedAt = 0;
		}
	}
});

ig.Timer._last = 0;
ig.Timer.time = Number.MIN_VALUE;
ig.Timer.timeScale = 1;
ig.Timer.maxStep = 0.05;

ig.Timer.step = function() {
	var current = Date.now();
	var delta = (current - ig.Timer._last) / 1000;
	ig.Timer.time += Math.min(delta, ig.Timer.maxStep) * ig.Timer.timeScale;
	ig.Timer._last = current;
};

});

// lib/impact/system.js
ig.baked=true;ig.module(
	'impact.system'
).requires(
	'impact.timer',
	'impact.image'
).defines(function(){ "use strict";

ig.System = ig.Class.extend({
	fps: 30,
	width: 320,
	height: 240,
	realWidth: 320,
	realHeight: 240,
	scale: 1,
	
	tick: 0,
	animationId: 0,
	newGameClass: null,
	running: false,
	
	delegate: null,
	clock: null,
	canvas: null,
	context: null,
	
	init: function( canvasId, fps, width, height, scale ) {
		this.fps = fps;
		
		this.clock = new ig.Timer();
		this.canvas = ig.$(canvasId);
		this.resize( width, height, scale );
		this.context = this.canvas.getContext('2d');
		
		this.getDrawPos = ig.System.drawMode;

		// Automatically switch to crisp scaling when using a scale
		// other than 1
		if( this.scale != 1 ) {
			ig.System.scaleMode = ig.System.SCALE.CRISP;
		}
		ig.System.scaleMode( this.canvas, this.context );
	},
	
	
	resize: function( width, height, scale ) {
		this.width = width;
		this.height = height;
		this.scale = scale || this.scale;
		
		this.realWidth = this.width * this.scale;
		this.realHeight = this.height * this.scale;
		this.canvas.width = this.realWidth;
		this.canvas.height = this.realHeight;
	},
	
	
	setGame: function( gameClass ) {
		if( this.running ) {
			this.newGameClass = gameClass;
		}
		else {
			this.setGameNow( gameClass );
		}
	},
	
	
	setGameNow: function( gameClass ) {
		ig.game = new (gameClass)();	
		ig.system.setDelegate( ig.game );
	},
	
	
	setDelegate: function( object ) {
		if( typeof(object.run) == 'function' ) {
			this.delegate = object;
			this.startRunLoop();
		} else {
			throw( 'System.setDelegate: No run() function in object' );
		}
	},
	
	
	stopRunLoop: function() {
		ig.clearAnimation( this.animationId );
		this.running = false;
	},
	
	
	startRunLoop: function() {
		this.stopRunLoop();
		this.animationId = ig.setAnimation( this.run.bind(this), this.canvas );
		this.running = true;
	},
	
	
	clear: function( color ) {
		if(color) {
			this.context.fillStyle = color;
			this.context.fillRect( 0, 0, this.realWidth, this.realHeight );
		} else {
			this.context.clearRect( 0, 0, this.realWidth, this.realHeight );
		}
	},
	
	
	run: function() {
		ig.Timer.step();
		this.tick = this.clock.tick();
		
		this.delegate.run();
		ig.input.clearPressed();
		
		if( this.newGameClass ) {
			this.setGameNow( this.newGameClass );
			this.newGameClass = null;
		}
	},
	
	
	getDrawPos: null // Set through constructor
});

ig.System.DRAW = {
	AUTHENTIC: function( p ) { return Math.round(p) * this.scale; },
	SMOOTH: function( p ) { return Math.round(p * this.scale); },
	SUBPIXEL: function( p ) { return p * this.scale; }
};
ig.System.drawMode = ig.System.DRAW.SUBPIXEL;

ig.System.SCALE = {
	CRISP: function( canvas, context ) {
		ig.setVendorAttribute( context, 'imageSmoothingEnabled', false );
		canvas.style.imageRendering = '-moz-crisp-edges';
		canvas.style.imageRendering = '-o-crisp-edges';
		canvas.style.imageRendering = '-webkit-optimize-contrast';
		canvas.style.imageRendering = 'crisp-edges';
		canvas.style.msInterpolationMode = 'nearest-neighbor'; // No effect on Canvas :/
	},
	SMOOTH: function( canvas, context ) {
		ig.setVendorAttribute( context, 'imageSmoothingEnabled', true );
		canvas.style.imageRendering = '';
		canvas.style.msInterpolationMode = '';
	}
};
ig.System.scaleMode = ig.System.SCALE.SMOOTH;

});


// lib/impact/input.js
ig.baked=true;ig.module(
	'impact.input'
).defines(function(){ "use strict";
	
ig.KEY = {
	'MOUSE1': -1,
	'MOUSE2': -3,
	'MWHEEL_UP': -4,
	'MWHEEL_DOWN': -5,
	
	'BACKSPACE': 8,
	'TAB': 9,
	'ENTER': 13,
	'PAUSE': 19,
	'CAPS': 20,
	'ESC': 27,
	'SPACE': 32,
	'PAGE_UP': 33,
	'PAGE_DOWN': 34,
	'END': 35,
	'HOME': 36,
	'LEFT_ARROW': 37,
	'UP_ARROW': 38,
	'RIGHT_ARROW': 39,
	'DOWN_ARROW': 40,
	'INSERT': 45,
	'DELETE': 46,
	'_0': 48,
	'_1': 49,
	'_2': 50,
	'_3': 51,
	'_4': 52,
	'_5': 53,
	'_6': 54,
	'_7': 55,
	'_8': 56,
	'_9': 57,
	'A': 65,
	'B': 66,
	'C': 67,
	'D': 68,
	'E': 69,
	'F': 70,
	'G': 71,
	'H': 72,
	'I': 73,
	'J': 74,
	'K': 75,
	'L': 76,
	'M': 77,
	'N': 78,
	'O': 79,
	'P': 80,
	'Q': 81,
	'R': 82,
	'S': 83,
	'T': 84,
	'U': 85,
	'V': 86,
	'W': 87,
	'X': 88,
	'Y': 89,
	'Z': 90,
	'NUMPAD_0': 96,
	'NUMPAD_1': 97,
	'NUMPAD_2': 98,
	'NUMPAD_3': 99,
	'NUMPAD_4': 100,
	'NUMPAD_5': 101,
	'NUMPAD_6': 102,
	'NUMPAD_7': 103,
	'NUMPAD_8': 104,
	'NUMPAD_9': 105,
	'MULTIPLY': 106,
	'ADD': 107,
	'SUBSTRACT': 109,
	'DECIMAL': 110,
	'DIVIDE': 111,
	'F1': 112,
	'F2': 113,
	'F3': 114,
	'F4': 115,
	'F5': 116,
	'F6': 117,
	'F7': 118,
	'F8': 119,
	'F9': 120,
	'F10': 121,
	'F11': 122,
	'F12': 123,
	'SHIFT': 16,
	'CTRL': 17,
	'ALT': 18,
	'PLUS': 187,
	'COMMA': 188,
	'MINUS': 189,
	'PERIOD': 190
};


ig.Input = ig.Class.extend({
	bindings: {},
	actions: {},
	presses: {},
	locks: {},
	delayedKeyup: {},
	
	isUsingMouse: false,
	isUsingKeyboard: false,
	isUsingAccelerometer: false,
	mouse: {x: 0, y: 0},
	accel: {x: 0, y: 0, z: 0},
	
	
	initMouse: function() {
		if( this.isUsingMouse ) { return; }
		this.isUsingMouse = true;
		var mouseWheelBound = this.mousewheel.bind(this);
		ig.system.canvas.addEventListener('mousewheel', mouseWheelBound, false );
		ig.system.canvas.addEventListener('DOMMouseScroll', mouseWheelBound, false );
		
		ig.system.canvas.addEventListener('contextmenu', this.contextmenu.bind(this), false );
		ig.system.canvas.addEventListener('mousedown', this.keydown.bind(this), false );
		ig.system.canvas.addEventListener('mouseup', this.keyup.bind(this), false );
		ig.system.canvas.addEventListener('mousemove', this.mousemove.bind(this), false );
		
		// Change to Impact lib: Removing this condition as it seems to break in some versions of IE.
		// TODO: It may be possible to fix this by re-visiting the conditions of ig.ua.touchDevice.
		//if( ig.ua.touchDevice ) {
		// Standard
		ig.system.canvas.addEventListener('touchstart', this.keydown.bind(this), false );
		ig.system.canvas.addEventListener('touchend', this.keyup.bind(this), false );
		ig.system.canvas.addEventListener('touchmove', this.mousemove.bind(this), false );
		
		// MS
		ig.system.canvas.addEventListener('MSPointerDown', this.keydown.bind(this), false );
		ig.system.canvas.addEventListener('MSPointerUp', this.keyup.bind(this), false );
		ig.system.canvas.addEventListener('MSPointerMove', this.mousemove.bind(this), false );
		ig.system.canvas.style.msTouchAction = 'none';
		//}
	},

	
	initKeyboard: function() {
		if( this.isUsingKeyboard ) { return; }
		this.isUsingKeyboard = true;
		window.addEventListener('keydown', this.keydown.bind(this), false );
		window.addEventListener('keyup', this.keyup.bind(this), false );
	},
	
	
	initAccelerometer: function() {
		if( this.isUsingAccelerometer ) { return; }
		this.isUsingAccelerometer = true;
		window.addEventListener('devicemotion', this.devicemotion.bind(this), false );
	},

	
	mousewheel: function( event ) {
		var delta = event.wheelDelta ? event.wheelDelta : (event.detail * -1);
		var code = delta > 0 ? ig.KEY.MWHEEL_UP : ig.KEY.MWHEEL_DOWN;
		var action = this.bindings[code];
		if( action ) {
			this.actions[action] = true;
			this.presses[action] = true;
			this.delayedKeyup[action] = true;
			event.stopPropagation();
			event.preventDefault();
		}
	},
	
	
	mousemove: function( event ) {		
		var internalWidth = parseInt(ig.system.canvas.offsetWidth) || ig.system.realWidth;
		var scale = ig.system.scale * (internalWidth / ig.system.realWidth);
		
		var pos = {left: 0, top: 0};
		if( ig.system.canvas.getBoundingClientRect ) {
			pos = ig.system.canvas.getBoundingClientRect();
		}
		
		var ev = event.touches ? event.touches[0] : event;
		this.mouse.x = (ev.clientX - pos.left) / scale;
		this.mouse.y = (ev.clientY - pos.top) / scale;
	},
	
	
	contextmenu: function( event ) {
		if( this.bindings[ig.KEY.MOUSE2] ) {
			event.stopPropagation();
			event.preventDefault();
		}
	},
	
	
	keydown: function( event ) {
		var tag = event.target.tagName;
		if( tag == 'INPUT' || tag == 'TEXTAREA' ) { return; }
		
		var code = event.type == 'keydown' 
			? event.keyCode 
			: (event.button == 2 ? ig.KEY.MOUSE2 : ig.KEY.MOUSE1);
		
		// Focus window element for mouse clicks. Prevents issues when
		// running the game in an iframe.
		if( code < 0 && !ig.ua.mobile ) {
			window.focus();
		}
		
		if( event.type == 'touchstart' || event.type == 'mousedown' ) {
			this.mousemove( event );
		}
			
		var action = this.bindings[code];
		if( action ) {
			this.actions[action] = true;
			if( !this.locks[action] ) {
				this.presses[action] = true;
				this.locks[action] = true;
			}
			event.preventDefault();
		}
	},
	
	
	keyup: function( event ) {
		var tag = event.target.tagName;
		if( tag == 'INPUT' || tag == 'TEXTAREA' ) { return; }
		
		var code = event.type == 'keyup' 
			? event.keyCode 
			: (event.button == 2 ? ig.KEY.MOUSE2 : ig.KEY.MOUSE1);
		
		var action = this.bindings[code];
		if( action ) {
			this.delayedKeyup[action] = true;
			event.preventDefault();
		}
	},
	
	
	devicemotion: function( event ) {
		this.accel = event.accelerationIncludingGravity;
	},
	
	
	bind: function( key, action ) {
		if( key < 0 ) { this.initMouse(); }
		else if( key > 0 ) { this.initKeyboard(); }
		this.bindings[key] = action;
	},
	
	
	bindTouch: function( selector, action ) {
		var element = ig.$( selector );
		
		var that = this;
		element.addEventListener('touchstart', function(ev) {that.touchStart( ev, action );}, false);
		element.addEventListener('touchend', function(ev) {that.touchEnd( ev, action );}, false);
		element.addEventListener('MSPointerDown', function(ev) {that.touchStart( ev, action );}, false);
		element.addEventListener('MSPointerUp', function(ev) {that.touchEnd( ev, action );}, false);
	},
	
	
	unbind: function( key ) {
		var action = this.bindings[key];
		this.delayedKeyup[action] = true;
		
		this.bindings[key] = null;
	},
	
	
	unbindAll: function() {
		this.bindings = {};
		this.actions = {};
		this.presses = {};
		this.locks = {};
		this.delayedKeyup = {};
	},
	
	
	state: function( action ) {
		return this.actions[action];
	},
	
	
	pressed: function( action ) {
		return this.presses[action];
	},
	
	released: function( action ) {
		return !!this.delayedKeyup[action];
	},
		
	clearPressed: function() {
		for( var action in this.delayedKeyup ) {
			this.actions[action] = false;
			this.locks[action] = false;
		}
		this.delayedKeyup = {};
		this.presses = {};
	},
	
	touchStart: function( event, action ) {
		this.actions[action] = true;
		this.presses[action] = true;
		
		event.stopPropagation();
		event.preventDefault();
		return false;
	},
	
	
	touchEnd: function( event, action ) {
		this.delayedKeyup[action] = true;
		event.stopPropagation();
		event.preventDefault();
		return false;
	}
});

});

// lib/impact/impact.js
ig.baked=true;ig.module(
	'impact.impact'
).requires(
	'dom.ready',
	'impact.loader',
	'impact.system',
	'impact.input',
	'impact.sound'
).defines(function(){ "use strict";

ig.main = function( canvasId, gameClass, fps, width, height, scale, loaderClass ) {
	ig.system = new ig.System( canvasId, fps, width, height, scale || 1 );
	ig.input = new ig.Input();
	ig.soundManager = new ig.SoundManager();
	ig.music = new ig.Music();
	ig.ready = true;
	
	var loader = new (loaderClass || ig.Loader)( gameClass, ig.resources );
	loader.load();
};

});

// Versioning of the gamewheel library uses Semantic Versioning - http://semver.org/

// Override ig.global to make sure we don't get naming conflicts.
// We do this here to make sure it happens before any script definitions are run.
ig.global = {};




// lib/impact/animation.js
ig.baked=true;ig.module(
	'impact.animation'
).requires(
	'impact.timer',
	'impact.image' 
).defines(function(){ "use strict";

ig.AnimationSheet = ig.Class.extend({
	width: 8,
	height: 8,
	image: null,
	
	init: function( path, width, height ) {
		this.width = width;
		this.height = height;
		
		this.image = new ig.Image( path );
	}
});



ig.Animation = ig.Class.extend({
	sheet: null,
	timer: null,
	
	sequence: [],	
	flip: {x: false, y: false},
	pivot: {x: 0, y: 0},
	
	frame: 0,
	tile: 0,
	loopCount: 0,
	alpha: 1,
	angle: 0,
	
	
	init: function( sheet, frameTime, sequence, stop ) {
		this.sheet = sheet;
		this.pivot = {x: sheet.width/2, y: sheet.height/2 };
		this.timer = new ig.Timer();

		this.frameTime = frameTime;
		this.sequence = sequence;
		this.stop = !!stop;
		this.tile = this.sequence[0];
	},
	
	
	rewind: function() {
		this.timer.set();
		this.loopCount = 0;
		this.frame = 0;
		this.tile = this.sequence[0];
		return this;
	},
	
	
	gotoFrame: function( f ) {
		// Offset the timer by one tenth of a millisecond to make sure we
		// jump to the correct frame and circumvent rounding errors
		this.timer.set( this.frameTime * -f - 0.0001 );
		this.update();
	},
	
	
	gotoRandomFrame: function() {
		this.gotoFrame( Math.floor(Math.random() * this.sequence.length) )
	},
	
	
	update: function() {
		var frameTotal = Math.floor(this.timer.delta() / this.frameTime);
		this.loopCount = Math.floor(frameTotal / this.sequence.length);
		if( this.stop && this.loopCount > 0 ) {
			this.frame = this.sequence.length - 1;
		}
		else {
			this.frame = frameTotal % this.sequence.length;
		}
		this.tile = this.sequence[ this.frame ];
	},
	
	
	draw: function( targetX, targetY ) {
		var bbsize = Math.max(this.sheet.width, this.sheet.height);
		
		// On screen?
		if(
		   targetX > ig.system.width || targetY > ig.system.height ||
		   targetX + bbsize < 0 || targetY + bbsize < 0
		) {
			return;
		}
		
		if( this.alpha != 1) {
			ig.system.context.globalAlpha = this.alpha;
		}
		
		if( this.angle == 0 ) {		
			this.sheet.image.drawTile(
				targetX, targetY,
				this.tile, this.sheet.width, this.sheet.height,
				this.flip.x, this.flip.y
			);
		}
		else {
			ig.system.context.save();
			ig.system.context.translate(
				ig.system.getDrawPos(targetX + this.pivot.x),
				ig.system.getDrawPos(targetY + this.pivot.y)
			);
			ig.system.context.rotate( this.angle );
			this.sheet.image.drawTile(
				-this.pivot.x, -this.pivot.y,
				this.tile, this.sheet.width, this.sheet.height,
				this.flip.x, this.flip.y
			);
			ig.system.context.restore();
		}
		
		if( this.alpha != 1) {
			ig.system.context.globalAlpha = 1;
		}
	}
});

});

// lib/impact/entity.js
ig.baked=true;ig.module(
	'impact.entity'
).requires(
	'impact.animation',
	'impact.impact'
).defines(function(){ "use strict";

ig.Entity = ig.Class.extend({
	id: 0,
	settings: {},
	
	size: {x: 16, y:16},
	offset: {x: 0, y: 0},
	
	pos: {x: 0, y:0},
	last: {x: 0, y:0},
	vel: {x: 0, y: 0},
	accel: {x: 0, y: 0},
	friction: {x: 0, y: 0},
	maxVel: {x: 100, y: 100},
	zIndex: 0,
	gravityFactor: 1,
	standing: false,
	bounciness: 0,
	minBounceVelocity: 40,
	
	anims: {},
	animSheet: null,
	currentAnim: null,
	health: 10,
	
	type: 0, // TYPE.NONE
	checkAgainst: 0, // TYPE.NONE
	collides: 0, // COLLIDES.NEVER
	
	_killed: false,
	
	slopeStanding: {min: (44).toRad(), max: (136).toRad() },
	
	init: function( x, y, settings ) {
		this.id = ++ig.Entity._lastId;
		this.pos.x = this.last.x = x;
		this.pos.y = this.last.y = y;
		
		ig.merge( this, settings );

		if(this.anchor) {
			ig.util.align(this, this.anchor);
			this.last.x = this.pos.x;
			this.last.y = this.pos.y;
		}
	},
	
	reset: function( x, y, settings ) {
		var proto = this.constructor.prototype;
		this.pos.x = x;
		this.pos.y = y;
		this.last.x = x;
		this.last.y = y;
		this.vel.x = proto.vel.x;
		this.vel.y = proto.vel.y;
		this.accel.x = proto.accel.x;
		this.accel.y = proto.accel.y;
		this.health = proto.health;
		this._killed = proto._killed;
		this.standing = proto.standing;
		
		this.type = proto.type;
		this.checkAgainst = proto.checkAgainst;
		this.collides = proto.collides;
		
		ig.merge( this, settings );
	},
	
	addAnim: function( name, frameTime, sequence, stop ) {
		if( !this.animSheet ) {
			throw( 'No animSheet to add the animation '+name+' to.' );
		}
		var a = new ig.Animation( this.animSheet, frameTime, sequence, stop );
		this.anims[name] = a;
		if( !this.currentAnim ) {
			this.currentAnim = a;
		}
		
		return a;
	},
	
	update: function() {
		this.last.x = this.pos.x;
		this.last.y = this.pos.y;
		this.vel.y += ig.game.gravity * ig.system.tick * this.gravityFactor;
		
		this.vel.x = this.getNewVelocity( this.vel.x, this.accel.x, this.friction.x, this.maxVel.x );
		this.vel.y = this.getNewVelocity( this.vel.y, this.accel.y, this.friction.y, this.maxVel.y );
		
		// movement & collision
		var mx = this.vel.x * ig.system.tick;
		var my = this.vel.y * ig.system.tick;
		var res = ig.game.collisionMap.trace( 
			this.pos.x, this.pos.y, mx, my, this.size.x, this.size.y
		);
		this.handleMovementTrace( res );
		
		if( this.currentAnim ) {
			this.currentAnim.update();
		}
	},
	
	
	getNewVelocity: function( vel, accel, friction, max ) {
		if( accel ) {
			return ( vel + accel * ig.system.tick ).limit( -max, max );
		}
		else if( friction ) {
			var delta = friction * ig.system.tick;
			
			if( vel - delta > 0) {
				return vel - delta;
			} 
			else if( vel + delta < 0 ) {
				return vel + delta;
			}
			else {
				return 0;
			}
		}
		return vel.limit( -max, max );
	},
	
	
	handleMovementTrace: function( res ) {
		this.standing = false;
		
		if( res.collision.y ) {
			if( this.bounciness > 0 && Math.abs(this.vel.y) > this.minBounceVelocity ) {
				this.vel.y *= -this.bounciness;				
			}
			else {
				if( this.vel.y > 0 ) {
					this.standing = true;
				}
				this.vel.y = 0;
			}
		}
		if( res.collision.x ) {
			if( this.bounciness > 0 && Math.abs(this.vel.x) > this.minBounceVelocity ) {
				this.vel.x *= -this.bounciness;				
			}
			else {
				this.vel.x = 0;
			}
		}
		if( res.collision.slope ) {
			var s = res.collision.slope;
			
			if( this.bounciness > 0 ) {
				var proj = this.vel.x * s.nx + this.vel.y * s.ny;
				
				this.vel.x = (this.vel.x - s.nx * proj * 2) * this.bounciness;
				this.vel.y = (this.vel.y - s.ny * proj * 2) * this.bounciness;
			}
			else {
				var lengthSquared = s.x * s.x + s.y * s.y;
				var dot = (this.vel.x * s.x + this.vel.y * s.y)/lengthSquared;
				
				this.vel.x = s.x * dot;
				this.vel.y = s.y * dot;
				
				var angle = Math.atan2( s.x, s.y );
				if( angle > this.slopeStanding.min && angle < this.slopeStanding.max ) {
					this.standing = true;
				}
			}
		}
		
		this.pos = res.pos;
	},
	
	
	draw: function() {
		if( this.currentAnim ) {
			this.currentAnim.draw(
				this.pos.x - this.offset.x - ig.game._rscreen.x,
				this.pos.y - this.offset.y - ig.game._rscreen.y
			);
		}
	},
	
	
	kill: function() {
		ig.game.removeEntity( this );
	},
	
	
	receiveDamage: function( amount, from ) {
		this.health -= amount;
		if( this.health <= 0 ) {
			this.kill();
		}
	},
	
	
	touches: function( other ) {		
		return !(
			this.pos.x >= other.pos.x + other.size.x ||
			this.pos.x + this.size.x <= other.pos.x ||
			this.pos.y >= other.pos.y + other.size.y ||
			this.pos.y + this.size.y <= other.pos.y
		);
	},
	
	
	distanceTo: function( other ) {
		var xd = (this.pos.x + this.size.x/2) - (other.pos.x + other.size.x/2); 
		var yd = (this.pos.y + this.size.y/2) - (other.pos.y + other.size.y/2);
		return Math.sqrt( xd*xd + yd*yd );
	},
	
	
	angleTo: function( other ) {
		return Math.atan2(
			(other.pos.y + other.size.y/2) - (this.pos.y + this.size.y/2),
			(other.pos.x + other.size.x/2) - (this.pos.x + this.size.x/2)
		);
	},
	
	
	check: function( other ) {},
	collideWith: function( other, axis ) {},
	ready: function() {},
	erase: function() {}
});


// Last used entity id; incremented with each spawned entity

ig.Entity._lastId = 0;


// Collision Types - Determine if and how entities collide with each other

// In ACTIVE vs. LITE or FIXED vs. ANY collisions, only the "weak" entity moves,
// while the other one stays fixed. In ACTIVE vs. ACTIVE and ACTIVE vs. PASSIVE
// collisions, both entities are moved. LITE or PASSIVE entities don't collide
// with other LITE or PASSIVE entities at all. The behaiviour for FIXED vs.
// FIXED collisions is undefined.

ig.Entity.COLLIDES = {
	NEVER: 0,
	LITE: 1,
	PASSIVE: 2,
	ACTIVE: 4,
	FIXED: 8
};


// Entity Types - used for checks

ig.Entity.TYPE = {
	NONE: 0,
	A: 1,
	B: 2,
	BOTH: 3
};



ig.Entity.checkPair = function( a, b ) {
	
	// Do these entities want checks?
	if( a.checkAgainst & b.type ) {
		a.check( b );
	}
	
	if( b.checkAgainst & a.type ) {
		b.check( a );
	}
	
	// If this pair allows collision, solve it! At least one entity must
	// collide ACTIVE or FIXED, while the other one must not collide NEVER.
	if(
		a.collides && b.collides &&
		a.collides + b.collides > ig.Entity.COLLIDES.ACTIVE
	) {
		ig.Entity.solveCollision( a, b );
	}
};


ig.Entity.solveCollision = function( a, b ) {
	
	// If one entity is FIXED, or the other entity is LITE, the weak
	// (FIXED/NON-LITE) entity won't move in collision response
	var weak = null;
	if(
		a.collides == ig.Entity.COLLIDES.LITE ||
		b.collides == ig.Entity.COLLIDES.FIXED
	) {
		weak = a;
	}
	else if(
		b.collides == ig.Entity.COLLIDES.LITE ||
		a.collides == ig.Entity.COLLIDES.FIXED
	) {
		weak = b;
	}
		
	
	// Did they already overlap on the X-axis in the last frame? If so,
	// this must be a vertical collision!
	if(
		a.last.x + a.size.x > b.last.x &&
		a.last.x < b.last.x + b.size.x
	) {
		// Which one is on top?
		if( a.last.y < b.last.y ) {
			ig.Entity.seperateOnYAxis( a, b, weak );
		}
		else {
			ig.Entity.seperateOnYAxis( b, a, weak );
		}
		a.collideWith( b, 'y' );
		b.collideWith( a, 'y' );
	}
	
	// Horizontal collision
	else if(
		a.last.y + a.size.y > b.last.y &&
		a.last.y < b.last.y + b.size.y
	){
		// Which one is on the left?
		if( a.last.x < b.last.x ) {
			ig.Entity.seperateOnXAxis( a, b, weak );
		}
		else {
			ig.Entity.seperateOnXAxis( b, a, weak );
		}
		a.collideWith( b, 'x' );
		b.collideWith( a, 'x' );
	}
};


// FIXME: This is a mess. Instead of doing all the movements here, the entities
// should get notified of the collision (with all details) and resolve it
// themselfs.

ig.Entity.seperateOnXAxis = function( left, right, weak ) {
	var nudge = (left.pos.x + left.size.x - right.pos.x);
	
	// We have a weak entity, so just move this one
	if( weak ) {
		var strong = left === weak ? right : left;
		weak.vel.x = -weak.vel.x * weak.bounciness + strong.vel.x;
		
		var resWeak = ig.game.collisionMap.trace( 
			weak.pos.x, weak.pos.y, weak == left ? -nudge : nudge, 0, weak.size.x, weak.size.y
		);
		weak.pos.x = resWeak.pos.x;
	}
	
	// Normal collision - both move
	else {
		var v2 = (left.vel.x - right.vel.x)/2;
		left.vel.x = -v2;
		right.vel.x = v2;
	
		var resLeft = ig.game.collisionMap.trace( 
			left.pos.x, left.pos.y, -nudge/2, 0, left.size.x, left.size.y
		);
		left.pos.x = Math.floor(resLeft.pos.x);
		
		var resRight = ig.game.collisionMap.trace( 
			right.pos.x, right.pos.y, nudge/2, 0, right.size.x, right.size.y
		);
		right.pos.x = Math.ceil(resRight.pos.x);
	}
};


ig.Entity.seperateOnYAxis = function( top, bottom, weak ) {
	var nudge = (top.pos.y + top.size.y - bottom.pos.y);
	
	// We have a weak entity, so just move this one
	if( weak ) {
		var strong = top === weak ? bottom : top;
		weak.vel.y = -weak.vel.y * weak.bounciness + strong.vel.y;
		
		// Riding on a platform?
		var nudgeX = 0;
		if( weak == top && Math.abs(weak.vel.y - strong.vel.y) < weak.minBounceVelocity ) {
			weak.standing = true;
			nudgeX = strong.vel.x * ig.system.tick;
		}
		
		var resWeak = ig.game.collisionMap.trace( 
			weak.pos.x, weak.pos.y, nudgeX, weak == top ? -nudge : nudge, weak.size.x, weak.size.y
		);
		weak.pos.y = resWeak.pos.y;
		weak.pos.x = resWeak.pos.x;
	}
	
	// Bottom entity is standing - just bounce the top one
	else if( ig.game.gravity && (bottom.standing || top.vel.y > 0) ) {	
		var resTop = ig.game.collisionMap.trace( 
			top.pos.x, top.pos.y, 0, -(top.pos.y + top.size.y - bottom.pos.y), top.size.x, top.size.y
		);
		top.pos.y = resTop.pos.y;
		
		if( top.bounciness > 0 && top.vel.y > top.minBounceVelocity ) {
			top.vel.y *= -top.bounciness;		
		}
		else {
			top.standing = true;
			top.vel.y = 0;
		}
	}
	
	// Normal collision - both move
	else {
		var v2 = (top.vel.y - bottom.vel.y)/2;
		top.vel.y = -v2;
		bottom.vel.y = v2;
		
		var nudgeX = bottom.vel.x * ig.system.tick;
		var resTop = ig.game.collisionMap.trace( 
			top.pos.x, top.pos.y, nudgeX, -nudge/2, top.size.x, top.size.y
		);
		top.pos.y = resTop.pos.y;
		
		var resBottom = ig.game.collisionMap.trace( 
			bottom.pos.x, bottom.pos.y, 0, nudge/2, bottom.size.x, bottom.size.y
		);
		bottom.pos.y = resBottom.pos.y;
	}
};

});

// lib/impact/map.js
ig.baked=true;ig.module(
	'impact.map'
).defines(function(){ "use strict";

ig.Map = ig.Class.extend({
	tilesize: 8,
	width: 1,
	height: 1,
	data: [[]],
	name: null,
	
	
	init: function( tilesize, data ) {
		this.tilesize = tilesize;
		this.data = data;
		this.height = data.length;
		this.width = data[0].length;

		this.pxWidth = this.width * this.tilesize;
		this.pxHeight = this.height * this.tilesize;
	},
	
	
	getTile: function( x, y ) {
		var tx = Math.floor( x / this.tilesize );
		var ty = Math.floor( y / this.tilesize );
		if( 
			(tx >= 0 && tx <  this.width) &&
			(ty >= 0 && ty < this.height)
		) {
			return this.data[ty][tx];
		} 
		else {
			return 0;
		}
	},
	
	
	setTile: function( x, y, tile ) {
		var tx = Math.floor( x / this.tilesize );
		var ty = Math.floor( y / this.tilesize );
		if( 
			(tx >= 0 && tx < this.width) &&
			(ty >= 0 && ty < this.height)
		) {
			this.data[ty][tx] = tile;
		}
	}
});

});

// lib/impact/collision-map.js
ig.baked=true;ig.module(
	'impact.collision-map'
).requires(
	'impact.map'
).defines(function(){ "use strict";

ig.CollisionMap = ig.Map.extend({
	
	lastSlope: 1,
	tiledef: null,
	
	init: function( tilesize, data, tiledef ) {
		this.parent( tilesize, data );
		this.tiledef = tiledef || ig.CollisionMap.defaultTileDef;
		
		for( var t in this.tiledef ) {
			if( t|0 > this.lastSlope ) {
				this.lastSlope = t|0;
			}
		}
	},
	
	
	trace: function( x, y, vx, vy, objectWidth, objectHeight ) {
		// Set up the trace-result
		var res = {
			collision: {x: false, y: false, slope: false},
			pos: {x: x, y: y},
			tile: {x: 0, y: 0}
		};
		
		// Break the trace down into smaller steps if necessary
		var steps = Math.ceil(Math.max(Math.abs(vx), Math.abs(vy)) / this.tilesize);
		if( steps > 1 ) {
			var sx = vx / steps;
			var sy = vy / steps;
			
			for( var i = 0; i < steps && (sx || sy); i++ ) {
				this._traceStep( res, x, y, sx, sy, objectWidth, objectHeight, vx, vy, i );
				
				x = res.pos.x;
				y = res.pos.y;
				if( res.collision.x ) {	sx = 0; vx = 0; }
				if( res.collision.y ) {	sy = 0;	vy = 0; }
				if( res.collision.slope ) { break; }
			}
		}
		
		// Just one step
		else {
			this._traceStep( res, x, y, vx, vy, objectWidth, objectHeight, vx, vy, 0 );
		}
		
		return res;
	},
	
	
	_traceStep: function( res, x, y, vx, vy, width, height, rvx, rvy, step ) {
		
		res.pos.x += vx;
		res.pos.y += vy;
		
		var t = 0;
		
		// Horizontal collision (walls)
		if( vx ) {
			var pxOffsetX = (vx > 0 ? width : 0);
			var tileOffsetX = (vx < 0 ? this.tilesize : 0);
			
			var firstTileY = Math.max( Math.floor(y / this.tilesize), 0 );
			var lastTileY = Math.min( Math.ceil((y + height) / this.tilesize), this.height );
			var tileX = Math.floor( (res.pos.x + pxOffsetX) / this.tilesize );
			
			// We need to test the new tile position as well as the current one, as we
			// could still collide with the current tile if it's a line def.
			// We can skip this test if this is not the first step or the new tile position
			// is the same as the current one.
			var prevTileX = Math.floor( (x + pxOffsetX) / this.tilesize );
			if( step > 0 || tileX == prevTileX || prevTileX < 0 || prevTileX >= this.width ) {
				prevTileX = -1;
			}
			
			// Still inside this collision map?
			if(	tileX >= 0 && tileX < this.width ) {
				for( var tileY = firstTileY; tileY < lastTileY; tileY++ ) {
					if( prevTileX != -1 ) {
						t = this.data[tileY][prevTileX];
						if(	
							t > 1 && t <= this.lastSlope && 
							this._checkTileDef(res, t, x, y, rvx, rvy, width, height, prevTileX, tileY) 
						) {
							break;
						}
					}
					
					t = this.data[tileY][tileX];
					if(
						t == 1 || t > this.lastSlope || // fully solid tile?
						(t > 1 && this._checkTileDef(res, t, x, y, rvx, rvy, width, height, tileX, tileY)) // slope?
					) {
						if( t > 1 && t <= this.lastSlope && res.collision.slope ) {
							break;
						}
						
						// full tile collision!
						res.collision.x = true;
						res.tile.x = t;
						x = res.pos.x = tileX * this.tilesize - pxOffsetX + tileOffsetX;
						rvx = 0;
						break;
					}
				}
			}
		}
		
		// Vertical collision (floor, ceiling)
		if( vy ) {
			var pxOffsetY = (vy > 0 ? height : 0);
			var tileOffsetY = (vy < 0 ? this.tilesize : 0);
			
			var firstTileX = Math.max( Math.floor(res.pos.x / this.tilesize), 0 );
			var lastTileX = Math.min( Math.ceil((res.pos.x + width) / this.tilesize), this.width );
			var tileY = Math.floor( (res.pos.y + pxOffsetY) / this.tilesize );
			
			var prevTileY = Math.floor( (y + pxOffsetY) / this.tilesize );
			if( step > 0 || tileY == prevTileY || prevTileY < 0 || prevTileY >= this.height ) {
				prevTileY = -1;
			}
			
			// Still inside this collision map?
			if( tileY >= 0 && tileY < this.height ) {
				for( var tileX = firstTileX; tileX < lastTileX; tileX++ ) {
					if( prevTileY != -1 ) {
						t = this.data[prevTileY][tileX];
						if( 
							t > 1 && t <= this.lastSlope &&
							this._checkTileDef(res, t, x, y, rvx, rvy, width, height, tileX, prevTileY) ) {
							break;
						}
					}
					
					t = this.data[tileY][tileX];
					if(
						t == 1 || t > this.lastSlope || // fully solid tile?
						(t > 1 && this._checkTileDef(res, t, x, y, rvx, rvy, width, height, tileX, tileY)) // slope?
					) {
						if( t > 1 && t <= this.lastSlope && res.collision.slope ) {
							break;
						}
						
						// full tile collision!
						res.collision.y = true;
						res.tile.y = t;
						res.pos.y = tileY * this.tilesize - pxOffsetY + tileOffsetY;
						break;
					}
				}
			}
		}
		
		// res is changed in place, nothing to return
	},
	
	
	_checkTileDef: function( res, t, x, y, vx, vy, width, height, tileX, tileY ) {
		var def = this.tiledef[t];
		if( !def ) { return false; }
		
		var lx = (tileX + def[0]) * this.tilesize,
			ly = (tileY + def[1]) * this.tilesize,
			lvx = (def[2] - def[0]) * this.tilesize,
			lvy = (def[3] - def[1]) * this.tilesize,
			solid = def[4];
		
		// Find the box corner to test, relative to the line
		var tx = x + vx + (lvy < 0 ? width : 0) - lx,
			ty = y + vy + (lvx > 0 ? height : 0) - ly;
		
		// Is the box corner behind the line?
		if( lvx * ty - lvy * tx > 0 ) {
			
			// Lines are only solid from one side - find the dot product of
			// line normal and movement vector and dismiss if wrong side
			if( vx * -lvy + vy * lvx < 0 ) {
				return solid;
			}
			
			// Find the line normal
			var length = Math.sqrt(lvx * lvx + lvy * lvy);
			var nx = lvy/length,
				ny = -lvx/length;
			
			// Project out of the line
			var proj = tx * nx + ty * ny;
			var px = nx * proj,
				py = ny * proj;
			
			// If we project further out than we moved in, then this is a full
			// tile collision for solid tiles.
			// For non-solid tiles, make sure we were in front of the line. 
			if( px*px+py*py >= vx*vx+vy*vy ) {
				return solid || (lvx * (ty-vy) - lvy * (tx-vx) < 0.5);
			}
			
			res.pos.x = x + vx - px;
			res.pos.y = y + vy - py;
			res.collision.slope = {x: lvx, y: lvy, nx: nx, ny: ny};
			return true;
		}
		
		return false;
	}
});


// Default Slope Tile definition. Each tile is defined by an array of 5 vars:
// - 4 for the line in tile coordinates (0 -- 1)
// - 1 specifing whether the tile is 'filled' behind the line or not
// [ x1, y1, x2, y2, solid ]

// Defining 'half', 'one third' and 'two thirds' as vars  makes it a bit
// easier to read... I hope.
var H = 1/2,
	N = 1/3,
	M = 2/3,
	SOLID = true,
	NON_SOLID = false;
	
ig.CollisionMap.defaultTileDef = {	
	/* 15 NE */	 5: [0,1, 1,M, SOLID],  6: [0,M, 1,N, SOLID],  7: [0,N, 1,0, SOLID],
	/* 22 NE */	 3: [0,1, 1,H, SOLID],  4: [0,H, 1,0, SOLID],
	/* 45 NE */  2: [0,1, 1,0, SOLID],
	/* 67 NE */ 10: [H,1, 1,0, SOLID], 21: [0,1, H,0, SOLID],
	/* 75 NE */ 32: [M,1, 1,0, SOLID], 43: [N,1, M,0, SOLID], 54: [0,1, N,0, SOLID],
	
	/* 15 SE */	27: [0,0, 1,N, SOLID], 28: [0,N, 1,M, SOLID], 29: [0,M, 1,1, SOLID],
	/* 22 SE */	25: [0,0, 1,H, SOLID], 26: [0,H, 1,1, SOLID],
	/* 45 SE */	24: [0,0, 1,1, SOLID],
	/* 67 SE */	11: [0,0, H,1, SOLID], 22: [H,0, 1,1, SOLID],
	/* 75 SE */	33: [0,0, N,1, SOLID], 44: [N,0, M,1, SOLID], 55: [M,0, 1,1, SOLID],
	
	/* 15 NW */	16: [1,N, 0,0, SOLID], 17: [1,M, 0,N, SOLID], 18: [1,1, 0,M, SOLID],
	/* 22 NW */	14: [1,H, 0,0, SOLID], 15: [1,1, 0,H, SOLID],
	/* 45 NW */	13: [1,1, 0,0, SOLID],
	/* 67 NW */	 8: [H,1, 0,0, SOLID], 19: [1,1, H,0, SOLID],
	/* 75 NW */	30: [N,1, 0,0, SOLID], 41: [M,1, N,0, SOLID], 52: [1,1, M,0, SOLID],
	
	/* 15 SW */ 38: [1,M, 0,1, SOLID], 39: [1,N, 0,M, SOLID], 40: [1,0, 0,N, SOLID],
	/* 22 SW */ 36: [1,H, 0,1, SOLID], 37: [1,0, 0,H, SOLID],
	/* 45 SW */ 35: [1,0, 0,1, SOLID],
	/* 67 SW */  9: [1,0, H,1, SOLID], 20: [H,0, 0,1, SOLID],
	/* 75 SW */ 31: [1,0, M,1, SOLID], 42: [M,0, N,1, SOLID], 53: [N,0, 0,1, SOLID],
	
	/* Go N  */ 12: [0,0, 1,0, NON_SOLID],
	/* Go S  */ 23: [1,1, 0,1, NON_SOLID],
	/* Go E  */ 34: [1,0, 1,1, NON_SOLID],
	/* Go W  */ 45: [0,1, 0,0, NON_SOLID]
	
	// Now that was fun!
};


// Static Dummy CollisionMap; never collides
ig.CollisionMap.staticNoCollision = { trace: function( x, y, vx, vy ) {
	return {
		collision: {x: false, y: false, slope: false},
		pos: {x: x+vx, y: y+vy},
		tile: {x: 0, y: 0}
	};
}};

});

// lib/impact/background-map.js
ig.baked=true;ig.module(
	'impact.background-map'
).requires(
	'impact.map',
	'impact.image'
).defines(function(){ "use strict";

ig.BackgroundMap = ig.Map.extend({	
	tiles: null,
	scroll: {x: 0, y:0},
	distance: 1,
	repeat: false,
	tilesetName: '',
	foreground: false,
	enabled: true,
	
	preRender: false,
	preRenderedChunks: null,
	chunkSize: 512,
	debugChunks: false,
	
	
	anims: {},
	
	
	init: function( tilesize, data, tileset ) {
		this.parent( tilesize, data );
		this.setTileset( tileset );
	},
	
	
	setTileset: function( tileset ) {
		this.tilesetName  = tileset instanceof ig.Image ? tileset.path : tileset;
		this.tiles = new ig.Image( this.tilesetName );
		this.preRenderedChunks = null;
	},
	
	
	setScreenPos: function( x, y ) {
		this.scroll.x = x / this.distance;
		this.scroll.y = y / this.distance;
	},
	
	
	preRenderMapToChunks: function() {
		var totalWidth = this.width * this.tilesize * ig.system.scale,
			totalHeight = this.height * this.tilesize * ig.system.scale;
		
		// If this layer is smaller than the chunkSize, adjust the chunkSize
		// accordingly, so we don't have as much overdraw
		this.chunkSize = Math.min( Math.max(totalWidth, totalHeight), this.chunkSize );
			
		var chunkCols = Math.ceil(totalWidth / this.chunkSize),
			chunkRows = Math.ceil(totalHeight / this.chunkSize);
		
		this.preRenderedChunks = [];
		for( var y = 0; y < chunkRows; y++ ) {
			this.preRenderedChunks[y] = [];
			
			for( var x = 0; x < chunkCols; x++ ) {
				
				
				var chunkWidth = (x == chunkCols-1)
					? totalWidth - x * this.chunkSize
					: this.chunkSize;
					
				var chunkHeight = (y == chunkRows-1)
					? totalHeight - y * this.chunkSize
					: this.chunkSize;
					
				this.preRenderedChunks[y][x] = this.preRenderChunk( x, y, chunkWidth, chunkHeight );
			}
		}
	},
	
	
	preRenderChunk: function( cx, cy, w, h ) {
		var tw = w / this.tilesize / ig.system.scale + 1,
			th = h / this.tilesize / ig.system.scale + 1;
		
		var nx = (cx * this.chunkSize / ig.system.scale) % this.tilesize,
			ny = (cy * this.chunkSize / ig.system.scale) % this.tilesize;
		
		var tx = Math.floor(cx * this.chunkSize / this.tilesize / ig.system.scale),
			ty = Math.floor(cy * this.chunkSize / this.tilesize / ig.system.scale);
		
		
		var chunk = ig.$new('canvas');
		chunk.width = w;
		chunk.height = h;
		chunk.retinaResolutionEnabled = false; // Opt out for Ejecta
		
		var chunkContext = chunk.getContext('2d');
		ig.System.scaleMode(chunk, chunkContext);
		
		var screenContext = ig.system.context;
		ig.system.context = chunkContext;
		
		for( var x = 0; x < tw; x++ ) {
			for( var y = 0; y < th; y++ ) {
				if( x + tx < this.width && y + ty < this.height ) {
					var tile = this.data[y+ty][x+tx];
					if( tile ) {
						this.tiles.drawTile(
							x * this.tilesize - nx,	y * this.tilesize - ny,
							tile - 1, this.tilesize
						);
					}
				}
			}
		}
		ig.system.context = screenContext;
		
		return chunk;
	},
	
	
	draw: function() {
		if( !this.tiles.loaded || !this.enabled ) {
			return;
		}
		
		if( this.preRender ) {
			this.drawPreRendered();
		}
		else {
			this.drawTiled();
		}
	},
		
	
	drawPreRendered: function() {
		if( !this.preRenderedChunks ) {
			this.preRenderMapToChunks();
		}
		
		var dx = ig.system.getDrawPos(this.scroll.x),
			dy = ig.system.getDrawPos(this.scroll.y);
			
			
		if( this.repeat ) {
			var w = this.width * this.tilesize * ig.system.scale;
			dx = (dx%w + w) % w;

			var h = this.height * this.tilesize * ig.system.scale;
			dy = (dy%h + h) % h;
		}
		
		var minChunkX = Math.max( Math.floor(dx / this.chunkSize), 0 ),
			minChunkY = Math.max( Math.floor(dy / this.chunkSize), 0 ),
			maxChunkX = Math.ceil((dx+ig.system.realWidth) / this.chunkSize),
			maxChunkY = Math.ceil((dy+ig.system.realHeight) / this.chunkSize),
			maxRealChunkX = this.preRenderedChunks[0].length,
			maxRealChunkY = this.preRenderedChunks.length;
			
		
		if( !this.repeat ) {
			maxChunkX = Math.min( maxChunkX, maxRealChunkX );
			maxChunkY = Math.min( maxChunkY, maxRealChunkY );
		}
		
		
		var nudgeY = 0;
		for( var cy = minChunkY; cy < maxChunkY; cy++ ) {
			
			var nudgeX = 0;
			for( var cx = minChunkX; cx < maxChunkX; cx++ ) {
				var chunk = this.preRenderedChunks[cy % maxRealChunkY][cx % maxRealChunkX];
				
				var x = -dx + cx * this.chunkSize - nudgeX;
				var y = -dy + cy * this.chunkSize - nudgeY;
				ig.system.context.drawImage( chunk, x, y);
				ig.Image.drawCount++;
				
				if( this.debugChunks ) {
					ig.system.context.strokeStyle = '#f0f';
					ig.system.context.strokeRect( x, y, this.chunkSize, this.chunkSize );
				}
				
				// If we repeat in X and this chunk's width wasn't the full chunk size
				// and the screen is not already filled, we need to draw anohter chunk
				// AND nudge it to be flush with the last chunk
				if( this.repeat && chunk.width < this.chunkSize && x + chunk.width < ig.system.realWidth ) {
					nudgeX += this.chunkSize - chunk.width;
					maxChunkX++;
				}
			}
			
			// Same as above, but for Y
			if( this.repeat && chunk.height < this.chunkSize && y + chunk.height < ig.system.realHeight	) {
				nudgeY += this.chunkSize - chunk.height;
				maxChunkY++;
			}
		}
	},
	
	
	drawTiled: function() {	
		var tile = 0,
			anim = null,
			tileOffsetX = (this.scroll.x / this.tilesize).toInt(),
			tileOffsetY = (this.scroll.y / this.tilesize).toInt(),
			pxOffsetX = this.scroll.x % this.tilesize,
			pxOffsetY = this.scroll.y % this.tilesize,
			pxMinX = -pxOffsetX - this.tilesize,
			pxMinY = -pxOffsetY - this.tilesize,
			pxMaxX = ig.system.width + this.tilesize - pxOffsetX,
			pxMaxY = ig.system.height + this.tilesize - pxOffsetY;
			
		
		// FIXME: could be sped up for non-repeated maps: restrict the for loops
		// to the map size instead of to the screen size and skip the 'repeat'
		// checks inside the loop.
		
		for( var mapY = -1, pxY = pxMinY; pxY < pxMaxY; mapY++, pxY += this.tilesize) {
			var tileY = mapY + tileOffsetY;
				
			// Repeat Y?
			if( tileY >= this.height || tileY < 0 ) {
				if( !this.repeat ) { continue; }
				tileY = (tileY%this.height + this.height) % this.height;
			}
			
			for( var mapX = -1, pxX = pxMinX; pxX < pxMaxX; mapX++, pxX += this.tilesize ) {
				var tileX = mapX + tileOffsetX;
				
				// Repeat X?
				if( tileX >= this.width || tileX < 0 ) {
					if( !this.repeat ) { continue; }
					tileX = (tileX%this.width + this.width) % this.width;
				}
				
				// Draw!
				if( (tile = this.data[tileY][tileX]) ) {
					if( (anim = this.anims[tile-1]) ) { 
						anim.draw( pxX, pxY );
					}
					else {
						this.tiles.drawTile( pxX, pxY, tile-1, this.tilesize );
					}
				}
			} // end for x
		} // end for y
	}
});

});

// lib/impact/game.js
ig.baked=true;ig.module(
	'impact.game'
).requires(
	'impact.impact',
	'impact.entity',
	'impact.collision-map',
	'impact.background-map'
).defines(function(){ "use strict";

ig.Game = ig.Class.extend({
	
	clearColor: '#000000',
	gravity: 0,
	screen: {x: 0, y: 0},
	_rscreen: {x: 0, y: 0},
	
	entities: [],
	
	namedEntities: {},
	collisionMap: ig.CollisionMap.staticNoCollision,
	backgroundMaps: [],
	backgroundAnims: {},
	
	autoSort: false,
	sortBy: null,
	
	cellSize: 64,
	
	_deferredKill: [],
	_levelToLoad: null,
	_doSortEntities: false,
	
	staticInstantiate: function() {
		this.sortBy = this.sortBy || ig.Game.SORT.Z_INDEX;
		ig.game = this;
		return null;
	},
	
	
	loadLevel: function( data ) {
		this.screen = {x: 0, y: 0};

		// Entities
		this.entities = [];
		this.namedEntities = {};
		for( var i = 0; i < data.entities.length; i++ ) {
			var ent = data.entities[i];
			this.spawnEntity( ent.type, ent.x, ent.y, ent.settings );
		}
		this.sortEntities();
		
		// Map Layer
		this.collisionMap = ig.CollisionMap.staticNoCollision;
		this.backgroundMaps = [];
		for( var i = 0; i < data.layer.length; i++ ) {
			var ld = data.layer[i];
			if( ld.name == 'collision' ) {
				this.collisionMap = new ig.CollisionMap(ld.tilesize, ld.data );
			}
			else {
				var newMap = new ig.BackgroundMap(ld.tilesize, ld.data, ld.tilesetName);
				newMap.anims = this.backgroundAnims[ld.tilesetName] || {};
				newMap.repeat = ld.repeat;
				newMap.distance = ld.distance;
				newMap.foreground = !!ld.foreground;
				newMap.preRender = !!ld.preRender;
				newMap.name = ld.name;
				this.backgroundMaps.push( newMap );
			}
		}
		
		// Call post-init ready function on all entities
		for( var i = 0; i < this.entities.length; i++ ) {
			this.entities[i].ready();
		}
	},
	
	
	loadLevelDeferred: function( data ) {
		this._levelToLoad = data;
	},
	
	
	getMapByName: function( name ) {
		if( name == 'collision' ) {
			return this.collisionMap;
		}
		
		for( var i = 0; i < this.backgroundMaps.length; i++ ) {
			if( this.backgroundMaps[i].name == name ) {
				return this.backgroundMaps[i];
			}
		}
		
		return null;
	},
	
	
	getEntityByName: function( name ) {
		return this.namedEntities[name];
	},
	
	
	getEntitiesByType: function( type ) {
		var entityClass = typeof(type) === 'string'
			? ig.global[type]
			: type;
			
		var a = [];
		for( var i = 0; i < this.entities.length; i++ ) {
			var ent = this.entities[i];
			if( ent instanceof entityClass && !ent._killed ) {
				a.push( ent );
			}
		}
		return a;
	},
	
	
	spawnEntity: function( type, x, y, settings ) {
		var entityClass = typeof(type) === 'string'
			? ig.global[type]
			: type;
			
		if( !entityClass ) {
			throw("Can't spawn entity of type " + type);
		}
		var ent = new (entityClass)( x, y, settings || {} );
		this.entities.push( ent );
		if( ent.name ) {
			this.namedEntities[ent.name] = ent;
		}
		return ent;
	},
	
	
	sortEntities: function() {
		this.entities.sort( this.sortBy );
	},
	
	
	sortEntitiesDeferred: function() {
		this._doSortEntities = true;
	},
	
	
	removeEntity: function( ent ) {
		// Remove this entity from the named entities
		if( ent.name ) {
			delete this.namedEntities[ent.name];
		}
		
		// We can not remove the entity from the entities[] array in the midst
		// of an update cycle, so remember all killed entities and remove
		// them later.
		// Also make sure this entity doesn't collide anymore and won't get
		// updated or checked
		ent._killed = true;
		ent.type = ig.Entity.TYPE.NONE;
		ent.checkAgainst = ig.Entity.TYPE.NONE;
		ent.collides = ig.Entity.COLLIDES.NEVER;
		this._deferredKill.push( ent );
	},
	
	
	run: function() {
		this.update();
		this.draw();
	},
	
	
	update: function(){
		// load new level?
		if( this._levelToLoad ) {
			this.loadLevel( this._levelToLoad );
			this._levelToLoad = null;
		}
		
		// update entities
		this.updateEntities();
		this.checkEntities();
		
		// remove all killed entities
		for( var i = 0; i < this._deferredKill.length; i++ ) {
			this._deferredKill[i].erase();
			this.entities.erase( this._deferredKill[i] );
		}
		this._deferredKill = [];
		
		// sort entities?
		if( this._doSortEntities || this.autoSort ) {
			this.sortEntities();
			this._doSortEntities = false;
		}
		
		// update background animations
		for( var tileset in this.backgroundAnims ) {
			var anims = this.backgroundAnims[tileset];
			for( var a in anims ) {
				anims[a].update();
			}
		}
	},
	
	
	updateEntities: function() {
		for( var i = 0; i < this.entities.length; i++ ) {
			var ent = this.entities[i];
			if( !ent._killed ) {
				ent.update();
			}
		}
	},
	
	
	draw: function(){
		ig.system.clear( this.clearColor );
		
		// This is a bit of a circle jerk. Entities reference game._rscreen 
		// instead of game.screen when drawing themselfs in order to be 
		// "synchronized" to the rounded(?) screen position
		this._rscreen.x = ig.system.getDrawPos(this.screen.x)/ig.system.scale;
		this._rscreen.y = ig.system.getDrawPos(this.screen.y)/ig.system.scale;
		
		
		var mapIndex;
		for( mapIndex = 0; mapIndex < this.backgroundMaps.length; mapIndex++ ) {
			var map = this.backgroundMaps[mapIndex];
			if( map.foreground ) {
				// All foreground layers are drawn after the entities
				break;
			}
			map.setScreenPos( this.screen.x, this.screen.y );
			map.draw();
		}
		
		
		this.drawEntities();
		
		
		for( mapIndex; mapIndex < this.backgroundMaps.length; mapIndex++ ) {
			var map = this.backgroundMaps[mapIndex];
			map.setScreenPos( this.screen.x, this.screen.y );
			map.draw();
		}
	},
	
	
	drawEntities: function() {
		for( var i = 0; i < this.entities.length; i++ ) {
			this.entities[i].draw();
		}
	},

	
	checkEntities: function() {
		// Insert all entities into a spatial hash and check them against any
		// other entity that already resides in the same cell. Entities that are
		// bigger than a single cell, are inserted into each one they intersect
		// with.
		
		// A list of entities, which the current one was already checked with,
		// is maintained for each entity.
		
		var hash = {};
		for( var e = 0; e < this.entities.length; e++ ) {
			var entity = this.entities[e];
			
			// Skip entities that don't check, don't get checked and don't collide
			if(
				entity.type == ig.Entity.TYPE.NONE &&
				entity.checkAgainst == ig.Entity.TYPE.NONE &&
				entity.collides == ig.Entity.COLLIDES.NEVER
			) {
				continue;
			}
			
			var checked = {},
				xmin = Math.floor( entity.pos.x/this.cellSize ),
				ymin = Math.floor( entity.pos.y/this.cellSize ),
				xmax = Math.floor( (entity.pos.x+entity.size.x)/this.cellSize ) + 1,
				ymax = Math.floor( (entity.pos.y+entity.size.y)/this.cellSize ) + 1;
			
			for( var x = xmin; x < xmax; x++ ) {
				for( var y = ymin; y < ymax; y++ ) {
					
					// Current cell is empty - create it and insert!
					if( !hash[x] ) {
						hash[x] = {};
						hash[x][y] = [entity];
					}
					else if( !hash[x][y] ) {
						hash[x][y] = [entity];
					}
					
					// Check against each entity in this cell, then insert
					else {
						var cell = hash[x][y];
						for( var c = 0; c < cell.length; c++ ) {
							
							// Intersects and wasn't already checkd?
							if( entity.touches(cell[c]) && !checked[cell[c].id] ) {
								checked[cell[c].id] = true;
								ig.Entity.checkPair( entity, cell[c] );
							}
						}
						cell.push(entity);
					}
				} // end for y size
			} // end for x size
		} // end for entities
	}
});

ig.Game.SORT = {
	Z_INDEX: function( a, b ){ return a.zIndex - b.zIndex; },
	POS_X: function( a, b ){ return (a.pos.x+a.size.x) - (b.pos.x+b.size.x); },
	POS_Y: function( a, b ){ return (a.pos.y+a.size.y) - (b.pos.y+b.size.y); }
};

});

// lib/gamewheel/helpers/util.js
ig.baked=true;ig.module(
    'gamewheel.helpers.util'
).requires(
    'impact.system'
).defines(function() { "use strict";

        /**
         * Static utilities for general use.
         **/
        if(!ig.util) {
            ig.util = {};    
        }
        

        /**
         * Adjusts the given x and y values 
         **/
        ig.util.locationToPos = function(location, size){
            var pos = {x: 0, y: 0};
            size = size || {x: 0, y: 0};

            if( 'x' in location ) {
                location.left = location.x;
            }
            if( 'left' in location ) {
                if(location.left == 'center' || location.left == 'centre' || location.left == 'middle') {
                    pos.x = (ig.system.width / 2) - (size.x / 2);
                } else {
                    pos.x = location.left;
                }
            }
            else if( 'right' in location ) {
                if(location.right == 'center' || location.right == 'centre' || location.right == 'middle') {
                    pos.x = (ig.system.width / 2) - (size.x / 2);
                } else {
                    pos.x = ig.system.width - location.right - size.x;
                }
            }

            if( 'y' in location ) {
                location.top = location.y;
            }
            if( 'top' in location ) {
                if(location.top == 'center' || location.top == 'centre' || location.top == 'middle') {
                    pos.y = (ig.system.height / 2) - (size.y / 2);
                } else {
                    pos.y = location.top;
                }
            }
            else if( 'bottom' in location ) {
                if(location.bottom == 'center' || location.bottom == 'centre' || location.bottom == 'middle') {
                    pos.y = (ig.system.height / 2) - (size.y / 2);
                } else {
                    pos.y = ig.system.height - location.bottom - size.y;
                }
            }
            return pos;
        };

        ig.util.align = function (entity, anchor, bounds) {
            // Default to align relative to screen when no bounds provided.
            bounds = bounds || {
                    pos: {x: 0, y: 0},
                    size: {x: ig.system.width, y: ig.system.height}
                };


            // Align horizontally.
            if(anchor.left || anchor.left === 0) {
                entity.pos.x = bounds.pos.x + anchor.left;
                if(anchor.right || anchor.right === 0) {
                    // Anchor on left and right, so resize the entity.
                    entity.size.x = bounds.size.x - (anchor.left + anchor.right);
                }
            } else if(anchor.right || anchor.right === 0) {
                entity.pos.x = bounds.pos.x + bounds.size.x - (entity.size.x + anchor.right);
            }

            // Align vertically.
            if(anchor.top || anchor.top === 0) {
                entity.pos.y = bounds.pos.y + anchor.top;
                if(anchor.bottom || anchor.bottom === 0) {
                    // Anchor on top and bottom, so resize the entity.
                    entity.size.y = bounds.size.y - (anchor.top + anchor.bottom);
                }
            } else if(anchor.bottom || anchor.bottom === 0) {
                entity.pos.y = bounds.pos.y + bounds.size.y - (entity.size.y + anchor.bottom);
            }
        };

        // Returns a random integer within range of minValue and maxValue (inclusive).
        // If only one value is provided produces a random integer between 0 and that number.
        ig.util.randomInt = function (minValue, maxValue) {
            if (maxValue == null) {
                maxValue = minValue;
                minValue = 0;
            }
            return minValue + Math.floor(Math.random() * (maxValue - minValue + 1));
        };
        
        // Returns a random float that is greater than or equal to minValue and less than upperBound.
        // If only one value is provided produces a random integer between 0 and that number.
        ig.util.randomFloat = function(minValue, upperBound) {
            if (upperBound == null) {
                upperBound = minValue;
                minValue = 0;
            }
            return (Math.random() * (upperBound - minValue)) + minValue;
        };

        // Returns a random float that is equal to the base value plus or minus up to half the range value.
        ig.util.rangedFloat = function(base, range) {
            return base + ig.util.randomFloat(-range / 2, range / 2);
        };

        // Returns a random boolean value.
        ig.util.randomBool = function () {
            return Math.floor(Math.random() * 2) == 0;
        };
        
        // Returns a shuffled copy of the provided list.
        ig.util.shuffledList = function(list) {
            return ig.util.shuffleListInPlace(ig.copy(list));
        };
        
        // Shuffles the provided list in place (does not copy).
        ig.util.shuffleListInPlace = function(list) {
            var i, j, t;
            for (i = 1; i < list.length; i++) {
                j = Math.floor(Math.random()*(1+i));  // choose j in [0..i]
                if (j != i) {
                    t = list[i]; // swap list[i] and list[j]
                    list[i] = list[j];
                    list[j] = t;
                }
            }
            return list;
        };
        
        ig.util.insertionSort = function(list, compareFunction) {
            for(var i = 1; i < list.length; i++) {
                var x = list[i];
                var j = i-1;
                while (j >= 0 && compareFunction(list[j], x) > 0){
                    list[j+1] = list[j];
                    j = j-1;
                }
                list[j+1] = x;
            }
            return list;
        };

        // Returns a nested object using a string as the path to the property.
        ig.util.objectByString = function(object, propPath) {
            propPath = propPath.replace(/\[(\w+)\]/g, '.$1'); // convert indexes to properties
            propPath = propPath.replace(/^\./, '');           // strip a leading dot
            var a = propPath.split('.');
            for (var i = 0, n = a.length; i < n; ++i) {
                var k = a[i];
                if (k in object) {
                    object = object[k];
                } else {
                    return;
                }
            }
            return object;
        };

        // Sets a nested object using a string as the path to the property. Returns true if successful.
        ig.util.setObjectByString = function(object, propPath, val) {
            propPath = propPath.replace(/\[(\w+)\]/g, '.$1'); // convert indexes to properties
            propPath = propPath.replace(/^\./, '');           // strip a leading dot
            var props = propPath.split('.');

            for (var i = 0, n = props.length - 1; i < n; ++i) {
                if (props[i] in object) {
                    object = object[props[i]];
                } else {
                    return;
                }
            }

            if(props.length && props[props.length -1] in object) {
                object[props[props.length -1]] = val;
                return true;
            }

            return false;
        };
        
    });

// lib/gamewheel/core/game.js
ig.baked=true;ig.module(
    'gamewheel.core.game'
).requires(
    'impact.game',
    'gamewheel.helpers.util'
).defines(function(){ "use strict";
        
        ig.Game.inject({

            
            pushEntity: function( entity ) {
                this.entities.push( entity );
                if( entity.name ) {
                    this.namedEntities[entity.name] = entity;
                }
            },
            
            ending: function() {
                
            },
            
            sortEntities: function() {
                // Impact's default behaviour of using Array.sort() does not guarantee stable sorting.
                // Override and use an insertion sort to guarantee that elements with equal value will
                // maintain their order in the entities list.
                return ig.util.insertionSort(this.entities, this.sortBy);
            },
            
            spawnConfigEntity: function(key, settingsOverride, entityTypeOverride) {
                // Get the data for the element.
                var entityData = ig.config.entities[key];
                 
                if(entityData) {
                    // Build and return entity
                    return this.spawnEntityWithSetup(entityData, settingsOverride, entityTypeOverride);
                } else {
                    // Report missing entity
                    console.log('No entity information defined for ' + key);
                }
            },

            spawnEntityWithSetup: function(setupData, settingsOverride, entityType, doNotAdd) {
                // Get a settings object. Automatically adds image and animation data to object when present.
                var settings = this._genAnimationSettings(setupData);

                // Merge in extra settings from the entity definition.
                if(setupData.settings) {
                    ig.merge(settings, setupData.settings);
                }

                // Merge in passed in settings.
                if (settingsOverride) {
                    ig.merge(settings,  settingsOverride);
                }

                // If location is provided, calculate the position.
                var pos = {x:0, y:0};
                if(setupData.location) {
                    pos = ig.util.locationToPos(setupData.location, settings.size);
                }

                // Work out what type of entity to spawn.
                var type = entityType || setupData['type'] || ig.Entity;
                
                // Spawn the entity in the game object and return it.
                return this.spawnEntity(type, pos.x, pos.y, settings);
                
            },

            _genAnimationSettings: function (entityData) {
                var settings = {};

                if(entityData.image) {

                    settings.anims = {};
                    settings.currentAnim = null;

                    var image = ig.content.imageNamed(entityData.image);
                    settings.animSheet = new ig.AnimationSheet( image.path, image.width, image.height);

                    if (entityData.animated) {
                        settings.animSheet.width = entityData.animated['frameSize'].x;
                        settings.animSheet.height = entityData.animated['frameSize'].y;

                        var animDefs = entityData.animated['animations'];
                        for (var key in animDefs) {
                            if (animDefs.hasOwnProperty(key)) {
                                var anim = animDefs[key];

                                var a = new ig.Animation( settings.animSheet, anim.frameTime, anim.frames, !anim.loop );
                                settings.anims[key] = a;
                                if(!settings.currentAnim) {
                                    settings.currentAnim = a;
                                }
                            }
                        }

                    } else {
                        // Set default animation.
                        var idle = new ig.Animation( settings.animSheet, 1, [0], false );
                        settings.anims['idle'] = idle;
                        settings.currentAnim = idle;
                    }

                    // Set default size based on animation frame size.
                    settings.size = {x: settings.animSheet.width, y: settings.animSheet.height};
                }

                return settings;
            }

        });

    });/*
 The content class is used to pre-load and maintain instances to named assets as defined as part of the game's configuration.
 */
/*
 This is a loader class to force google web fonts to be loaded during Impact's loading phase.
 https://www.google.com/fonts
 */


// lib/gamewheel/core/googleFonts.js
ig.baked=true;ig.module(
    'gamewheel.core.googleFonts'
).defines(function() { "use strict";

        ig.GoogleFonts = ig.Class.extend({

            fonts: {families:[]},
            loaded: false,
            path: 'googleFonts',
            failed: false,
            loadCallback: null,

            staticInstantiate: function( fonts ) {
                // Fonts will only be loaded one time. Prevent creation of a second instance.
                return ig.GoogleFonts.instance || null;
            },

            init: function(fonts) {
                this.fonts = fonts;
                this.load();
                ig.GoogleFonts.instance = this;
            },

            load: function( loadCallback ) {
                if( this.loaded ) {
                    if( loadCallback ) {
                        loadCallback( this.path, true );
                    }
                }
                else if( !this.loaded && ig.ready ) {
                    this.loadCallback = loadCallback || null;

                    window.WebFont.load({
                        google: this.fonts,
                        active: this.onLoad.bind(this),
                        inactive: this.onError.bind(this),
                        timeout: 2000
                    });

                }
                else {
                    ig.addResource( this );
                }
            },

            onError: function(event) {
                this.failed = true;
                if( this.loadCallback ) {
                    // Returning true even though we failed as we don't want to kill the game. Just use default fonts. 
                    this.loadCallback( this.path, true );
                }
            },

            onLoad: function(event) {
                this.loaded = true;
                if( this.loadCallback ) {
                    this.loadCallback( this.path, true );
                }
            }

        });

    });



/* Self hosting Google Web Font Loader:
 *
 * Copyright 2016 Small Batch, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
/* Web Font Loader v1.6.16 - (c) Adobe Systems, Google. License: Apache 2.0 */
(function(){function aa(a,b,c){return a.call.apply(a.bind,arguments)}function ba(a,b,c){if(!a)throw Error();if(2<arguments.length){var d=Array.prototype.slice.call(arguments,2);return function(){var c=Array.prototype.slice.call(arguments);Array.prototype.unshift.apply(c,d);return a.apply(b,c)}}return function(){return a.apply(b,arguments)}}function n(a,b,c){n=Function.prototype.bind&&-1!=Function.prototype.bind.toString().indexOf("native code")?aa:ba;return n.apply(null,arguments)}var p=Date.now||function(){return+new Date};function r(a,b){this.F=a;this.k=b||a;this.H=this.k.document}var ca=!!window.FontFace;r.prototype.createElement=function(a,b,c){a=this.H.createElement(a);if(b)for(var d in b)b.hasOwnProperty(d)&&("style"==d?a.style.cssText=b[d]:a.setAttribute(d,b[d]));c&&a.appendChild(this.H.createTextNode(c));return a};function s(a,b,c){a=a.H.getElementsByTagName(b)[0];a||(a=document.documentElement);a.insertBefore(c,a.lastChild)}
    function t(a,b,c){b=b||[];c=c||[];for(var d=a.className.split(/\s+/),e=0;e<b.length;e+=1){for(var f=!1,g=0;g<d.length;g+=1)if(b[e]===d[g]){f=!0;break}f||d.push(b[e])}b=[];for(e=0;e<d.length;e+=1){f=!1;for(g=0;g<c.length;g+=1)if(d[e]===c[g]){f=!0;break}f||b.push(d[e])}a.className=b.join(" ").replace(/\s+/g," ").replace(/^\s+|\s+$/,"")}function u(a,b){for(var c=a.className.split(/\s+/),d=0,e=c.length;d<e;d++)if(c[d]==b)return!0;return!1}
    function v(a){if("string"===typeof a.fa)return a.fa;var b=a.k.location.protocol;"about:"==b&&(b=a.F.location.protocol);return"https:"==b?"https:":"http:"}function x(a,b,c){function d(){l&&e&&f&&(l(g),l=null)}b=a.createElement("link",{rel:"stylesheet",href:b,media:"all"});var e=!1,f=!0,g=null,l=c||null;ca?(b.onload=function(){e=!0;d()},b.onerror=function(){e=!0;g=Error("Stylesheet failed to load");d()}):setTimeout(function(){e=!0;d()},0);s(a,"head",b)}
    function y(a,b,c,d){var e=a.H.getElementsByTagName("head")[0];if(e){var f=a.createElement("script",{src:b}),g=!1;f.onload=f.onreadystatechange=function(){g||this.readyState&&"loaded"!=this.readyState&&"complete"!=this.readyState||(g=!0,c&&c(null),f.onload=f.onreadystatechange=null,"HEAD"==f.parentNode.tagName&&e.removeChild(f))};e.appendChild(f);setTimeout(function(){g||(g=!0,c&&c(Error("Script load timeout")))},d||5E3);return f}return null};function z(){this.S=0;this.K=null}function A(a){a.S++;return function(){a.S--;B(a)}}function C(a,b){a.K=b;B(a)}function B(a){0==a.S&&a.K&&(a.K(),a.K=null)};function D(a){this.ea=a||"-"}D.prototype.d=function(a){for(var b=[],c=0;c<arguments.length;c++)b.push(arguments[c].replace(/[\W_]+/g,"").toLowerCase());return b.join(this.ea)};function E(a,b){this.Q=a;this.M=4;this.L="n";var c=(b||"n4").match(/^([nio])([1-9])$/i);c&&(this.L=c[1],this.M=parseInt(c[2],10))}E.prototype.getName=function(){return this.Q};function da(a){return F(a)+" "+(a.M+"00")+" 300px "+G(a.Q)}function G(a){var b=[];a=a.split(/,\s*/);for(var c=0;c<a.length;c++){var d=a[c].replace(/['"]/g,"");-1!=d.indexOf(" ")||/^\d/.test(d)?b.push("'"+d+"'"):b.push(d)}return b.join(",")}function I(a){return a.L+a.M}
    function F(a){var b="normal";"o"===a.L?b="oblique":"i"===a.L&&(b="italic");return b}function ea(a){var b=4,c="n",d=null;a&&((d=a.match(/(normal|oblique|italic)/i))&&d[1]&&(c=d[1].substr(0,1).toLowerCase()),(d=a.match(/([1-9]00|normal|bold)/i))&&d[1]&&(/bold/i.test(d[1])?b=7:/[1-9]00/.test(d[1])&&(b=parseInt(d[1].substr(0,1),10))));return c+b};function fa(a,b){this.a=a;this.j=a.k.document.documentElement;this.O=b;this.f="wf";this.e=new D("-");this.da=!1!==b.events;this.u=!1!==b.classes}function ga(a){a.u&&t(a.j,[a.e.d(a.f,"loading")]);J(a,"loading")}function K(a){if(a.u){var b=u(a.j,a.e.d(a.f,"active")),c=[],d=[a.e.d(a.f,"loading")];b||c.push(a.e.d(a.f,"inactive"));t(a.j,c,d)}J(a,"inactive")}function J(a,b,c){if(a.da&&a.O[b])if(c)a.O[b](c.getName(),I(c));else a.O[b]()};function ha(){this.t={}}function ia(a,b,c){var d=[],e;for(e in b)if(b.hasOwnProperty(e)){var f=a.t[e];f&&d.push(f(b[e],c))}return d};function L(a,b){this.a=a;this.h=b;this.m=this.a.createElement("span",{"aria-hidden":"true"},this.h)}function M(a,b){var c=a.m,d;d="display:block;position:absolute;top:-9999px;left:-9999px;font-size:300px;width:auto;height:auto;line-height:normal;margin:0;padding:0;font-variant:normal;white-space:nowrap;font-family:"+G(b.Q)+";"+("font-style:"+F(b)+";font-weight:"+(b.M+"00")+";");c.style.cssText=d}function N(a){s(a.a,"body",a.m)}L.prototype.remove=function(){var a=this.m;a.parentNode&&a.parentNode.removeChild(a)};function O(a,b,c,d,e,f){this.G=a;this.J=b;this.g=d;this.a=c;this.v=e||3E3;this.h=f||void 0}O.prototype.start=function(){function a(){p()-d>=c.v?c.J(c.g):b.fonts.load(da(c.g),c.h).then(function(b){1<=b.length?c.G(c.g):setTimeout(a,25)},function(){c.J(c.g)})}var b=this.a.k.document,c=this,d=p();a()};function P(a,b,c,d,e,f,g){this.G=a;this.J=b;this.a=c;this.g=d;this.h=g||"BESbswy";this.s={};this.v=e||3E3;this.Z=f||null;this.D=this.C=this.A=this.w=null;this.w=new L(this.a,this.h);this.A=new L(this.a,this.h);this.C=new L(this.a,this.h);this.D=new L(this.a,this.h);M(this.w,new E(this.g.getName()+",serif",I(this.g)));M(this.A,new E(this.g.getName()+",sans-serif",I(this.g)));M(this.C,new E("serif",I(this.g)));M(this.D,new E("sans-serif",I(this.g)));N(this.w);N(this.A);N(this.C);N(this.D)}
    var Q={ia:"serif",ha:"sans-serif"},R=null;function S(){if(null===R){var a=/AppleWebKit\/([0-9]+)(?:\.([0-9]+))/.exec(window.navigator.userAgent);R=!!a&&(536>parseInt(a[1],10)||536===parseInt(a[1],10)&&11>=parseInt(a[2],10))}return R}P.prototype.start=function(){this.s.serif=this.C.m.offsetWidth;this.s["sans-serif"]=this.D.m.offsetWidth;this.ga=p();ja(this)};function ka(a,b,c){for(var d in Q)if(Q.hasOwnProperty(d)&&b===a.s[Q[d]]&&c===a.s[Q[d]])return!0;return!1}
    function ja(a){var b=a.w.m.offsetWidth,c=a.A.m.offsetWidth,d;(d=b===a.s.serif&&c===a.s["sans-serif"])||(d=S()&&ka(a,b,c));d?p()-a.ga>=a.v?S()&&ka(a,b,c)&&(null===a.Z||a.Z.hasOwnProperty(a.g.getName()))?T(a,a.G):T(a,a.J):la(a):T(a,a.G)}function la(a){setTimeout(n(function(){ja(this)},a),50)}function T(a,b){setTimeout(n(function(){this.w.remove();this.A.remove();this.C.remove();this.D.remove();b(this.g)},a),0)};function U(a,b,c){this.a=a;this.p=b;this.P=0;this.ba=this.Y=!1;this.v=c}var V=null;U.prototype.V=function(a){var b=this.p;b.u&&t(b.j,[b.e.d(b.f,a.getName(),I(a).toString(),"active")],[b.e.d(b.f,a.getName(),I(a).toString(),"loading"),b.e.d(b.f,a.getName(),I(a).toString(),"inactive")]);J(b,"fontactive",a);this.ba=!0;ma(this)};
    U.prototype.W=function(a){var b=this.p;if(b.u){var c=u(b.j,b.e.d(b.f,a.getName(),I(a).toString(),"active")),d=[],e=[b.e.d(b.f,a.getName(),I(a).toString(),"loading")];c||d.push(b.e.d(b.f,a.getName(),I(a).toString(),"inactive"));t(b.j,d,e)}J(b,"fontinactive",a);ma(this)};function ma(a){0==--a.P&&a.Y&&(a.ba?(a=a.p,a.u&&t(a.j,[a.e.d(a.f,"active")],[a.e.d(a.f,"loading"),a.e.d(a.f,"inactive")]),J(a,"active")):K(a.p))};function na(a){this.F=a;this.q=new ha;this.$=0;this.T=this.U=!0}na.prototype.load=function(a){this.a=new r(this.F,a.context||this.F);this.U=!1!==a.events;this.T=!1!==a.classes;oa(this,new fa(this.a,a),a)};
    function pa(a,b,c,d,e){var f=0==--a.$;(a.T||a.U)&&setTimeout(function(){var a=e||null,l=d||null||{};if(0===c.length&&f)K(b.p);else{b.P+=c.length;f&&(b.Y=f);var h,k=[];for(h=0;h<c.length;h++){var m=c[h],w=l[m.getName()],q=b.p,H=m;q.u&&t(q.j,[q.e.d(q.f,H.getName(),I(H).toString(),"loading")]);J(q,"fontloading",H);q=null;null===V&&(V=window.FontFace?(q=/Gecko.*Firefox\/(\d+)/.exec(window.navigator.userAgent))?42<parseInt(q[1],10):!0:!1);q=V?new O(n(b.V,b),n(b.W,b),b.a,m,b.v,w):new P(n(b.V,b),n(b.W,b),
        b.a,m,b.v,a,w);k.push(q)}for(h=0;h<k.length;h++)k[h].start()}},0)}function oa(a,b,c){var d=[],e=c.timeout;ga(b);var d=ia(a.q,c,a.a),f=new U(a.a,b,e);a.$=d.length;b=0;for(c=d.length;b<c;b++)d[b].load(function(b,c,d){pa(a,f,b,c,d)})};function qa(a,b,c){this.N=a?a:b+ra;this.o=[];this.R=[];this.ca=c||""}var ra="//fonts.googleapis.com/css";function sa(a,b){for(var c=b.length,d=0;d<c;d++){var e=b[d].split(":");3==e.length&&a.R.push(e.pop());var f="";2==e.length&&""!=e[1]&&(f=":");a.o.push(e.join(f))}}
    qa.prototype.d=function(){if(0==this.o.length)throw Error("No fonts to load!");if(-1!=this.N.indexOf("kit="))return this.N;for(var a=this.o.length,b=[],c=0;c<a;c++)b.push(this.o[c].replace(/ /g,"+"));a=this.N+"?family="+b.join("%7C");0<this.R.length&&(a+="&subset="+this.R.join(","));0<this.ca.length&&(a+="&text="+encodeURIComponent(this.ca));return a};function ta(a){this.o=a;this.aa=[];this.I={}}
    var ua={latin:"BESbswy",cyrillic:"&#1081;&#1103;&#1046;",greek:"&#945;&#946;&#931;",khmer:"&#x1780;&#x1781;&#x1782;",Hanuman:"&#x1780;&#x1781;&#x1782;"},va={thin:"1",extralight:"2","extra-light":"2",ultralight:"2","ultra-light":"2",light:"3",regular:"4",book:"4",medium:"5","semi-bold":"6",semibold:"6","demi-bold":"6",demibold:"6",bold:"7","extra-bold":"8",extrabold:"8","ultra-bold":"8",ultrabold:"8",black:"9",heavy:"9",l:"3",r:"4",b:"7"},wa={i:"i",italic:"i",n:"n",normal:"n"},xa=/^(thin|(?:(?:extra|ultra)-?)?light|regular|book|medium|(?:(?:semi|demi|extra|ultra)-?)?bold|black|heavy|l|r|b|[1-9]00)?(n|i|normal|italic)?$/;
    ta.prototype.parse=function(){for(var a=this.o.length,b=0;b<a;b++){var c=this.o[b].split(":"),d=c[0].replace(/\+/g," "),e=["n4"];if(2<=c.length){var f;var g=c[1];f=[];if(g)for(var g=g.split(","),l=g.length,h=0;h<l;h++){var k;k=g[h];if(k.match(/^[\w-]+$/))if(k=xa.exec(k.toLowerCase()),null==k)k="";else{var m;m=k[1];if(null==m||""==m)m="4";else{var w=va[m];m=w?w:isNaN(m)?"4":m.substr(0,1)}k=k[2];k=[null==k||""==k?"n":wa[k],m].join("")}else k="";k&&f.push(k)}0<f.length&&(e=f);3==c.length&&(c=c[2],f=
        [],c=c?c.split(","):f,0<c.length&&(c=ua[c[0]])&&(this.I[d]=c))}this.I[d]||(c=ua[d])&&(this.I[d]=c);for(c=0;c<e.length;c+=1)this.aa.push(new E(d,e[c]))}};function ya(a,b){this.a=a;this.c=b}var za={Arimo:!0,Cousine:!0,Tinos:!0};ya.prototype.load=function(a){var b=new z,c=this.a,d=new qa(this.c.api,v(c),this.c.text),e=this.c.families;sa(d,e);var f=new ta(e);f.parse();x(c,d.d(),A(b));C(b,function(){a(f.aa,f.I,za)})};function W(a,b){this.a=a;this.c=b;this.X=[]}W.prototype.B=function(a){var b=this.a;return v(this.a)+(this.c.api||"//f.fontdeck.com/s/css/js/")+(b.k.location.hostname||b.F.location.hostname)+"/"+a+".js"};
    W.prototype.load=function(a){var b=this.c.id,c=this.a.k,d=this;b?(c.__webfontfontdeckmodule__||(c.__webfontfontdeckmodule__={}),c.__webfontfontdeckmodule__[b]=function(b,c){for(var g=0,l=c.fonts.length;g<l;++g){var h=c.fonts[g];d.X.push(new E(h.name,ea("font-weight:"+h.weight+";font-style:"+h.style)))}a(d.X)},y(this.a,this.B(b),function(b){b&&a([])})):a([])};function X(a,b){this.a=a;this.c=b}X.prototype.B=function(a){return(this.c.api||"https://use.typekit.net")+"/"+a+".js"};X.prototype.load=function(a){var b=this.c.id,c=this.a.k;b?y(this.a,this.B(b),function(b){if(b)a([]);else if(c.Typekit&&c.Typekit.config&&c.Typekit.config.fn){b=c.Typekit.config.fn;for(var e=[],f=0;f<b.length;f+=2)for(var g=b[f],l=b[f+1],h=0;h<l.length;h++)e.push(new E(g,l[h]));try{c.Typekit.load({events:!1,classes:!1,async:!0})}catch(k){}a(e)}},2E3):a([])};function Y(a,b){this.a=a;this.c=b}Y.prototype.B=function(a,b){var c=v(this.a),d=(this.c.api||"fast.fonts.net/jsapi").replace(/^.*http(s?):(\/\/)?/,"");return c+"//"+d+"/"+a+".js"+(b?"?v="+b:"")};
    Y.prototype.load=function(a){function b(){if(e["__mti_fntLst"+c]){var d=e["__mti_fntLst"+c](),g=[],l;if(d)for(var h=0;h<d.length;h++){var k=d[h].fontfamily;void 0!=d[h].fontStyle&&void 0!=d[h].fontWeight?(l=d[h].fontStyle+d[h].fontWeight,g.push(new E(k,l))):g.push(new E(k))}a(g)}else setTimeout(function(){b()},50)}var c=this.c.projectId,d=this.c.version;if(c){var e=this.a.k;y(this.a,this.B(c,d),function(c){c?a([]):b()}).id="__MonotypeAPIScript__"+c}else a([])};function Aa(a,b){this.a=a;this.c=b}Aa.prototype.load=function(a){var b,c,d=this.c.urls||[],e=this.c.families||[],f=this.c.testStrings||{},g=new z;b=0;for(c=d.length;b<c;b++)x(this.a,d[b],A(g));var l=[];b=0;for(c=e.length;b<c;b++)if(d=e[b].split(":"),d[1])for(var h=d[1].split(","),k=0;k<h.length;k+=1)l.push(new E(d[0],h[k]));else l.push(new E(d[0]));C(g,function(){a(l,f)})};var Z=new na(window);Z.q.t.custom=function(a,b){return new Aa(b,a)};Z.q.t.fontdeck=function(a,b){return new W(b,a)};Z.q.t.monotype=function(a,b){return new Y(b,a)};Z.q.t.typekit=function(a,b){return new X(b,a)};Z.q.t.google=function(a,b){return new ya(b,a)};var $={load:n(Z.load,Z)};"function"===typeof define&&define.amd?define(function(){return $}):"undefined"!==typeof module&&module.exports?module.exports=$:(window.WebFont=$,window.WebFontConfig&&Z.load(window.WebFontConfig));}());


// lib/gamewheel/assetDefinitions/urlAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.urlAsset'
).defines(function() { "use strict";

        ig.URLAsset = ig.Class.extend({

            url : "",

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.url = fixedAssetUrl;
            }

        });

    });

// lib/gamewheel/assetDefinitions/externalURLAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.externalURLAsset'
).defines(function() { "use strict";

        ig.ExternalURLAsset = ig.Class.extend({

            url : "",

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.url = assetFileDescription.location;
            }

        });

    });

// lib/gamewheel/assetDefinitions/dataBaseQueryAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.dataBaseQueryAsset'
).defines(function() { "use strict";

        ig.DataBaseQueryAsset = ig.Class.extend({

            databaseURL : '',

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.databaseURL = fixedAssetUrl;
            },

            getRows : function(rowAmount) {
                return this.__request(this.databaseURL + "?amount=" + rowAmount);
            },

            getRandomRows : function(rowAmount) {
                return this.__request(this.databaseURL + "?random=" + rowAmount);
            },

            __request : function(url) {
                var xhr = new XMLHttpRequest();
                xhr.open('GET', url, false);
                xhr.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');
                xhr.send();
                return JSON.parse(xhr.responseText);
            }

        });

    });

// lib/gamewheel/assetDefinitions/dummy/dataBaseQueryAssetDummy.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.dummy.dataBaseQueryAssetDummy'
).defines(function() { "use strict";

        /**
         * This is a data base query mockup.
         * To make this work, we need to supply an url to a .json file located in the media folder
         * */
        ig.DataBaseQueryAssetDummy = ig.Class.extend({
            databaseURL : "",
            internalDB   : [],

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.databaseURL = fixedAssetUrl;

                // Here we do a synchronous xhr request, to grab the data.
                var xhr = new XMLHttpRequest();
                xhr.open('GET', this.databaseURL, false);
                xhr.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');
                xhr.send();

                this.internalDB = JSON.parse(xhr.responseText);
            },

            getRows : function(rowAmount) {
                if(this.internalDB.length < rowAmount) {
                    rowAmount = this.internalDB.length;
                }

                return this.internalDB.slice(0, rowAmount);
            },

            getRandomRows : function(rowAmount) {
                if(this.internalDB.length < rowAmount) {
                    rowAmount = this.internalDB.length;
                }

                var usedIndices = {};
                var result = [];

                for(var i = 0; i < rowAmount; ++i) {
                    var randomIndex = Math.floor(Math.random() * this.internalDB.length);

                    // make sure to use an index, we did not use previously!
                    while(usedIndices[randomIndex] != null) {
                        randomIndex = Math.floor(Math.random() * this.internalDB.length);
                    }

                    usedIndices[randomIndex] = true;
                    result.push(this.internalDB[randomIndex]);
                }

                return result;
            },

            // this function parses a csv string into a json object
            __csvToJson : function(csv, returnAsObject) {
                var lines = csv.split("\r");
                if(lines.length == 1) {
                    lines = csv.split("\n");
                }

                var result = [];
                var headers = lines[0].split(";");

                for(var i = 1, l = lines.length; i < l; ++i) {
                    var obj = {};
                    var currentLine = lines[i].split(";");

                    for(var j = 0, jl = headers.length; j < jl; ++j) {
                        obj[headers[j]] = currentLine[j];
                    }

                    result.push(obj);
                }

                if(returnAsObject) {
                    return result;
                }

                return JSON.stringify(result);
            }
        });

    });

// lib/gamewheel/assetDefinitions/jsonAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.jsonAsset'
).defines(function() { "use strict";

        ig.JSONAsset = ig.Class.extend({

            url  : "",
            data : undefined,

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.url  = fixedAssetUrl;
                this.data = this.__request(this.url);
            },

            __request : function(url) {
                var xhr = new XMLHttpRequest();
                xhr.open('GET', url, false);
                xhr.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');
                xhr.send();
                return JSON.parse(xhr.responseText);
            }


        });

    });

// lib/gamewheel/assetDefinitions/scriptAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.scriptAsset'
).defines(function() { "use strict";

        ig.ScriptAsset = ig.Class.extend({

            url  : "",

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.url  = fixedAssetUrl;
                this.__load( this.url );
            },

            __load : function(url) {
                var script =  document.createElement('script');
                script.type = 'text/javascript';
                script.src = url;
                script.onload = function () {};

                document.getElementsByTagName( 'head' )[0].appendChild(script);
            }

        });

    });

// lib/gamewheel/assetDefinitions/cssAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.cssAsset'
).defines(function() { "use strict";

        ig.CSSAsset = ig.Class.extend({

            url  : "",

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.url  = fixedAssetUrl;
                this.__load( this.url );
            },

            __load : function(url) {
                var link =  document.createElement('link');
                link.type = 'text/css';
                link.href = url;
                link.rel = "stylesheet";
                link.onload = function () {};

                document.getElementsByTagName( 'head' )[0].appendChild(link);
            }

        });

    });

// lib/gamewheel/assetDefinitions/htmlAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.htmlAsset'
).defines(function() { "use strict";

        ig.HTMLAsset = ig.Class.extend({

            url  : "",
            assetName : "",

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.url  = fixedAssetUrl;
                this.assetName = assetName;
                this.__load( this.url );
            },

            __load : function(url) {
                var xhr = new XMLHttpRequest();
                xhr.open('GET', url, false);
                xhr.setRequestHeader('Content-Type', 'document; charset=UTF-8');
                xhr.send();

                var htmlContent =  document.createElement('div');
                htmlContent.id = this.assetName;
                htmlContent.innerHTML = xhr.responseText;
                document.getElementById('htmlAssets').appendChild(htmlContent);
            }

        });

    });

// lib/gamewheel/assetDefinitions/fontAsset.js
ig.baked=true;ig.module(
    'gamewheel.assetDefinitions.fontAsset'
).defines(function() { "use strict";

        ig.FontAsset = ig.Class.extend({

            url  : "",
            assetName : "",

            init: function (assetName, assetFileDescription, fixedAssetUrl) {
                this.url  = fixedAssetUrl;
                this.assetName = assetName;
                this.__load( this.url );
            },

            __load : function(url) {
                var style =  document.createElement('style');
                style.innerHTML = "@font-face { font-family: " + this.assetName + "; \nsrc : url('" + url + "'); }";
                document.getElementById( 'htmlAssets' ).appendChild(style);
            }

        });

    });

// lib/gamewheel/core/fileFactory.js
ig.baked=true;ig.module(
    'gamewheel.core.fileFactory'
).requires(
        'gamewheel.assetDefinitions.urlAsset',
        'gamewheel.assetDefinitions.externalURLAsset',
        'gamewheel.assetDefinitions.dataBaseQueryAsset',
        'gamewheel.assetDefinitions.dummy.dataBaseQueryAssetDummy',
        'gamewheel.assetDefinitions.jsonAsset',
        'gamewheel.assetDefinitions.scriptAsset',
        'gamewheel.assetDefinitions.cssAsset',
        'gamewheel.assetDefinitions.htmlAsset',
        'gamewheel.assetDefinitions.fontAsset'
    ).defines(function() { "use strict";

        ig.FileFactory = ig.Class.extend({

            //
            staticInstantiate: function () {
                // Only one singleton instance of the class is allowed.
                return ig.FileFactory.instance || null;
            },

            init: function () {
                // Set the singleton reference.
                ig.FileFactory.instance = this;

                // Rewire assets to be dummy objects, if we are running from a dev environment
                // This key gets removed when the server merges our config file.
                if(ig.config.global.hasOwnProperty('development') && ig.config.global.development == true) {
                    console.log("Init development file factory");
                    ig.DataBaseQueryAsset = ig.DataBaseQueryAssetDummy;
                }
            },

            createFile : function(assetName, assetFileDescription, fixedAssetUrl) {
                var assetType = assetFileDescription.type;

                if(assetType === 'url') {
                    return new ig.URLAsset(assetName, assetFileDescription, fixedAssetUrl);
                } else if (assetType === 'externalURL') {
                    return new ig.ExternalURLAsset(assetName, assetFileDescription, fixedAssetUrl);
                } else if(assetType === 'dataBaseQuery') {
                    return new ig.DataBaseQueryAsset(assetName, assetFileDescription, fixedAssetUrl);
                } else if(assetType === 'json') {
                    return new ig.JSONAsset(assetName, assetFileDescription, fixedAssetUrl);
                } else if(assetType === 'script') {
                    return new ig.ScriptAsset(assetName, assetFileDescription, fixedAssetUrl);
                } else if(assetType === 'css') {
                    return new ig.CSSAsset(assetName, assetFileDescription, fixedAssetUrl);
                } else if(assetType === 'html') {
                    return new ig.HTMLAsset(assetName, assetFileDescription, fixedAssetUrl);
                } else if(assetType === 'font') {
                    return new ig.FontAsset(assetName, assetFileDescription, fixedAssetUrl);
                }

                console.error('Undefined asset type', assetFileDescription);
                throw 'Could not interpret the asset type of (' + assetName + ').\nMake sure you are using a defined asset type.';
            }

        });

    });

// lib/gamewheel/core/content.js
ig.baked=true;ig.module(
    'gamewheel.core.content'
).requires(
    'impact.image',
    'impact.font',
    'impact.sound',
    'gamewheel.core.googleFonts',
    'gamewheel.core.fileFactory'
).defines(function() { "use strict";

        ig.Content = ig.Class.extend({

            images: {},
            fonts: {},
            sounds: {},

            staticInstantiate: function () {
                // Only one singleton instance of the class is allowed.
                return ig.Content.instance || null;
            },

            init: function () {
                // Set the singleton reference.
                ig.Content.instance = this;
            },

            load: function (assetData, mediaPath) {

                // Extract proper url from image model, base64 string, url or local asset path
                function fixAssetUrl(path) {

                    if (typeof path === 'object' && path && path.data) { // Image model

                        return path.data;
                    } else if (typeof path === 'string' && path.match(/^(data|http|https)\:/i) ) { // base64 string or url

                        return path;
                    }

                    // Local asset path
                    return mediaPath + path;
                }

                // Create image assets.
                var imageList = assetData['images'];
                if (imageList) {
                    for (var imageName in imageList) {
                        if (imageList.hasOwnProperty(imageName)) {
                            this.images[imageName] = new ig.Image(fixAssetUrl(imageList[imageName]));
                        }
                    }
                }

                // Create fonts.
                var fontList = assetData['fonts'];
                if (fontList) {
                    for (var fontName in fontList) {
                        if (fontList.hasOwnProperty(fontName)) {
                            this.fonts[fontName] = new ig.Font(fixAssetUrl(fontList[fontName]));
                        }
                    }
                }

                // Pre-load sounds.
                var soundList = assetData['sounds'];
                if(soundList){
                    for (var soundName in soundList) {
                        if (soundList.hasOwnProperty(soundName)) {
                            this.sounds[soundName] = new ig.Sound(fixAssetUrl(soundList[soundName]));
                        }
                    }
                }

                // Load google web fonts if required.
                if(assetData['googleWebFonts']) {
                    // assetData['googleWebFonts'] should be in the form {families: [ 'Open+Sans::latin' ]}
                    new ig.GoogleFonts(assetData['googleWebFonts']);
                }

                // Create file assets
                var fileList = assetData['files'];
                if(fileList) {
                    for (var fileName in fileList) {
                        if (fileList.hasOwnProperty(fileName)) {
                            var file = fileList[fileName];
                            var fixedAssetUrl = '';

                            if(file.hasOwnProperty("location")) {
                                fixedAssetUrl = fixAssetUrl(file.location);
                            } else {
                                throw fileName + ' has no location defined in the config!';
                            }

                            if(typeof ig.fileFactory === "undefined" ) {
                                ig.fileFactory = new ig.FileFactory();
                                this.files = this.files || {};
                            }
                            this.files[fileName] = ig.fileFactory.createFile(fileName, file, fixedAssetUrl);
                        }
                    }
                }

            },

            imageNamed: function (name) {
                return this.images[name];
            },

            fontNamed: function (name) {
                return this.fonts[name];
            },

            soundNamed:function (name) {
                return this.sounds[name];
            },

            playSoundNamed: function (name) {
                if(this.sounds[name]) {
                    this.sounds[name].play();
                    return true;
                }
                return false;
            }

        });

        // We need to init the global instance and load content so that it is
        // usable by other classes as part of their definitions (pre call to ig.main).
        ig.content = new ig.Content();
        // Note that ig.config.mediaPath should only exist when loading
        // from a template (not compiled). Normally we default to media/.
        ig.content.load(ig.config['assets'], ig.config.mediaPath || 'media/');

    });



// lib/gamewheel/core/localization.js
ig.baked=true;ig.module(
    'gamewheel.core.localization'
).defines(function() {
        "use strict";

        // TODO: Localization pulls text strings from config until we implement this properly
        
        ig.Localization = ig.Class.extend({
            
            _langData: undefined,
            currentLanguage: 'en',
            languageDefault: 'en',

            staticInstantiate: function () {
                // Only one singleton instance of the class is allowed.
                return ig.Localization.instance || null;
            },
            
            init: function () {
                // Set the singleton reference.
                ig.Localization.instance = this;
                this._langData = ig.config['assets']['text'] || {};
            },
            
            selectLanguage: function() {
                
            },

            textForKey: function( key, defaultText ){
                if(!ig.ua.mobile && this._langData[key + 'Desktop'] !== undefined) {
                    return this._langData[key + 'Desktop'];
                } else if(this._langData[key] !== undefined){
                    return this._langData[key];
                } else {
                    return defaultText || '';
                }
            }

        });
        

    });


/**
 * Optionally displays orientation, splash and help screens.
 */



// lib/gamewheel/scenes/basicScene.js
ig.baked=true;ig.module(
    'gamewheel.scenes.basicScene'
).requires(
    'impact.game',
    'gamewheel.core.game'
).defines(function(){ "use strict";
        
        ig.BasicScene = ig.Game.extend({

            onSceneStart : undefined,
            onSceneEnd: undefined,
            elements: undefined,

            init: function () {
                this.bindInput();
                this.spawnElements();

                if( this.onSceneStart ) {
                    ig.actions.perform(this.onSceneStart);
                }
            },
            
            bindInput: function () {
                ig.input.bind(ig.KEY.SPACE, 'space');
                ig.input.bind(ig.KEY.MOUSE1, 'click');
            },
            
            spawnElements: function() {
                // Add elements.
                if(this.elements) {
                    for(var i = 0; i < this.elements.length; i++) {
                        var e = this.spawnConfigEntity(this.elements[i]);
                        this.namedEntities[ this.elements[i] ] = e;
                    }
                    delete this.elements;
                }
            },
            
            endScene: function() {
                if(this.onSceneEnd) {
                    ig.actions.perform(this.onSceneEnd);
                }  
            }

        });


    });



// lib/gamewheel/core/sessionStats.js
ig.baked=true;ig.module(
    'gamewheel.core.sessionStats'
).defines(function() { "use strict";

        // Game stats tracking system.
        ig.SessionStats = ig.Class.extend({
            
            sessionBase: {
                startTime: 0,
                time: 0,
                score: 0,
                died: false,
                resultString: '',
                resultID: null,

                toString: function() {
                    var summary = "Time: " + this.time + " Score: " + this.score + " Died: " + this.died;
                    if(this.resultID) {
                        summary += " Result: " + this.resultID;
                    }
                    return summary;
                },
                
                complete: function(){
                    // Set the duration if it hasn't been set manually/already set.
                    if(this.time === 0) {
                        this.time = (Date.now() - this.startTime) / 1000;
                    }
                }
            },

            currentSession: undefined,
            sessions: [],

            staticInstantiate: function () {
                // Only one singleton instance of the class is allowed.
                return ig.SessionStats.instance || null;
            },
            
            init: function() {
                this.newSession();
                // Set the singleton reference.
                ig.SessionStats.instance = this;
            },

            newSession: function() {
                if(this.currentSession) {
                    // Push the completed session to the list of played sessions.
                    this.sessions.push(this.currentSession);
                }
                // Copy the session base object as a new session.
                this.currentSession = ig.copy(this.sessionBase);
                // Set the start time so we can easily track duration.
                this.currentSession.startTime = Date.now();
                // Return the new session object.
                return this.currentSession;
            }

        });

    });




// lib/gamewheel/core/output.js
ig.baked=true;ig.module(
    'gamewheel.core.output'
).requires(
    'impact.impact'
).defines(function(){ "use strict";

        ig.Output = ig.Class.extend({

            // Output functions.
            sessionUpdate : function (sessionData) {}, // Called each frame of the game.
            gameEvent	  : function (event, data) {}, // Game events that aren't covered by sessionUpdate.
            ready		  : function () {
                console.log('ready');
            }, 		   // Called when the game is loaded and ready to go.
            over          : function (sessionData) {
                console.log('over', sessionData);
            }, // Called when the game exits.
            sceneLoaded : function (sceneName) {
                console.log("scene loaded", sceneName);
            },


            staticInstantiate: function () {
                // Only one singleton instance of the class is allowed.
                return ig.Output.instance || null;
            },


            init: function() {

                // Set the singleton reference.
                ig.Output.instance = this;

                var normalize = function (fn) {
                    // If it doesn't exist or is not a function, return dummy function
                    return (fn && typeof(fn) === 'function') ? fn : function(){};
                };
                
                // Make sure methods are available to be called, even if they do nothing.
                var settings = ig.config.global.output;
                
                if(settings) {
                    this.sessionUpdate  = normalize(settings['sessionUpdate']);
                    this.gameEvent      = normalize(settings['gameEvent']);
                    this.ready          = normalize(settings['ready']);
                    this.over           = normalize(settings['over']);
                    this.sceneLoaded    = normalize(settings['sceneLoaded']);
                }
            }
            
        });


    });

/**
 * Runtime API utils module definition
 */

/**
 * Settings module definition
 */


// lib/gamewheel/core/settings.js
ig.baked=true;ig.module('gamewheel.core.settings').defines(function() { "use strict";

        /**
         * Module exports
         */
        ig.settings = ig.settings || {};
        ig.settings.tokens = {};

        // Get the game instance from url (could not be there is a masked url)
        var match = /instances\/[^/]*\/?(\d+)/.exec(window.location.pathname),
              instanceId = match && match.length && match[1] && parseInt(match[1], 10);

        // We got the instance id in the url
        if (instanceId) {

            // Set instance
            ig.settings.instance = {
                id: instanceId
            };

        // No instance is, is it a masked url??
        } else {

            // Set instance masked url
            ig.settings.instance = {
                url: window.location.host + window.location.pathname
            };
        }
        
        // Set API anyway
        setApi(window.location.host);

        /**
         * Api settings Helper function
         *
         * @param host
         */
        function setApi(_host) {

            // Get url matches for runtime and initialize variables
            var PATTERN_LOCAL   = /^(127.0.0.[0-9]+|localhost)(?::([0-9]+))?$/;
            var PATTERN_DOMAIN  = /^((([a-z]+)-)?[a-z]+)\.(.*)$/;
            
            var domainMatch     = _host.match(PATTERN_DOMAIN);
            var localMatch      = _host.match(PATTERN_LOCAL);
            
            if(!localMatch && !domainMatch){
                console.error("Domain ("+ _host +") not supported for Runtime API");
                return;
            }

            if(!localMatch){
                var host                = domainMatch[1];
                var environmentPrefix   = domainMatch[2] || '';
                var environment         = domainMatch[3] || 'live';
                var domain              = domainMatch[4];
                var apiHost             = environmentPrefix + 'api-runtime';
                
                ig.settings.api = {
                    endpoint: 'https://' + [apiHost, domain].join('.'),
                    key: 'gw_rocks'
                };
            }
            else {
                ig.settings.api = {
                    run : function(event, data){
                        console.log(event, data);
                    }
                };
                ig.settings.tokens = 'local';
            }
        }

    });

/**
 * Runtime API utils module definition
 */


// lib/gamewheel/helpers/storage.js
ig.baked=true;ig.module('gamewheel.helpers.storage').defines(function() { "use strict";

        if(!localStorage) { localStorage = { getItem : function(){}, setItem : function(){} } }
        
        var storageName             = ig.config.global.storageName || 'gamewheel';

        ig.storage                  = getStorageDate;
        ig.storage.set              = setStorageData;
        
        ig.storage.session          = getSession;
        ig.storage.session.add      = addSession;
        
        ig.storage.gameState        = getGameState;
        ig.storage.gameState.set    = setGameState;
        
        ig.storage.form             = getFormState;
        ig.storage.form.set         = setFormState;

        ig.storage.path             = getPath;
        ig.storage.path.set         = setPath;


        if(!getStorageDate()){ setStorageData({id:Math.random().toString(36).substring(7)}); }

        function getStorageDate() {
            return JSON.parse(localStorage.getItem(storageName));
        }

        function setStorageData (_data) {
            localStorage.setItem(storageName, JSON.stringify(_data));
        }

        function getSession (){
            var storageData = getStorageDate();
            return storageData.currentSession || null;
        }

        function addSession (_session) {
            var storageData             = getStorageDate();
                storageData.sessions    = storageData.sessions || [];

            var lastSession = storageData.currentSession || null;

            if(lastSession){ storageData.sessions.push(lastSession); }

            storageData.currentSession = _session;
            setStorageData(storageData);
        }

        function setGameState (_game, _state){
            var storageData = getStorageDate();
            storageData.games = storageData.games || {};
            storageData.games[_game] = _state;
            setStorageData(storageData);
        }

        function getGameState (_game){
            var storageData = getStorageDate();
            return (storageData.games || {})[_game] || null;
        }

        function setFormState (_state){
            var storageData = getStorageDate();
            storageData.form = _state || {};
            setStorageData(storageData);
        }

        function getFormState (){
            var storageData = getStorageDate();
            return storageData.form || {};
        }

        function setPath (){
            var storageData = getStorageDate();
            storageData.currentPath = ig.config.global.path;
            setStorageData(storageData);
        }

        function getPath (){
            var storageData = getStorageDate();
            return storageData.currentPath;
        }

    });

// lib/gamewheel/helpers/apiUtil.js
ig.baked=true;ig.module('gamewheel.helpers.apiUtil').requires(
    'gamewheel.core.settings',
    'gamewheel.helpers.storage'
).defines(function() { "use strict";

        /**
         * Normalizes a function (mainly for callbacks support)
         *
         * @param fn
         * @returns function
         */
        function normalizeFunction(fn) {

            // If it doesn't exist or is not a function, return dummy function
            return (fn && typeof(fn) === 'function') ? fn : function(){};
        }

        /**
         * AJAX Helper function
         *
         * @param path
         * @param method
         * @param headers
         * @param body
         * @param fn
         */
        function xhr(path, method, headers, body, fn) {

            // Normalize callback
            fn = normalizeFunction(fn);

            // Check api settings, therwise return error
            if (!ig.settings.api) {
                return fn(500, {error: "Runtime API settings not defined"});
            }

            // Create request object
            var xhr = new XMLHttpRequest();

            // Open connection
            xhr.open(method, ig.settings.api.endpoint + path);

            // Our APIs are always JSON-based
            xhr.setRequestHeader('Content-Type', 'application/json');

            // Pass always the api_key
            xhr.setRequestHeader('x-gw-api-rt-key', ig.settings.api.key);

            // Pass always the api_token (if any)
            // FIXME: Remove entirely from local storage if no issues ... it makes it inconsistent
            xhr.setRequestHeader('x-gw-api-rt-token', /*localStorage && localStorage.getItem('gw-api-rt-token') || */ig.settings.tokens.api_runtime);

            // Set custom headers, if any
            if (headers) {
                Object.keys(headers).forEach(function (key) {
                    xhr.setRequestHeader(key, headers[key]);
                });
            }

            // Set response listener
            xhr.onreadystatechange = function() {

                // Init parsed body
                var body;

                // Check if response received
                if (xhr.readyState === XMLHttpRequest.DONE) {

                    // Try to parse body, our APIs always return either JSON or text error
                    try {
                        body = JSON.parse(xhr.responseText);

                        // Otherwise treat response text as error
                    } catch(e) {
                        body = { error: xhr.responseText };
                    }

                    // Call handler with response status and body
                    fn(xhr.status, body);
                }
            };

            // Send request with body, if any (and stringified)
            xhr.send(JSON.stringify(body));
        }

        /**
         * Authentication against runtime API
         *
         * @param fn
         */
        function authenticate(fn) {

            // Normalize callback
            fn = normalizeFunction(fn);

            // Request body
            var body = { instance_id: ig.settings.instance.id, masked_url: ig.settings.instance.url };

            // Perform auth login request
            xhr('/auth/login', 'POST', null, body, function(status, body) {

                // Error in request, falsy callback
                if (status !== 200) {
                    return fn(false);
                }

                // FIXME: Remove entirely from local storage if no issues ... it makes it inconsistent
                // Store token and instance data
                // if (localStorage) {
                //     try {
                //         localStorage.setItem('gw-api-rt-token', body.token);
                //     } catch (e) {
                //         console.log('Browser mode does not support storage for item');
                //     }
                // }

                // Store in game anyway
                ig.settings.tokens.api_runtime = body.token;
                ig.storage.session.add(body.token);

                // Store instance data in config
                ig.settings.instance = body.instance;

                // Callback truly
                fn(true);
            });
        }

        /**
         * Create session for events in runtime API
         *
         * @param fn
         */
        function createSession(fn) {

            // Normalize callback
            fn = normalizeFunction(fn);

            // Request body
            // TODO: Should this data contain anything else???
            var body = {};

            // Perform auth login request
            xhr('/game_sessions', 'POST', null, body, function(status, body) {

                // Error in request, falsy callback
                if (status !== 200 && status !== 201) {
                    return fn(false);
                }

                // Store session token in game
                ig.settings.tokens.api_runtime_session = body.token;

                // Callback truly
                fn(true);
            });
        }

        /**
         * Sends an event to Runtime API
         *
         * @param event
         * @param data
         */
        function sendEvent(event, data) {

            // Request body
            var body = {
                session_token: ig.settings.tokens.api_runtime_session,
                name: event,
                data: data
            };

            if(ig.settings.api.run ) {
                ig.settings.api.run(body.event, body.data);
            }
            else {
                // Perform auth login request
                xhr('/game_events', 'POST', null, body, function(status, body) {
                    // Just log the result
                    //console.log(status, body);
                });
            }

        }

        /**
         * Module exports
         *
         * @type {{xhr: xhr, authenticate: authenticate, createSession: createSession, sendEvent: sendEvent}}
         */
        ig.apiUtil = {
            xhr: xhr,
            authenticate: authenticate,
            createSession: createSession,
            sendEvent: sendEvent,
            normalizeFunction: normalizeFunction
        };
    });

// lib/gamewheel/api_analytics/analytics.js
ig.baked=true;ig.module(
    'gamewheel.api_analytics.analytics'
).requires(
    'gamewheel.helpers.apiUtil'
).defines(function(){ "use strict";

        var apiAnalytics = {
            
            enabled: false,
            waiting: false,
            eventStack : [],

            init: function(fn) {
                fn = ig.apiUtil.normalizeFunction(fn);
                var self = this;
                ig.apiUtil.authenticate(function(success) {
                    if (success) {
                        ig.apiUtil.createSession(function (success) {
                            self.enabled = success;
                            fn();
                        });
                    } else {
                        fn();
                    }
                });
                
            },

            sendEvent: function(event, data) {
                
                var self = this;

                if(this.waiting) {
                    this.eventStack.push({
                        event : event,
                        data : data
                    });   
                }

                else if(!this.enabled){
                    this.eventStack.push({
                        event : event,
                        data : data
                    });
                    this.waiting = true;
                    this.init(function(){
                        if(self.enabled){
                            var eventCache;
                            while (self.eventStack.length){
                                eventCache = self.eventStack.shift();
                                ig.apiUtil.sendEvent(eventCache.event, eventCache.data);
                            }    
                        }
                        self.waiting = false;
                    });
                }

                else if (this.enabled) {
                    ig.apiUtil.sendEvent(event, data);
                }
            }
        };

        ig.apiAnalytics = ig.apiAnalytics || apiAnalytics;
    });


// lib/gamewheel/core/analytics.js
ig.baked=true;ig.module(
    'gamewheel.core.analytics'
).requires(
    'gamewheel.scenes.basicScene',
    'gamewheel.core.sessionStats',
    'gamewheel.core.output',
    'gamewheel.api_analytics.analytics'
).defines(function(){ "use strict";

        // Text strings for analytics events.
        ig.ANALYTICS_EVENT = {
            PAGE_LOADED                 : "pageLoaded",
            GAME_LOADED                 : "gameLoaded",
            PRESSED_FOOTER              : "pressedFooter",
            PRESSED_HELP                : "pressedHelp",
            PRESSED_SHARE               : "pressedShare",
            PRESSED_CTA                 : "pressedCTA",
            PRESSED_REPLAY              : "pressedReplay",
            GAME_STARTED                : "gameStarted",
            GAME_COMPLETE               : "gameCompleted",
            EXIT_GAME                   : "exitGame",
            FACEBOOK_SHARE_INIT_FAILED  : "facebookInitFailed",
            FACEBOOK_SHARE_INIT         : "shareOnFacebookInitiated",
            FACEBOOK_SHARE_COMPLETE     : "shareOnFacebookCompleted",
            FACEBOOK_SHARE_CANCELED     : "shareOnFacebookCanceled",
            SUBMITTED_LEAD_FORM         : "playerSubmittedForm"
        };

        ig.IAnalytics = ig.Class.extend({
            submitEvent: function(event, data) { throw 'Not implemented'; }
        });
        
        ig.Analytics = ig.IAnalytics.extend({
            
            disable                  : false,
            sessionSetup             : false,
            playId                   : null,
            logSubmits               : false,
            didSubmitGameLoadedEvent : false,
            
            staticInstantiate: function () {
                // Only one singleton instance of the class is allowed.
                return ig.Analytics.instance || null;
            },
            
            setupComplete: function() {
                this.log('Analytics setup complete. Disabled: ' + this.disable + ', Session OK: ' + this.sessionSetup);
                this.submitEvent(ig.ANALYTICS_EVENT.PAGE_LOADED);
            },

            init: function() {
                
                // Set the singleton reference.
                ig.Analytics.instance = this;
                
                // Note that we default to enabling analytics when information is missing.
                if(window['disableAnalytics'] === true) {
                    this.disable = true;
                } else if(ig.config.global['analytics'] && ig.config.global['analytics']['gw']) {
                    this.disable = ig.config.global['analytics']['gw']['disable'] === true;
                }

                var self = this;
                if(!this.disable) {
                    self.sessionSetup = true;
                    self.setupComplete();
                } else {
                    self.setupComplete();
                }
            },
            
            // Removing support for this method.
            submitGameSession: function(){
                // Throw an error to make it obvious if a game is using this method.
                throw 'Analytics.submitGameSession is no longer supported';
            },

            submitEvent: function(event, data) {

                // Create object with basic event data.
                var dataObject = {
                    playId: this.playId,
                    time: Date.now()
                };
                
                // Merge in any extra provided data.
                if(data && typeof data === 'object'){
                    ig.merge(dataObject, data);
                }
                
                // Log the submit.
                this.log("Analytics Submit: " + event + " " + JSON.stringify(dataObject));

                // Send the event.
                if(this.sessionSetup && !this.disable) {
                    ig.apiAnalytics.sendEvent(event, data);
                }
            },
            
            log: function(message) {
                if(this.logSubmits) {
                    console.log(message);
                }
            },

            startGame: function() {
                this.playId = Math.random().toString(36).substring(7, 12);
                this.submitEvent(ig.ANALYTICS_EVENT.GAME_STARTED);
            },

            endGame: function() {
                var playData = {};
                var cs = ig.sessionStats.currentSession;
                if(cs) {
                    playData.duration  = cs.time;
                    playData.score     = cs.score;
                    playData.died      = cs.died;
                }
                this.submitEvent(ig.ANALYTICS_EVENT.GAME_COMPLETE, playData);
            }
            
        });

        ig.System.inject({
            startRunLoop: function() {
                if(ig.analytics && !ig.analytics.didSubmitGameLoadedEvent) {
                    ig.analytics.didSubmitGameLoadedEvent = true;
                    ig.analytics.submitEvent(ig.ANALYTICS_EVENT.GAME_LOADED);
                }
                this.parent();
            }
        });
        
        // TODO: This stuff should be inserted into a generic game scene.
        ig.BasicScene.inject({
            autoTrackGameAnalytics: false,

            init: function () {
                this.parent();
                if(this.autoTrackGameAnalytics) {
                    ig.analytics.startGame();
                }
            },

            ending: function() {
                if(this.autoTrackGameAnalytics) {
                    ig.analytics.endGame();
                }
                this.parent();
            }

        });


    });


// lib/gamewheel/core/actions.js
ig.baked=true;ig.module(
    'gamewheel.core.actions'
).requires(
    'gamewheel.core.analytics'
).defines(function() { "use strict";

        ig.Actions = ig.Class.extend({
            
            submitAnalyticsEvent: function(event) {
                ig.analytics.submitEvent(event);
            },
            
            goToScene: function(sceneName) {
                ig.system.setScene(sceneName);
                ig.output.sceneLoaded(sceneName);
            },

            performActionBasedOnDevice: function(args) {
                // Expect args to be an object containing a list of action objects.
                // The key for each object should be one of 'default', 'iOS', 'android' or 'mobile'.
                // Example: {'mobile': <action object>, 'default': <action object>}

                // Try get default action.
                var action = args['default'];

                if(ig.ua.iOS && args['iOS']) {
                    action = args['iOS'];
                } else if(ig.ua.android && args['android'] ) {
                    action = args['android'];
                } else if (ig.ua.mobile && args['mobile'] ) {
                    action = args['mobile'];
                }

                if(action) {
                    ig.actions.perform(action);
                }
            },

            performActionBasedOnScore: function(args) {
                // Expect args to be an array with a list of objects stating the required minimum score
                // to access, along with the action to perform.
                // I.e: [{requiredScore: <number>, action: <action object>}].
                // The requiredScore value can be set to null to use that action as the default.

                var playerScore = ig.sessionStats.currentSession.score;
                var pickedSet = null;
                var rs = null;

                for(var i = 0; i < args.length; i++) {
                    var set = args[i];
                    if(set.requiredScore === undefined || set.requiredScore === null) {
                        if(!pickedSet) {
                            // Default action case. Required score not defined and we have yet to pick an action.
                            pickedSet = set;
                        }
                    } else if(playerScore >= set.requiredScore) {
                        console.log(set.requiredScore);
                        // Can access this set, but first check we haven't already picked a set with a higher score.
                        if(!pickedSet || rs === null || set.requiredScore > rs) {
                            // This set has the highest required score we can access so far.
                            rs = set.requiredScore;
                            pickedSet = set;
                        }
                    }
                }

                if(pickedSet && pickedSet.action) {
                    // Perform the action.
                    ig.actions.perform(pickedSet.action);
                } else {
                    console.error('No accessible action to perform based on score.');
                }
            },

            goToSceneBasedOnScore: function(args) {
                // Expect args to be an object with a list of scene names and the required minimum score 
                // to access that scene. I.e: {<scene name>: <required score>}. The required score value
                // can be set to null to use that scene as the default fallback scene.
                var playerScore = ig.sessionStats.currentSession.score;
                var rs = null;
                var scene = null;
                
                for(var sceneName in args) {
                    if(args.hasOwnProperty(sceneName)) {
                        var requiredScore = args[sceneName];

                        if(requiredScore === undefined || requiredScore === null) {
                            if(rs === null) {
                                // Default scene case. Required score not defined and we have yet to pick a scene.
                                scene = sceneName;
                            }
                        } else if(playerScore >= requiredScore && (rs === null || requiredScore > rs)) {
                            // Scene has highest required score we can access so far.
                            rs = requiredScore;
                            scene = sceneName;
                        }
                    }
                }
                
                if(scene) {
                    // Switch to scene.
                    ig.system.setScene(scene);
                } else {
                    console.error('No accessible scene.');
                }
            },
            
            endScene: function() {
                ig.game.endScene();  
            },

            startScene : function( sceneName ) {
                ig.output.sceneLoaded( sceneName );
            },

            navigateToUrlNewWindow: function(url) {
                // Search for and insert session token.
                url = url.replace('%SESSION%', ig.settings.tokens.api_runtime_session || 'none');

                // Make sure we have http:// or https://
                if (!/^https?:\/\//i.test(url)) {
                    url = 'http://' + url;
                }

                // Send analytics event.
                ig.analytics.submitEvent(ig.ANALYTICS_EVENT.EXIT_GAME);
                
                window.open(url, '_blank');
            },

            navigateToUrl: function(url) {
                // Search for and insert session token.
                url = url.replace('%SESSION%', ig.settings.tokens.api_runtime_session  || 'none');

                // Make sure we have http:// or https://
                if (!/^https?:\/\//i.test(url)) {
                    url = 'http://' + url;
                }

                // Send analytics event.
                ig.analytics.submitEvent(ig.ANALYTICS_EVENT.EXIT_GAME);
                // Add slight delay to allow any analytics events to be sent.
                setTimeout(function () {
                    document.location.href = url;
                }, 200);
            },
            
            _facebookInit: false,
            
            shareOnFacebook: function(args) {

                if(!window.FB){
                    // Failed to download facebook script for some reason.
                    ig.analytics.submitEvent(ig.ANALYTICS_EVENT.FACEBOOK_SHARE_INIT_FAILED);
                    return;
                }
                
                // Announce that a facebook share is being attempted.
                ig.analytics.submitEvent(ig.ANALYTICS_EVENT.FACEBOOK_SHARE_INIT);
                
                
                args = args || {};
                if(typeof args == 'string') {
                    args = {
                        shareUrl: args, // URL to share. Backward compatible support.
                        redirectUrlSuccess: undefined, // URL to redirect to if share is completed.
                        redirectUrlCanceled: undefined // URL to redirect to if share is canceled.
                    }
                }

                var _self = this;

                if(!this._facebookInit) {
                    // Initialize facebook with app id.
                    window.FB.init({
                        appId : '1607515912863171',
                        xfbml : true,
                        version : 'v2.4'
                    });
                    this._facebookInit = true;
                }
                
                window.FB.ui({
                    method:  'share',
                    href: (args.shareUrl || window.location.href), // Share game url by default.
                    quote : ( typeof args.quote == 'function' ? args.quote() : null || null ),
                    hashtag : ( args.hashtag || null )
                }, function(response) {
                    if (response) { // shared
                        ig.analytics.submitEvent(ig.ANALYTICS_EVENT.FACEBOOK_SHARE_COMPLETE);

                        if(args.redirectUrlSuccess) {
                            _self.navigateToUrl(args.redirectUrlSuccess);
                        }
                    }
                    else { // not shared
                        ig.analytics.submitEvent(ig.ANALYTICS_EVENT.FACEBOOK_SHARE_CANCELED);

                        if(args.redirectUrlCanceled) {
                            _self.navigateToUrl(args.redirectUrlCanceled);
                        }
                    }
                });
                
            },
            

            shareOnTwitter: function(message) {
                message = message.replace('%URL%', window.location);
                var url = "https://twitter.com/intent/tweet?text=" + encodeURI(message);
                this.navigateToUrl(url)
            },
            
            perform: function( definition, trigger ) {
                if(definition && !(definition instanceof Array)) {
                    definition = [definition];
                }
                
                if(definition) {
                    for (var i = 0; i < definition.length; i++) {
                        if (definition[i] && definition[i].hasOwnProperty('action')) {
                            // Submit analytics event if present.
                            if(definition[i].analyticsEvent) {
                                ig.analytics.submitEvent(definition[i].analyticsEvent);
                            }

                            if(typeof definition[i].action === 'function') {
                                // Run the action as a function.
                                definition[i].action( definition[i]['args'], trigger );
                            } else if(ig.actions[definition[i].action]) {
                                // Run named action.
                                ig.actions[definition[i].action](definition[i]['args'], trigger);
                            }
                        }
                    }
                }
            }
            
        });
        
    });

(function(d, s, id){
    var js, fjs = d.getElementsByTagName(s)[0];
    if (d.getElementById(id)) {return;}
    js = d.createElement(s); js.id = id;
    js.src = "//connect.facebook.net/en_US/sdk.js";
    fjs.parentNode.insertBefore(js, fjs);
}(document, 'script', 'facebook-jssdk'));


// lib/gamewheel/core/gamewheel.js
ig.baked=true;ig.module(
    'gamewheel.core.gamewheel'
).requires(
    'impact.impact',
    'impact.system',
    'gamewheel.core.game',
    'gamewheel.core.content',
    'gamewheel.core.localization',
    'gamewheel.core.actions',
    'gamewheel.core.analytics',
    'gamewheel.core.sessionStats',
    'gamewheel.core.fileFactory',
    'gamewheel.core.output'
).defines(function(){ "use strict";


        // Override entry point method.
        ig.main = function( canvasId, gameClass, fps, width, height, scale, loaderClass ) {

            // Setup standard impact systems.
            ig.system = new ig.System( canvasId, fps, width, height, scale || 1 );

            ig.input = new ig.Input();

            ig.soundManager = new ig.SoundManager();

            ig.music = new ig.Music();

            // Setup media path.
            ig.mediaPath = ig.mediaPath || 'media/';

            // Setup output methods.
            ig.output = new ig.Output();

            // Create a global FileFactory object
            ig.fileFactory = new ig.FileFactory();

            // Load global action system.
            ig.actions = new ig.Actions();

            // Load global language system.
            ig.localization = new ig.Localization();

            // Load global session stats system.
            ig.sessionStats = new ig.SessionStats();

            // Load global analytics system.
            ig.analytics = new ig.Analytics();

            // Init content library.
            ig.content = new ig.Content();

            // TODO: fix this. Calling the load function twice or more is really bad...........
            // Trigger init of assets.
            // ig.content.load(ig.config['assets'], ig.mediaPath);

            // Need to load required assets for prefabs from gamewheel lib.
            ig.content.load(ig.Prefabs.collectAssets(), (ig.baked ? ig.mediaPath : ig.Prefabs.gwMediaPath) || 'media/');

            ig.ready = true;

            // Start with an empty scene when game class is not provided.
            gameClass = gameClass || ig.EmptyGame;

            // Load the game.
            var loader = new (loaderClass || ig.Loader)( gameClass, ig.resources );
            loader.load();
        };

        // Add method to ig that generates the definition for a scene.
        ig.getSceneClass = function(sceneName) {
            var sceneConfigData = ig.config['scenes'][sceneName];

            if(ig[sceneConfigData['sceneType']]) {
                return ig[sceneConfigData['sceneType']].extend(sceneConfigData['settings'] || {});
            }
        };

        // Convenience method that returns the boot scene.
        ig.getBootScene = function() {
            // Check url params for boot scene override.
            var boot = /boot=([^&]*)/.exec(window.location.search);
            boot = boot && boot[1];

            if(!boot || !ig.config['scenes'][boot]) {
                // NOTE: Opportunity here to default to first scene in list if bootScene not specified.
                boot = ig.config.global['bootScene'];
            }
            
            // Return the boot scene.
            return ig.getSceneClass(boot);
        };

        ig.System.inject({
            // Inject setScene to ig.system. This is basically like setGame but it accepts the name of a scene defined in the config.
            setScene: function(sceneName) {
                ig.game.ending();
                this.setGame(ig.getSceneClass(sceneName))
            }
        });

        // Pseudo Game class that clears the canvas and then does stops the game loop.
        // TODO: I think there's a better way to do this.
        ig.EmptyGame = ig.Class.extend({
            run: function () {
                ig.system.clear();
                ig.system.stopRunLoop();
            }
        });


        ig.start = function () {
            ig.system.setScene(ig.config.global['bootScene']);
        };

        // TODO: Need to implement this.
        ig.reload = function () {
            // Need to make sure the config is reloaded.

        };

        ig.exit = function () {
            // Alert of ending session.;
            ig.output.over(ig.game.session);

            // Empty game scene will stop the game loop. TODO this can surely be done better.
            ig.system.setGame(ig.EmptyGame);
        };

        ig.reloadImages = function () {
            // TODO: Is it possible to prevent redraw until all images are reloaded?
            // Do: ig.system.stopRunLoop(); Then need call back to call ig.system.startRunLoop(); when done loading.
            ig.Image.reloadCache();
        };


        // TODO: Revisit this.
        ig.reloadAssets = function () {
            ig.content.load(ig.config['assets'], ig.mediaPath);
        };

    });





        


// lib/gamewheel/core/gwLoader.js
ig.baked=true;ig.module(
    'gamewheel.core.gwLoader'
).requires(
        'impact.loader'
).defines(function(){ "use strict";
        
        // Subclass the default loader
        ig.GwLoader = ig.Loader.extend({


            _drawStatus: 0,
            /* arc per second */
            _wheelSpeed: 3*Math.PI,
            _startTime: -1,
            /* time since first draw in seconds */
            _showTime: 0,
            _bgColorHex: window.ImpactMixin.config.global['keyColor'] || '#FFFFFF',
            _themeThreshold: 0.7,
            _colorSettings: {
                'dark': {
                    'base': '#44423f'
                },
                'light': {
                    'base': '#FFFFFF'
                }
            },
            _theme: 'dark',

            init: function( gameClass, resources ) {

                // set theme, by measuring the background intensity
                var bgColorHSL = this._rgb2hsl(this._bgColorHex);
                this._theme = (bgColorHSL[2] < this._themeThreshold)? 'light' : 'dark';

                // draw settings, adapted to screen dimensions
                this.w = ig.system.realWidth;
                this.h = ig.system.realHeight;
                this.logo_src_w = 250;
                this.logo_src_h = 44;
                this.logo_target_w = this.w * 0.43;
                this.progressBar_target_w = 0.8 * this.logo_target_w;
                this.logo_scale = this.logo_target_w/this.logo_src_w;
                this.logo_margin_x = (this.w - this.logo_target_w)/2;
                this.logo_margin_y = (this.h - this.logo_src_h*this.logo_scale)/2;
                this.progressBar_margin_x = (this.w - this.progressBar_target_w)/2;
                this.progressBar_margin_y = this.logo_margin_y + (this.logo_src_h*this.logo_scale) + (this.logo_src_h*this.logo_scale)/3;
                
                this.parent(gameClass, resources );  
                
            },

            load: function() {
                ig.system.clear( this._bgColorHex );

                if( !this.resources.length ) {
                    this.end();
                    return;
                }

                for( var i = 0; i < this.resources.length; i++ ) {
                    this.loadResource( this.resources[i] );
                }
                this._intervalId = setInterval( this.draw.bind(this), 16 );
            },

            /**
             * Main draw function.
             *
             * @override
             */
            draw: function() {
                
                
                // set time (used for animation control)
                if(this._startTime == -1)
                    this._startTime = Date.now();
                this._showTime = (Date.now() - this._startTime) / 1000.0;

                
                this._drawStatus += (this.status - this._drawStatus)/5;
                
                //
                // clear context
                ig.system.context.clearRect(0, 0, this.w, this.h);
                
                //
                // draw bg with key color;
                ig.system.context.fillStyle = this._bgColorHex;
                ig.system.context.beginPath();
                ig.system.context.rect(0,0,this.w,this.h);
                ig.system.context.fill();
                ig.system.context.closePath();


                //
                // draw logo
                ig.system.context.save();
                ig.system.context.translate(this.logo_margin_x, this.logo_margin_y);
                ig.system.context.scale(this.logo_scale, this.logo_scale);
                this._drawLogo(ig.system.context);
                ig.system.context.restore();


                //
                // draw progress bar
                ig.system.context.save();
                ig.system.context.translate(this.progressBar_margin_x, this.progressBar_margin_y);
                this._drawProgressBar(ig.system.context, this.progressBar_target_w, this.progressBar_target_w/25);
                ig.system.context.restore();
            },


            /**
             * Draws the progress bar.
             *
             * @param ctx ... the canvas context
             * @param bar_w ... the bar width
             * @param bar_h ... the bar height
             * @private
             */
            _drawProgressBar: function(ctx, bar_w, bar_h) {
                ctx.save();
                ctx.lineWidth = bar_h;
                ctx.strokeStyle = "#9F9F9F";
                ctx.lineCap = "round";

                ctx.beginPath();
                ctx.moveTo(0, 0);
                ctx.lineTo(bar_w, 0);
                ctx.stroke();
                ctx.closePath();

                ctx.strokeStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(0,0);
                ctx.lineTo(bar_w*this._drawStatus, 0);
                ctx.stroke();
                ctx.closePath();


                ctx.restore();
            },


            /**
             * Draws the Logo with the wheel animated.
             *
             * @param ctx ... the canvas context
             * @private
             */
            _drawLogo: function(ctx) {

                //
                // initial drawing settings
                ctx.save();
                ctx.strokeStyle = 'rgba(0,0,0,0)';
                ctx.lineCap = 'butt';
                ctx.lineJoin = 'miter';
                ctx.miterLimit = 4;


                //
                // outer wheel circle
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(43.5,15.5);
                ctx.bezierCurveTo(42.6,10.9,40.3,6.300000000000001,35.2,3.4000000000000004);
                ctx.bezierCurveTo(33,2.2,30.300000000000004,1.3000000000000003,27.6,0.8000000000000003);
                ctx.bezierCurveTo(21.700000000000003,-0.3999999999999997,15.3,-0.19999999999999973,12.900000000000002,0.7000000000000003);
                ctx.bezierCurveTo(9.700000000000003,1.9000000000000004,7.3000000000000025,3.7,5.3000000000000025,6.2);
                ctx.bezierCurveTo(3.1000000000000023,8.9,1.5000000000000027,12.5,0.40000000000000213,17.2);
                ctx.lineTo(0,20.1);
                ctx.bezierCurveTo(0,22.3,0.2,24.5,0.6,26.700000000000003);
                ctx.bezierCurveTo(0.8,27.500000000000004,0.8999999999999999,28.400000000000002,1.2,29.200000000000003);
                ctx.bezierCurveTo(4,36.7,9,41.8,16.5,43.4);
                ctx.bezierCurveTo(18.5,43.8,20.6,44,22.8,43.9);
                ctx.bezierCurveTo(30.4,43.9,35.3,41.1,38.4,37.6);
                ctx.bezierCurveTo(43.8,31.5,44,23.200000000000003,44,23.200000000000003);
                ctx.bezierCurveTo(44.1,20.8,44,18.1,43.5,15.5);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();


                //
                // inner wheel
                ctx.save();
                ctx.strokeStyle = this._colorSettings[this._theme]['base'];

                // animate the wheel
                var wheel_rotation = ((this._showTime * this._wheelSpeed) % (2*Math.PI));

                // segment 1 (orange)
                ctx.save();
                ctx.fillStyle = "#f8962f";
                if(Math.floor(wheel_rotation/(2*Math.PI)*5.99999) == 0) ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(22.1,22);
                ctx.bezierCurveTo(23.900000000000002,19.5,34.2,6.699999999999999,28.5,4.699999999999999);
                ctx.bezierCurveTo(34.7,6.999999999999999,39.3,12.399999999999999,40.4,18.9);
                ctx.bezierCurveTo(41.4,24.8,25.2,22.4,22.1,22);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();

                // segment 2 (red)
                ctx.save();
                ctx.fillStyle = "#ec3b3e";
                if(Math.floor(wheel_rotation/(2*Math.PI)*5.99999) == 1) ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(22.1,22);
                ctx.bezierCurveTo(25.200000000000003,22.3,41.400000000000006,24.7,40.3,18.9);
                ctx.bezierCurveTo(41.4,25.4,38.9,32,33.9,36.2);
                ctx.bezierCurveTo(29.5,40,23.3,24.9,22.1,22);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();

                // segment 3 (purple)
                ctx.save();
                ctx.fillStyle = "#aa6ea8";
                if(Math.floor(wheel_rotation/(2*Math.PI)*5.99999) == 2) ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(22.1,22);
                ctx.bezierCurveTo(23.3,24.8,29.400000000000002,40,34,36.2);
                ctx.bezierCurveTo(28.9,40.400000000000006,21.9,41.6,15.8,39.300000000000004);
                ctx.bezierCurveTo(10.2,37.3,20.3,24.5,22.1,22);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();

                // segment 4 (blue)
                ctx.save();
                ctx.fillStyle = "#4067ab";
                if(Math.floor(wheel_rotation/(2*Math.PI)*5.99999) == 3) ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(22.1,22);
                ctx.bezierCurveTo(20.3,24.5,10.000000000000002,37.3,15.700000000000001,39.3);
                ctx.bezierCurveTo(9.5,37,5,31.6,3.9,25.1);
                ctx.bezierCurveTo(2.8,19.3,19,21.7,22.1,22);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();

                // segment 5 (green)
                ctx.save();
                ctx.fillStyle = "#4fb348";
                if(Math.floor(wheel_rotation/(2*Math.PI)*5.99999) == 4) ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(22.1,22);
                ctx.bezierCurveTo(19,21.7,2.8,19.3,3.9,25.1);
                ctx.bezierCurveTo(2.8,18.700000000000003,5.2,12.100000000000001,10.3,7.900000000000002);
                ctx.bezierCurveTo(14.9,4.1,20.9,19.2,22.1,22);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();

                // segment 6 (yellow)
                ctx.save();
                ctx.fillStyle = "#e8e73f";
                if(Math.floor(wheel_rotation/(2*Math.PI)*5.99999) == 5) ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(22.1,22);
                ctx.bezierCurveTo(20.900000000000002,19.2,14.8,4,10.200000000000001,7.800000000000001);
                ctx.bezierCurveTo(15.3,3.6000000000000005,22.3,2.4000000000000004,28.4,4.700000000000001);
                ctx.bezierCurveTo(34.2,6.8,24,19.6,22.1,22);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.restore();


                //
                // 'gamewheel' lettering
                ctx.save();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(56.1,25.2);
                ctx.bezierCurveTo(56.300000000000004,19.799999999999997,60.9,15.7,66.4,15.899999999999999);
                ctx.lineTo(66.4,15.899999999999999);
                ctx.bezierCurveTo(71.80000000000001,16.2,76.10000000000001,20.7,75.7,26.099999999999998);
                ctx.lineTo(75.7,26.099999999999998);
                ctx.bezierCurveTo(75.4,31.5,70.9,35.699999999999996,65.4,35.4);
                ctx.lineTo(65.4,35.4);
                ctx.bezierCurveTo(60,35.2,55.8,30.6,56.1,25.2);
                ctx.lineTo(56.1,25.2);
                ctx.closePath();
                ctx.moveTo(60.7,25.4);
                ctx.bezierCurveTo(60.6,28.2,62.7,30.599999999999998,65.60000000000001,30.7);
                ctx.lineTo(65.60000000000001,30.7);
                ctx.bezierCurveTo(68.4,30.8,70.80000000000001,28.7,70.9,25.799999999999997);
                ctx.lineTo(70.9,25.799999999999997);
                ctx.bezierCurveTo(71,22.999999999999996,68.9,20.599999999999998,66,20.499999999999996);
                ctx.lineTo(66,20.499999999999996);
                ctx.bezierCurveTo(63.3,20.5,60.9,22.6,60.7,25.4);
                ctx.lineTo(60.7,25.4);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(77.4,27);
                ctx.bezierCurveTo(76.9,21.6,80.7,16.8,86.10000000000001,16.3);
                ctx.lineTo(86.10000000000001,16.3);
                ctx.bezierCurveTo(91.50000000000001,15.700000000000001,96.4,19.6,96.9,25);
                ctx.lineTo(96.9,25);
                ctx.bezierCurveTo(97.5,30.4,93.60000000000001,35.2,88.2,35.7);
                ctx.lineTo(88.2,35.7);
                ctx.bezierCurveTo(82.8,36.3,78,32.4,77.4,27);
                ctx.lineTo(77.4,27);
                ctx.closePath();
                ctx.moveTo(82.1,26.5);
                ctx.bezierCurveTo(82.39999999999999,29.3,84.89999999999999,31.3,87.69999999999999,31);
                ctx.lineTo(87.69999999999999,31);
                ctx.bezierCurveTo(90.49999999999999,30.7,92.49999999999999,28.2,92.19999999999999,25.4);
                ctx.lineTo(92.19999999999999,25.4);
                ctx.bezierCurveTo(91.89999999999999,22.599999999999998,89.39999999999999,20.599999999999998,86.6,20.9);
                ctx.lineTo(86.6,20.9);
                ctx.bezierCurveTo(83.8,21.2,81.8,23.7,82.1,26.5);
                ctx.lineTo(82.1,26.5);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(209.8,35.4);
                ctx.bezierCurveTo(205.10000000000002,34.6,201.70000000000002,30.4,201.70000000000002,25.799999999999997);
                ctx.lineTo(201.70000000000002,25.799999999999997);
                ctx.bezierCurveTo(201.70000000000002,25.199999999999996,201.70000000000002,24.699999999999996,201.9,24.099999999999998);
                ctx.lineTo(201.9,24.099999999999998);
                ctx.bezierCurveTo(202.70000000000002,19.4,206.8,15.999999999999998,211.6,15.999999999999998);
                ctx.lineTo(211.6,15.999999999999998);
                ctx.bezierCurveTo(212.2,15.999999999999998,212.7,15.999999999999998,213.29999999999998,16.099999999999998);
                ctx.lineTo(213.29999999999998,16.099999999999998);
                ctx.bezierCurveTo(217.99999999999997,16.9,221.39999999999998,21,221.39999999999998,25.699999999999996);
                ctx.lineTo(221.39999999999998,25.699999999999996);
                ctx.bezierCurveTo(221.39999999999998,26.299999999999997,221.39999999999998,26.799999999999997,221.29999999999998,27.399999999999995);
                ctx.lineTo(221.29999999999998,27.399999999999995);
                ctx.bezierCurveTo(221.2,27.999999999999996,220.79999999999998,28.599999999999994,220.29999999999998,28.899999999999995);
                ctx.lineTo(220.29999999999998,28.899999999999995);
                ctx.bezierCurveTo(219.79999999999998,29.299999999999994,219.2,29.399999999999995,218.49999999999997,29.299999999999994);
                ctx.lineTo(218.49999999999997,29.299999999999994);
                ctx.lineTo(206.59999999999997,27.199999999999992);
                ctx.bezierCurveTo(207.09999999999997,28.999999999999993,208.59999999999997,30.39999999999999,210.59999999999997,30.799999999999994);
                ctx.lineTo(210.59999999999997,30.799999999999994);
                ctx.lineTo(210.59999999999997,30.799999999999994);
                ctx.bezierCurveTo(211.89999999999998,30.999999999999993,212.69999999999996,32.199999999999996,212.49999999999997,33.49999999999999);
                ctx.lineTo(212.49999999999997,33.49999999999999);
                ctx.bezierCurveTo(212.29999999999998,34.599999999999994,211.29999999999998,35.39999999999999,210.19999999999996,35.39999999999999);
                ctx.lineTo(210.19999999999996,35.39999999999999);
                ctx.bezierCurveTo(210.1,35.4,209.9,35.4,209.8,35.4);
                ctx.lineTo(209.8,35.4);
                ctx.closePath();
                ctx.moveTo(216.4,24.2);
                ctx.bezierCurveTo(215.8,22.5,214.4,21.099999999999998,212.4,20.7);
                ctx.lineTo(212.4,20.7);
                ctx.bezierCurveTo(212.1,20.599999999999998,211.8,20.599999999999998,211.5,20.599999999999998);
                ctx.lineTo(211.5,20.599999999999998);
                ctx.bezierCurveTo(209.9,20.599999999999998,208.5,21.299999999999997,207.5,22.499999999999996);
                ctx.lineTo(207.5,22.499999999999996);
                ctx.lineTo(216.4,24.2);
                ctx.lineTo(216.4,24.2);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(223.7,28);
                ctx.bezierCurveTo(223.5,27.2,223.39999999999998,26.4,223.39999999999998,25.6);
                ctx.lineTo(223.39999999999998,25.6);
                ctx.bezierCurveTo(223.39999999999998,21.200000000000003,226.39999999999998,17.200000000000003,230.79999999999998,16.1);
                ctx.lineTo(230.79999999999998,16.1);
                ctx.bezierCurveTo(231.6,15.900000000000002,232.39999999999998,15.8,233.2,15.8);
                ctx.lineTo(233.2,15.8);
                ctx.bezierCurveTo(237.6,15.8,241.6,18.8,242.7,23.200000000000003);
                ctx.lineTo(242.7,23.200000000000003);
                ctx.bezierCurveTo(242.79999999999998,23.800000000000004,242.79999999999998,24.400000000000002,242.39999999999998,25.000000000000004);
                ctx.lineTo(242.39999999999998,25.000000000000004);
                ctx.bezierCurveTo(242.09999999999997,25.500000000000004,241.59999999999997,25.900000000000002,240.99999999999997,26.100000000000005);
                ctx.lineTo(240.99999999999997,26.100000000000005);
                ctx.lineTo(229.29999999999998,28.900000000000006);
                ctx.bezierCurveTo(230.29999999999998,30.000000000000007,231.7,30.700000000000006,233.2,30.700000000000006);
                ctx.lineTo(233.2,30.700000000000006);
                ctx.bezierCurveTo(233.6,30.700000000000006,234,30.700000000000006,234.39999999999998,30.500000000000007);
                ctx.lineTo(234.39999999999998,30.500000000000007);
                ctx.bezierCurveTo(235.59999999999997,30.200000000000006,236.89999999999998,30.900000000000006,237.2,32.20000000000001);
                ctx.lineTo(237.2,32.20000000000001);
                ctx.bezierCurveTo(237.5,33.50000000000001,236.79999999999998,34.70000000000001,235.5,35.00000000000001);
                ctx.lineTo(235.5,35.00000000000001);
                ctx.bezierCurveTo(234.7,35.20000000000001,233.9,35.300000000000004,233.1,35.300000000000004);
                ctx.lineTo(233.1,35.300000000000004);
                ctx.bezierCurveTo(228.8,35.4,224.8,32.5,223.7,28);
                ctx.lineTo(223.7,28);
                ctx.closePath();
                ctx.moveTo(232,20.7);
                ctx.bezierCurveTo(230.1,21.2,228.7,22.599999999999998,228.3,24.4);
                ctx.lineTo(228.3,24.4);
                ctx.lineTo(237.10000000000002,22.299999999999997);
                ctx.bezierCurveTo(236.10000000000002,21.199999999999996,234.8,20.599999999999998,233.3,20.599999999999998);
                ctx.lineTo(233.3,20.599999999999998);
                ctx.bezierCurveTo(232.8,20.6,232.4,20.6,232,20.7);
                ctx.lineTo(232,20.7);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(75.6,33.7);
                ctx.bezierCurveTo(75.6,33.6,75.6,33.5,75.6,33.400000000000006);
                ctx.lineTo(75.69999999999999,25.900000000000006);
                ctx.lineTo(71,25.900000000000006);
                ctx.bezierCurveTo(71,26.000000000000007,71,26.000000000000007,71,26.100000000000005);
                ctx.bezierCurveTo(70.9,28.800000000000004,68.7,30.900000000000006,66,30.800000000000004);
                ctx.bezierCurveTo(65.9,30.800000000000004,65.9,30.800000000000004,65.8,30.800000000000004);
                ctx.bezierCurveTo(63.099999999999994,30.700000000000003,60.4,35.2,65.6,35.50000000000001);
                ctx.bezierCurveTo(65.69999999999999,35.50000000000001,65.8,35.50000000000001,65.89999999999999,35.50000000000001);
                ctx.lineTo(65.89999999999999,35.50000000000001);
                ctx.bezierCurveTo(67.69999999999999,35.50000000000001,69.39999999999999,35.10000000000001,70.8,34.20000000000001);
                ctx.bezierCurveTo(70.39999999999999,36.50000000000001,68.3,38.30000000000001,65.8,38.20000000000001);
                ctx.bezierCurveTo(64.5,38.20000000000001,63.4,39.20000000000001,63.4,40.50000000000001);
                ctx.bezierCurveTo(63.4,41.800000000000004,64.4,42.800000000000004,65.6,42.900000000000006);
                ctx.bezierCurveTo(65.6,42.900000000000006,65.6,42.900000000000006,65.69999999999999,42.900000000000006);
                ctx.lineTo(65.69999999999999,42.900000000000006);
                ctx.bezierCurveTo(71,42.9,75.3,38.9,75.6,33.7);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(123.9,35.4);
                ctx.bezierCurveTo(122.60000000000001,35.3,121.7,34.199999999999996,121.80000000000001,32.9);
                ctx.lineTo(122.4,25.5);
                ctx.bezierCurveTo(122.60000000000001,22.9,120.80000000000001,20.6,118.2,20.2);
                ctx.bezierCurveTo(118.10000000000001,20.2,117.9,20.2,117.8,20.2);
                ctx.lineTo(116,20.099999999999998);
                ctx.bezierCurveTo(116.2,20.999999999999996,116.2,21.799999999999997,116.2,22.599999999999998);
                ctx.lineTo(115.4,32.5);
                ctx.bezierCurveTo(115.30000000000001,33.8,114.2,34.7,112.9,34.6);
                ctx.bezierCurveTo(111.60000000000001,34.5,110.7,33.4,110.80000000000001,32.1);
                ctx.lineTo(111.60000000000001,22.200000000000003);
                ctx.bezierCurveTo(111.60000000000001,21.700000000000003,111.50000000000001,20.900000000000002,111.30000000000001,20.400000000000002);
                ctx.bezierCurveTo(111.20000000000002,20.1,111.00000000000001,20.000000000000004,110.9,19.900000000000002);
                ctx.bezierCurveTo(110.80000000000001,19.8,110.7,19.8,110.5,19.700000000000003);
                ctx.lineTo(110.4,19.700000000000003);
                ctx.lineTo(110.4,19.700000000000003);
                ctx.bezierCurveTo(107.80000000000001,19.500000000000004,105.5,21.300000000000004,105.10000000000001,23.900000000000002);
                ctx.bezierCurveTo(105.10000000000001,24.000000000000004,105.10000000000001,24.200000000000003,105.10000000000001,24.3);
                ctx.lineTo(104.50000000000001,31.700000000000003);
                ctx.bezierCurveTo(104.40000000000002,33,103.30000000000001,33.900000000000006,102.00000000000001,33.800000000000004);
                ctx.bezierCurveTo(100.70000000000002,33.7,99.80000000000001,32.6,99.90000000000002,31.300000000000004);
                ctx.lineTo(100.50000000000001,23.900000000000006);
                ctx.bezierCurveTo(100.50000000000001,23.700000000000006,100.50000000000001,23.400000000000006,100.60000000000001,23.200000000000006);
                ctx.bezierCurveTo(101.4,18.300000000000004,105.80000000000001,14.700000000000006,111.00000000000001,15.100000000000007);
                ctx.lineTo(111.00000000000001,15.100000000000007);
                ctx.lineTo(111.00000000000001,15.100000000000007);
                ctx.lineTo(118.50000000000001,15.700000000000006);
                ctx.bezierCurveTo(118.70000000000002,15.700000000000006,119.00000000000001,15.700000000000006,119.20000000000002,15.800000000000006);
                ctx.lineTo(119.20000000000002,15.800000000000006);
                ctx.bezierCurveTo(124.10000000000002,16.600000000000005,127.70000000000002,21.000000000000007,127.30000000000001,26.10000000000001);
                ctx.lineTo(126.70000000000002,33.50000000000001);
                ctx.bezierCurveTo(126.4,34.6,125.2,35.5,123.9,35.4);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(175.6,16.6);
                ctx.bezierCurveTo(174.29999999999998,16.5,173.2,17.5,173.1,18.8);
                ctx.lineTo(172.5,26.200000000000003);
                ctx.bezierCurveTo(172.3,28.800000000000004,170.1,30.800000000000004,167.6,30.800000000000004);
                ctx.bezierCurveTo(167.5,30.800000000000004,167.29999999999998,30.800000000000004,167.2,30.800000000000004);
                ctx.lineTo(165.39999999999998,30.700000000000003);
                ctx.bezierCurveTo(165.7,29.900000000000002,165.89999999999998,29.000000000000004,165.99999999999997,28.300000000000004);
                ctx.lineTo(166.69999999999996,18.400000000000006);
                ctx.bezierCurveTo(166.79999999999995,17.100000000000005,165.79999999999995,16.000000000000007,164.59999999999997,15.900000000000006);
                ctx.bezierCurveTo(163.29999999999995,15.800000000000006,162.19999999999996,16.800000000000004,162.09999999999997,18.100000000000005);
                ctx.lineTo(161.29999999999995,28.000000000000007);
                ctx.bezierCurveTo(161.29999999999995,28.500000000000007,160.99999999999994,29.300000000000008,160.69999999999996,29.800000000000008);
                ctx.bezierCurveTo(160.49999999999997,30.000000000000007,160.39999999999995,30.200000000000006,160.19999999999996,30.300000000000008);
                ctx.bezierCurveTo(160.09999999999997,30.40000000000001,159.99999999999997,30.40000000000001,159.79999999999995,30.40000000000001);
                ctx.lineTo(159.69999999999996,30.40000000000001);
                ctx.lineTo(159.69999999999996,30.40000000000001);
                ctx.bezierCurveTo(157.09999999999997,30.20000000000001,155.09999999999997,28.00000000000001,155.09999999999997,25.500000000000007);
                ctx.bezierCurveTo(155.09999999999997,25.400000000000006,155.09999999999997,25.200000000000006,155.09999999999997,25.10000000000001);
                ctx.lineTo(155.69999999999996,17.70000000000001);
                ctx.bezierCurveTo(155.79999999999995,16.40000000000001,154.79999999999995,15.30000000000001,153.49999999999997,15.20000000000001);
                ctx.bezierCurveTo(152.2,15.10000000000001,151.09999999999997,16.10000000000001,150.99999999999997,17.40000000000001);
                ctx.lineTo(150.39999999999998,24.80000000000001);
                ctx.bezierCurveTo(150.39999999999998,25.00000000000001,150.39999999999998,25.30000000000001,150.39999999999998,25.50000000000001);
                ctx.bezierCurveTo(150.39999999999998,30.50000000000001,154.2,34.70000000000001,159.29999999999998,35.10000000000001);
                ctx.lineTo(159.29999999999998,35.10000000000001);
                ctx.lineTo(159.29999999999998,35.10000000000001);
                ctx.lineTo(166.79999999999998,35.70000000000001);
                ctx.bezierCurveTo(166.99999999999997,35.70000000000001,167.29999999999998,35.70000000000001,167.49999999999997,35.70000000000001);
                ctx.lineTo(167.49999999999997,35.70000000000001);
                ctx.bezierCurveTo(172.49999999999997,35.70000000000001,176.69999999999996,31.80000000000001,177.19999999999996,26.80000000000001);
                ctx.lineTo(177.79999999999995,19.400000000000013);
                ctx.bezierCurveTo(177.9,17.8,176.9,16.7,175.6,16.6);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(199.8,32.2);
                ctx.lineTo(199.8,32.2);
                ctx.lineTo(199.4,24.800000000000004);
                ctx.bezierCurveTo(199.1,19.700000000000003,194.9,15.700000000000005,189.70000000000002,15.700000000000005);
                ctx.bezierCurveTo(189.50000000000003,15.700000000000005,189.4,15.700000000000005,189.20000000000002,15.700000000000005);
                ctx.bezierCurveTo(187.4,15.800000000000004,185.70000000000002,16.400000000000006,184.3,17.300000000000004);
                ctx.lineTo(184,8.6);
                ctx.bezierCurveTo(183.9,7.3,182.8,6.3,181.5,6.3999999999999995);
                ctx.bezierCurveTo(180.2,6.499999999999999,179.2,7.6,179.3,8.799999999999999);
                ctx.lineTo(180.5,33.199999999999996);
                ctx.bezierCurveTo(180.5,33.3,180.5,33.49999999999999,180.6,33.599999999999994);
                ctx.bezierCurveTo(180.6,33.699999999999996,180.6,33.8,180.7,33.8);
                ctx.bezierCurveTo(180.7,33.8,180.7,33.8,180.7,33.9);
                ctx.lineTo(180.7,34);
                ctx.bezierCurveTo(180.7,34,180.7,34.1,180.79999999999998,34.1);
                ctx.bezierCurveTo(180.79999999999998,34.2,180.89999999999998,34.2,180.89999999999998,34.300000000000004);
                ctx.lineTo(180.89999999999998,34.300000000000004);
                ctx.bezierCurveTo(180.89999999999998,34.300000000000004,180.89999999999998,34.400000000000006,180.99999999999997,34.400000000000006);
                ctx.bezierCurveTo(180.99999999999997,34.400000000000006,180.99999999999997,34.50000000000001,181.09999999999997,34.50000000000001);
                ctx.bezierCurveTo(181.09999999999997,34.50000000000001,181.09999999999997,34.50000000000001,181.09999999999997,34.60000000000001);
                ctx.lineTo(181.19999999999996,34.70000000000001);
                ctx.lineTo(181.29999999999995,34.80000000000001);
                ctx.lineTo(181.29999999999995,34.80000000000001);
                ctx.bezierCurveTo(181.29999999999995,34.80000000000001,181.39999999999995,34.90000000000001,181.49999999999994,34.90000000000001);
                ctx.lineTo(181.49999999999994,34.90000000000001);
                ctx.bezierCurveTo(181.89999999999995,35.20000000000001,182.39999999999995,35.40000000000001,182.89999999999995,35.40000000000001);
                ctx.lineTo(182.99999999999994,35.40000000000001);
                ctx.bezierCurveTo(184.29999999999995,35.30000000000001,185.29999999999995,34.20000000000001,185.19999999999993,32.90000000000001);
                ctx.lineTo(184.79999999999993,25.500000000000014);
                ctx.bezierCurveTo(184.79999999999993,25.400000000000013,184.79999999999993,25.300000000000015,184.79999999999993,25.200000000000014);
                ctx.bezierCurveTo(184.79999999999993,22.600000000000012,186.89999999999992,20.400000000000013,189.49999999999991,20.200000000000014);
                ctx.lineTo(189.49999999999991,20.200000000000014);
                ctx.bezierCurveTo(189.5999999999999,20.200000000000014,189.6999999999999,20.200000000000014,189.6999999999999,20.200000000000014);
                ctx.bezierCurveTo(192.2999999999999,20.200000000000014,194.49999999999991,22.300000000000015,194.6999999999999,24.900000000000013);
                ctx.lineTo(195.0999999999999,32.30000000000001);
                ctx.bezierCurveTo(195.1999999999999,33.60000000000001,196.2999999999999,34.60000000000001,197.49999999999991,34.500000000000014);
                ctx.bezierCurveTo(198.9,34.6,199.9,33.5,199.8,32.2);
                ctx.closePath();
                ctx.moveTo(189.3,15.7);
                ctx.lineTo(189.3,15.7);
                ctx.lineTo(189.3,15.7);
                ctx.lineTo(189.3,15.7);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(247.1,35.4);
                ctx.bezierCurveTo(245.79999999999998,35.4,244.79999999999998,34.3,244.79999999999998,33);
                ctx.lineTo(244.79999999999998,33);
                ctx.lineTo(245.29999999999998,8.600000000000001);
                ctx.bezierCurveTo(245.29999999999998,7.300000000000002,246.39999999999998,6.300000000000002,247.7,6.300000000000002);
                ctx.lineTo(247.7,6.300000000000002);
                ctx.bezierCurveTo(249,6.300000000000002,250,7.400000000000002,250,8.700000000000001);
                ctx.lineTo(250,8.700000000000001);
                ctx.lineTo(249.5,33.2);
                ctx.bezierCurveTo(249.4,34.4,248.4,35.4,247.1,35.4);
                ctx.lineTo(247.1,35.4);
                ctx.lineTo(247.1,35.4);
                ctx.lineTo(247.1,35.4);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(95.4,35);
                ctx.bezierCurveTo(94.10000000000001,35.1,93,34.1,92.9,32.9);
                ctx.lineTo(92.80000000000001,32.6);
                ctx.lineTo(96.9,25.200000000000003);
                ctx.lineTo(97.7,32.6);
                ctx.bezierCurveTo(97.7,33.8,96.8,34.9,95.4,35);
                ctx.lineTo(95.4,35);
                ctx.bezierCurveTo(95.5,35,95.5,35,95.4,35);
                ctx.lineTo(95.4,35);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.save();
                ctx.fillStyle = this._colorSettings[this._theme]['base'];
                ctx.beginPath();
                ctx.moveTo(129.1,28.1);
                ctx.bezierCurveTo(128.9,27.3,128.79999999999998,26.5,128.79999999999998,25.700000000000003);
                ctx.lineTo(128.79999999999998,25.700000000000003);
                ctx.bezierCurveTo(128.79999999999998,21.300000000000004,131.79999999999998,17.300000000000004,136.2,16.200000000000003);
                ctx.lineTo(136.2,16.200000000000003);
                ctx.bezierCurveTo(137,16.000000000000004,137.79999999999998,15.900000000000002,138.6,15.900000000000002);
                ctx.lineTo(138.6,15.900000000000002);
                ctx.bezierCurveTo(143,15.900000000000002,147,18.900000000000002,148.1,23.300000000000004);
                ctx.lineTo(148.1,23.300000000000004);
                ctx.bezierCurveTo(148.2,23.900000000000006,148.2,24.500000000000004,147.79999999999998,25.100000000000005);
                ctx.lineTo(147.79999999999998,25.100000000000005);
                ctx.bezierCurveTo(147.49999999999997,25.600000000000005,146.99999999999997,26.000000000000004,146.39999999999998,26.200000000000006);
                ctx.lineTo(146.39999999999998,26.200000000000006);
                ctx.lineTo(134.7,29);
                ctx.bezierCurveTo(135.7,30.1,137.1,30.8,138.6,30.8);
                ctx.lineTo(138.6,30.8);
                ctx.bezierCurveTo(139,30.8,139.4,30.8,139.79999999999998,30.6);
                ctx.lineTo(139.79999999999998,30.6);
                ctx.bezierCurveTo(140.99999999999997,30.3,142.29999999999998,31,142.6,32.300000000000004);
                ctx.lineTo(142.6,32.300000000000004);
                ctx.bezierCurveTo(142.9,33.6,142.2,34.800000000000004,140.9,35.1);
                ctx.lineTo(140.9,35.1);
                ctx.bezierCurveTo(140.1,35.300000000000004,139.3,35.4,138.5,35.4);
                ctx.lineTo(138.5,35.4);
                ctx.bezierCurveTo(134.2,35.5,130.2,32.5,129.1,28.1);
                ctx.lineTo(129.1,28.1);
                ctx.closePath();
                ctx.moveTo(137.3,20.8);
                ctx.bezierCurveTo(135.4,21.3,134,22.7,133.60000000000002,24.5);
                ctx.lineTo(133.60000000000002,24.5);
                ctx.lineTo(142.40000000000003,22.4);
                ctx.bezierCurveTo(141.40000000000003,21.299999999999997,140.10000000000002,20.7,138.60000000000002,20.7);
                ctx.lineTo(138.60000000000002,20.7);
                ctx.bezierCurveTo(138.2,20.6,137.8,20.7,137.3,20.8);
                ctx.lineTo(137.3,20.8);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();
                ctx.restore();
                ctx.restore();

                ctx.restore();
            },


            /**
             * TODO: Use library for that (tinyColor e.g.)
             */
            _rgb2hsl: function(hex) {
                var r = parseInt(hex.substring(1,3),16);
                var g = parseInt(hex.substring(3,5),16);
                var b = parseInt(hex.substring(5,7),16);

                r /= 255, g /= 255, b /= 255;
                var max = Math.max(r, g, b), min = Math.min(r, g, b);
                var h, s, l = (max + min) / 2;

                if(max == min){
                    h = s = 0; // achromatic
                }else{
                    var d = max - min;
                    s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
                    switch(max){
                        case r: h = (g - b) / d + (g < b ? 6 : 0); break;
                        case g: h = (b - r) / d + 2; break;
                        case b: h = (r - g) / d + 4; break;
                    }
                    h /= 6;
                }

                return [h, s, l];
            }
        });
});


// lib/gamewheel/entities/progressBar.js
ig.baked=true;ig.module(
    'gamewheel.entities.progressBar'
).requires(
    'impact.entity'
).defines(function() {

        ig.global.EntityProgressBar = ig.Entity.extend({

            completedColor: '#FFFFFF',
            uncompletedColor: '#000000',
            borderColor: '#FFFFFF',
            borderWidth: 2,

            percentage: 0, // In the range 0 - 1.
            verticalLayout: false,

            // Make scalable in weltmeister.
            _wmScalable: true,

            init: function( x, y, settings ) {
                this.parent( x, y, settings );
            },

            draw: function () {
                var ctx = ig.system.context;

                if(!this.verticalLayout) {

                    // Horizontal layout. Completed is on the left.
                    var completedH = Math.round(this.size.x * this.percentage).limit(0, this.size.x);
                    var remainingH = this.size.x - completedH;

                    if(this.completedColor) {
                        ctx.fillStyle = this.completedColor;
                        ctx.fillRect(
                            ig.system.getDrawPos(this.pos.x - this.offset.x - ig.game._rscreen.x),
                            ig.system.getDrawPos(this.pos.y - this.offset.y - ig.game._rscreen.y),
                            completedH * ig.system.scale,
                            this.size.y * ig.system.scale);
                    }

                    if(this.uncompletedColor) {
                        ctx.fillStyle = this.uncompletedColor;
                        ctx.fillRect(
                            ig.system.getDrawPos(this.pos.x + completedH - this.offset.x - ig.game._rscreen.x),
                            ig.system.getDrawPos(this.pos.y - this.offset.y - ig.game._rscreen.y),
                            remainingH * ig.system.scale,
                            this.size.y * ig.system.scale);
                    }

                } else {
                    // Vertical layout. Completed is on the bottom.
                    var completedV = Math.round(this.size.y * this.percentage).limit(0, this.size.y);
                    var remainingV = this.size.y - completedV;

                    if(this.completedColor) {
                        ctx.fillStyle = this.completedColor;
                        ctx.fillRect(
                            ig.system.getDrawPos(this.pos.x - this.offset.x - ig.game._rscreen.x),
                            ig.system.getDrawPos(this.pos.y + remainingV - this.offset.y - ig.game._rscreen.y),
                            this.size.x * ig.system.scale,
                            completedV * ig.system.scale);

                    }

                    if(this.uncompletedColor) {
                        ctx.fillStyle = this.uncompletedColor;
                        ctx.fillRect(
                            ig.system.getDrawPos(this.pos.x - this.offset.x - ig.game._rscreen.x),
                            ig.system.getDrawPos(this.pos.y - this.offset.y - ig.game._rscreen.y),
                            this.size.x * ig.system.scale,
                            remainingV * ig.system.scale);
                    }

                }

                // Draw any assigned image.
                this.parent();

                // Draw border.
                if(this.borderColor && this.borderWidth) {
                    ctx.strokeStyle = this.borderColor;
                    ctx.lineWidth = this.borderWidth * ig.system.scale;
                    ctx.strokeRect(
                        ig.system.getDrawPos(this.pos.x - this.offset.x - ig.game._rscreen.x),
                        ig.system.getDrawPos(this.pos.y - this.offset.y - ig.game._rscreen.y),
                        this.size.x * ig.system.scale,
                        this.size.y * ig.system.scale);
                }
            }

        })

    });
/*
 The content class is used to pre-load and maintain instances to named assets as defined as part of the game's configuration.
 */





// lib/gamewheel/extensions/entityScaling.js
ig.baked=true;ig.module(
    'gamewheel.extensions.entityScaling'
).requires(
    'impact.entity'
).defines(function() {
        "use strict";

        ig.Entity.inject({
            scale: {x: 1, y: 1},
            scalePoint: null,

            // Override the draw method to add support for scaling.
            draw: function() {
                if( this.currentAnim ) {
                    if (this.scale.x != 1 || this.scale.y != 1) {

                        var sx = this.scale.x / ig.system.scale;
                        var sy = this.scale.y / ig.system.scale;

                        if(!this.scalePoint) {
                            // Default to scale from the center.
                            this.scalePoint = {
                                x: this.currentAnim.sheet.width / 2,
                                y: this.currentAnim.sheet.height / 2
                            }
                        }

                        var ox = this.offset.x + (this.scalePoint.x * sx) - this.scalePoint.x;
                        var oy = this.offset.y + (this.scalePoint.y * sy) - this.scalePoint.y;

                        ig.system.context.save();
                        ig.system.context.translate(
                            ig.system.getDrawPos(this.pos.x - ox - ig.game._rscreen.x),
                            ig.system.getDrawPos(this.pos.y - oy - ig.game._rscreen.y)
                        );
                        ig.system.context.scale(sx, sy);
                        this.currentAnim.draw(0, 0);
                        ig.system.context.restore();

                    } else {
                        this.currentAnim.draw(
                            this.pos.x - this.offset.x - ig.game._rscreen.x,
                            this.pos.y - this.offset.y - ig.game._rscreen.y
                        );
                    }
                }
            }
        });

    });

// lib/gamewheel/entities/particle.js
ig.baked=true;ig.module(
    'gamewheel.entities.particle'
).requires(
    'impact.entity',
    'gamewheel.helpers.util',
    'gamewheel.extensions.entityScaling'
).defines(function(){

        ig.global.EntityParticle = ig.Entity.extend({
            
            sheetImageName: null,
            imageName: null,

            maxVel: {x: 2000, y: 2000},
            maxVelRange: null,
            friction: {x: 0, y: 0},

            lifeTime: 0.7,
            lifeTimeRange: null,
            fadeOutTime: 0.3,
            fadeInTime: 0,

            movementSpeed: null,
            movementSpeedRange: 0,
            
            movementAngle: null,
            movementAngleRange: 0,
            
            movementAccel: null,
            movementAccelRange: 0,

            angularVel: 0,
            angularVelRange: 0,
            randomizeAngle: false,
            
            scaleVel: {x: 0, y: 0},

            scaleRange: undefined,
            

            init: function (x, y, settings) {
                this.parent(x, y, settings);

                this.idleTimer = new ig.Timer();

                if(this.sheetImageName) {
                    // Use sprite sheet. Split into frames based on size. Pick a random frame.
                    var sheet = ig.content.imageNamed(this.sheetImageName);
                    if(sheet) {
                        this.animSheet = new ig.AnimationSheet(sheet.path, this.size.x, this.size.y);
                        var frames = Math.floor(sheet.width / this.size.x) * Math.floor(sheet.height / this.size.y);
                        this.anims['idle'] = new ig.Animation(this.animSheet, 10, [ig.util.randomInt(frames)], false);
                        this.currentAnim = this.anims['idle'];
                    }
                } else if(this.imageName) {
                    // Use whole image.
                    var image = ig.content.imageNamed(this.imageName);
                    if(image) {
                        this.animSheet = new ig.AnimationSheet(image.path, image.width, image.height);
                        this.anims['idle'] = new ig.Animation(this.animSheet, 1, [0], false);
                        this.currentAnim = this.anims['idle'];
                        this.size = {x: image.width, y: image.height};
                    }
                }

                // Position via center of the image if not overridden.
                if(!settings.offset) {
                    this.offset = {x: this.size.x / 2, y: this.size.y / 2};
                }

                // Move
                if(this.movementSpeed !== null && this.movementAngle !== null) {
                    var speed = ig.util.rangedFloat(this.movementSpeed, this.movementSpeedRange);
                    var angle = ig.util.rangedFloat(this.movementAngle, this.movementAngleRange);
                    this.vel = this.getMovement(angle, speed);
                    
                    if(this.movementAccel !== null) {
                        var accel = ig.util.rangedFloat(this.movementAccel, this.movementAccelRange);
                        this.accel = this.getMovement(angle, accel);
                    }
                }

                if(this.randomizeAngle) {
                    this.currentAnim.angle = ig.util.randomFloat(Math.PI * 2);
                }
                
                if(this.angularVelRange) {
                    this.angularVel = ig.util.rangedFloat(this.angularVel, this.angularVelRange);
                }

                if(this.lifeTimeRange) {
                    this.lifeTime = ig.util.rangedFloat(this.lifeTime, this.lifeTimeRange);
                }

                if(this.maxVelRange) {
                    this.maxVel = {
                        x: ig.util.rangedFloat(this.maxVel.x, this.maxVelRange.x),
                        y: ig.util.rangedFloat(this.maxVel.y, this.maxVelRange.y)
                    };
                }

                if(this.scaleRange) {
                    var mod = ig.util.rangedFloat(0, this.scaleRange);
                    this.scale = {
                        x: this.scale.x + mod,
                        y: this.scale.y + mod
                    };
                }
                
            },

            setMovement: function(angle, speed) {
                this.vel = this.getMovement(angle, speed);
            },

            getMovement: function(angle, speed) {
                return {x: Math.cos(angle) * speed, y: Math.sin(angle) * speed };
            },
            
            update: function() {

                // Kill if lifetime is up.
                if(this.lifeTime > 0 && this.idleTimer.delta() > this.lifeTime ) {
                    this.kill();
                } else {

                    // Scale
                    if(this.scaleVel.x !== 0 || this.scaleVel.y !== 0) {
                        this.scale.x += (this.scaleVel.x * ig.system.tick);
                        this.scale.y += (this.scaleVel.y * ig.system.tick);
                    }

                    if(this.currentAnim) {
                        // Fading. Note that fade-in overrides fade-out if lifetime is too short.
                        if (this.idleTimer.delta() <= this.fadeInTime) {
                            // Fade in. 
                            this.currentAnim.alpha = this.idleTimer.delta().map(0, this.fadeInTime, 0, 1);
                        } else {
                            // Fade out
                            this.currentAnim.alpha = this.idleTimer.delta().map(this.lifeTime - this.fadeOutTime, this.lifeTime, 1, 0);
                        }
                        // Rotate
                        this.currentAnim.angle += (this.angularVel * ig.system.tick);
                    }

                    this.parent();
                }

            }

        });


    });

// lib/gamewheel/entities/particleEmitter.js
ig.baked=true;ig.module(
    'gamewheel.entities.particleEmitter'
).requires(
    'impact.entity',
    'gamewheel.entities.particle',
    'gamewheel.helpers.util'
).defines(function(){

        ig.global.EntityParticleEmitter = ig.Entity.extend({

            size: {x: 100, y: 100}, // Size of area to spawn particles in.
            particlesPerSecond: 3, // How many particles to spawn per second.
            disable: false, // Should particles be spawned or not.
            maxParticles: 0, // Maximum number of particles that can be spawned. No limit if 0 or less.

            configParticleName: undefined, // Entity to spawn from config.
            particleSetup: undefined, // Spawn setup object
            spawnCallback: undefined,

            sortEntitiesOnSpawn: true,
            zIndexIncrease: 0.00001,
            
            centeredPos: false, // Particles will be spawned relative to this.pos being the center of the spawn area 

            _timeSinceSpawn: 0, // Time counter for tracking when to spawn the next particle.
            _spawnCount: 0,

            init: function (x, y, settings) {
                this.parent(x, y, settings);
                if(this.configParticleName) {
                    this.particleSetup = ig.config.entities[this.configParticleName];
                }
            },

            pause: function(duration) {
                this._timeSinceSpawn = -duration;
            },

            draw: function () { },

            reset: function() {
                this._spawnCount = 0;
                this._timeSinceSpawn = 0;
            },

            update: function() {
                this.parent();

                if(!this.disable && (this.maxParticles <= 0 || this._spawnCount < this.maxParticles)) {
                    // Spawn particles.
                    this._timeSinceSpawn += ig.system.tick;
                    while (this._timeSinceSpawn > 1 / this.particlesPerSecond) {
                        this._timeSinceSpawn -= 1 / this.particlesPerSecond;
                        this.spawnParticle();
                    }
                }
            },

            spawnParticle: function() {
                if(this.particleSetup) {
                    this._spawnCount++;

                    // Get random location.
                    var x = this.pos.x + ig.util.randomFloat(this.size.x);
                    var y = this.pos.y + ig.util.randomFloat(this.size.y);
                    
                    // Adjust location if we're treating this.pos as the center of the spawn area. 
                    if(this.centeredPos) {
                        x -= this.size.x / 2;
                        y -= this.size.y / 2;
                    }

                    // Spawn particle.
                    var p = ig.game.spawnEntityWithSetup(this.particleSetup, {pos: {x: x, y: y}});

                    if(this.particleSetup.zIndex === undefined) {
                        p.zIndex = this.zIndex + (this._spawnCount * this.zIndexIncrease);
                    }

                    // Run callback and pass along spawned particle.
                    if(this.spawnCallback) {
                        this.spawnCallback(p, x, y);
                    }

                    // Sort entities.
                    if(this.sortEntitiesOnSpawn) {
                        ig.game.sortEntitiesDeferred();
                    }
                }
            }

        });
    });

// lib/gamewheel/prefabs/confettiPrefabs.js
ig.baked=true;ig.module(
    'gamewheel.prefabs.confettiPrefabs'
).requires(
    'gamewheel.entities.particleEmitter'
).defines(function() { "use strict";
        
        ig.Prefabs = ig.Prefabs || {};

        ig.Prefabs.confetti = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 0, y: -60},
                size: {x: 640, y: 140},
                particlesPerSecond: 400,
                maxParticles: 170,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    image: 'pf_Confetti',
                    animated: {
                        frameSize: {x: 20, y: 20},
                        animations: {
                            yellow: { frames: [0,1,3,4,2,5,6], frameTime: 0.1, loop: true },
                            blue: { frames: [7,8,9,10,11,12,13], frameTime: 0.1, loop: true },
                            green: { frames: [14,15,16,17,18,19,20], frameTime: 0.1, loop: true },
                            red: { frames: [21,22,23,24,25,26,27], frameTime: 0.1, loop: true }
                        }
                    },
                    settings: {
                        size: {x: 20, y: 20},
                        scale: {x: 1, y: 1},
                        scaleRange: 0.6,
                        accel: {x: 0, y: 200},
                        maxVel: {x: 0, y: 100},
                        maxVelRange: {x: 0, y: 100},
                        lifeTime: 2.5,
                        lifeTimeRange: 1,
                        fadeOutTime: 0.8,
                        fadeInTime: 0.3,
                        randomizeAngle: true,
                        angularVelRange: 2.5
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: 100,
                spawnCallback: function(p, x, y){
                    p.currentAnim = p.anims[['yellow', 'blue', 'green', 'red'].random()];
                }
            },
            assets: {
                images: {
                    pf_Confetti: 'pf_Confetti.png'
                }
            }
        };
        
    });/*
 The content class is used to pre-load and maintain instances to named assets as defined as part of the game's configuration.
 */


// lib/gamewheel/prefabs/balloonPrefabs.js
ig.baked=true;ig.module(
    'gamewheel.prefabs.balloonPrefabs'
).requires(
    'gamewheel.entities.particleEmitter'
).defines(function() { "use strict";
        
        ig.Prefabs = ig.Prefabs || {};
        
        ig.Prefabs.hearts = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 0, y: 1000},
                size: {x: 640, y: 10},
                particlesPerSecond: 18,
                maxParticles: 40,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    settings: {
                        imageName: 'pf_Heart',
                        scale: {x: 1, y: 1},
                        vel: {x: 0, y: -70},
                        accel: {x: 0, y: -220},
                        maxVel: {x: 0, y: 800},
                        lifeTime: 4,
                        fadeOutTime: 0,
                        fadeInTime: 0,
                        randomizeAngle: false
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: 100
            },
            assets: {
                images: {
                    pf_Heart: 'pf_Heart.png'
                }
            }
        };
        

        ig.Prefabs.balloons = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 0, y: 1020},
                size: {x: 640, y: 10},
                particlesPerSecond: 15,
                maxParticles: 30,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    settings: {
                        sheetImageName: 'pf_Balloons',
                        size: {x: 77, y: 119},
                        scale: {x: 0.87, y: 0.87},
                        scaleRange: 0.25,
                        vel: {x: 0, y: -100},
                        accel: {x: 0, y: -180},
                        maxVel: {x: 0, y: 350},
                        maxVelRange: {x: 0, y: 50},
                        lifeTime: 4,
                        fadeOutTime: 0,
                        fadeInTime: 0,
                        randomizeAngle: false
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: 100
            },
            assets: {
                images: {
                    pf_Balloons: 'pf_Balloons.png'
                }
            }
        };
        


    });

// lib/gamewheel/prefabs/starPrefabs.js
ig.baked=true;ig.module(
    'gamewheel.prefabs.starPrefabs'
).requires(
    'gamewheel.entities.particleEmitter'
).defines(function() { "use strict";

        ig.Prefabs = ig.Prefabs || {};

        ig.Prefabs.glowStars = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 10, y: 10},
                size: {x: 620, y: 940},
                particlesPerSecond: 10,
                maxParticles: 40,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    settings: {
                        imageName: 'pf_GlowStar',
                        scale: {x: 0.1, y: 0.1},
                        maxVel: {x: 0, y: 0},
                        scaleVel: {x: 0.6, y: 0.6},
                        lifeTime: 1.1,
                        fadeOutTime: 0.5,
                        fadeInTime: 0.2,
                        randomizeAngle: true,
                        angularVelRange: 3
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: 100
            },
            assets: {
                images: {
                    pf_GlowStar: 'pf_GlowStar.png'
                }
            }
        };

        ig.Prefabs.glitterStars = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 10, y: 10},
                size: {x: 620, y: 940},
                particlesPerSecond: 150,
                maxParticles: 500,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    settings: {
                        sheetImageName: 'pf_GlitterStar',
                        size: {x: 12, y: 13},
                        maxVel: {x: 0, y: 0},
                        scaleVel: {x: 0.3, y: 0.3},
                        lifeTime: 1.2,
                        fadeOutTime: 0.5,
                        fadeInTime: 0.7,
                        randomizeAngle: true,
                        angularVelRange: 2
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: 100
            },
            assets: {
                images: {
                    pf_GlitterStar: 'pf_GlitterStar.png'
                }
            }
        };
        
        ig.Prefabs.stars = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 10, y: 10},
                size: {x: 620, y: 940},
                particlesPerSecond: 15,
                maxParticles: 50,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    settings: {
                        imageName: 'pf_Star',
                        scale: {x: 0.3, y: 0.3},
                        maxVel: {x: 0, y: 0},
                        scaleVel: {x: 0.3, y: 0.3},
                        lifeTime: 2.2,
                        fadeOutTime: 0.4,
                        fadeInTime: 1,
                        randomizeAngle: true,
                        angularVelRange: 3
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: 100,
                zIndexzIndexIncrease: -0.0001
            },
            assets: {
                images: {
                    pf_Star: 'pf_Star.png'
                }
            }
        };

    });

// lib/gamewheel/prefabs/rainPrefabs.js
ig.baked=true;ig.module(
    'gamewheel.prefabs.rainPrefabs'
).requires(
    'gamewheel.entities.particleEmitter'
).defines(function() { "use strict";

        ig.Prefabs = ig.Prefabs || {};

        ig.Prefabs.cometShower = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 10, y: -80},
                size: {x: 620, y: 10},
                particlesPerSecond: 5,
                maxParticles: null,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    settings: {
                        imageName: 'pf_BlueComet',
                        vel: {x: 0, y: 400},
                        lifeTime: 4,
                        fadeOutTime: 0,
                        fadeInTime: 0,
                        randomizeAngle: false
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: -1
            },
            assets: {
                images: {
                    pf_BlueComet: 'pf_BlueComet.png'
                }
            }
        };

        ig.Prefabs.coins = {
            type: ig.global.EntityParticleEmitter,
            settings: {
                pos: {x: 10, y: -80},
                size: {x: 620, y: 10},
                particlesPerSecond: 18,
                maxParticles: 40,
                particleSetup: {
                    type: ig.global.EntityParticle,
                    image: 'pf_Coin',
                    animated: {
                        frameSize: {x: 83, y: 92},
                        animations: {
                            idle: {
                                frames: [0,1,2,3,4,5,6,7],
                                frameTime: 0.1,
                                loop: true
                            }
                        }
                    },
                    settings: {
                        scale: {x: 0.8, y: 0.8},
                        vel: {x: 0, y: 200},
                        accel: {x: 0, y: 1000},
                        lifeTime: 4,
                        fadeOutTime: 0,
                        fadeInTime: 0,
                        randomizeAngle: false
                    }
                },
                sortEntitiesOnSpawn: true,
                zIndex: 100
            },
            assets: {
                images: {
                    pf_Coin: 'pf_Coin.png'
                }
            }
        };
        

    });


// lib/gamewheel/prefabs/fireworksPrefabs.js
ig.baked=true;ig.module(
    'gamewheel.prefabs.fireworksPrefabs'
).requires(
    'gamewheel.entities.particleEmitter'
).defines(function() { "use strict";
        
        ig.Prefabs = ig.Prefabs || {};

        ig.Prefabs.fireworks = {
            spawn: function() {
                var fw = {
                    type: ig.global.EntityParticleEmitter,
                    settings: {
                        pos: {x: 170, y: 170},
                        size: {x: 5, y: 5},
                        particlesPerSecond: 300,
                        maxParticles: 40,
                        particleSetup: {
                            type: ig.global.EntityParticle,
                            image: 'pf_ColoredStars',
                            animated: {
                                frameSize: {x: 55, y: 55},
                                animations: {
                                    idle: { frames: [1], frameTime: 2 }
                                }
                            },
                            settings: {
                                scale: {x: 0.1, y: 0.1},
                                scaleVel: {x: 0.6, y: 0.6},
                                lifeTime: 0.6,
                                fadeOutTime: 0.2,
                                fadeInTime: 0.3,
                                randomizeAngle: true,
                                angularVelRange: 3,
                                movementAngleRange: 6.28,
                                movementAngle: 0,
                                movementSpeed: 300,
                                movementSpeedRange: 300
                            }
                        },
                        sortEntitiesOnSpawn: true,
                        zIndex: 100,
                        zIndexzIndexIncrease: -0.0001
                    }
                };

                var emitter = ig.game.spawnEntityWithSetup(fw);

                ig.game.runActivity(null,ig.activity.sequence([

                    ig.activity.pause(0.2),
                    ig.activity.runCallback(function(){
                        emitter.particleSetup.animated.animations.idle.frames = [2];
                        emitter.pos = {x: 500, y: 640};
                        emitter.reset();
                    }),
                    ig.activity.pause(0.35),
                    ig.activity.runCallback(function(){
                        emitter.particleSetup.animated.animations.idle.frames = [3];
                        emitter.pos = {x: 420, y: 270};
                        emitter.reset();
                    }),
                    ig.activity.pause(0.5),
                    ig.activity.runCallback(function(){
                        emitter.particleSetup.animated.animations.idle.frames = [4];
                        emitter.pos = {x: 250, y: 700};
                        emitter.reset();
                    }),
                    ig.activity.pause(0.3),
                    ig.activity.runCallback(function(){
                        emitter.particleSetup.animated.animations.idle.frames = [0];
                        emitter.pos = {x: 320, y: 480};
                        emitter.reset();
                    })
                ]));

            },
            assets: {
                images: {
                    pf_ColoredStars: 'pf_ColoredStars.png'
                }
            }
        };

    });

// lib/gamewheel/prefabs/prefabCollection.js
ig.baked=true;ig.module(
    'gamewheel.prefabs.prefabCollection'
).requires(
    'impact.impact',
    'gamewheel.core.content',
    'gamewheel.core.game',
    'gamewheel.prefabs.confettiPrefabs',
    'gamewheel.prefabs.balloonPrefabs',
    'gamewheel.prefabs.starPrefabs',
    'gamewheel.prefabs.rainPrefabs',
    'gamewheel.prefabs.fireworksPrefabs'
).defines(function() { "use strict";
        ig.Prefabs = ig.Prefabs || {};
        
        ig.Prefabs.gwMediaPath = ig.Prefabs.gwMediaPath || 'lib/gamewheel/media/';

        ig.Game.inject({
            spawnPrefab: function( type, settings ) {
                var prefabObject = typeof(type) === 'string'
                    ? ig.Prefabs[type]
                    : type;

                if( !prefabObject ) {
                    throw("Can't spawn prefab of type " + type);
                }

                // Merge in additional settings.
                if(prefabObject.settings && settings) {
                    ig.merge(prefabObject.settings, settings);
                }

                // Work out how to spawn.
                if(typeof(prefabObject.spawn) === 'function'){
                    // Prefab has its own way of spawning. Let it do it's thing. 
                    return prefabObject.spawn();
                } else {
                    // No spawn method so assume it's just like any other entity setup object.
                    return ig.game.spawnEntityWithSetup(prefabObject);
                }
            }
        });
        
        
        ig.Prefabs.collectAssets = function() {
            var prefabList = [];

            // Collect a list of prefab names that are being used by the game. 
            var scenes = ig.config['scenes'];
            for (var sceneName in scenes) {
                if (scenes.hasOwnProperty(sceneName)) {
                    var ss = scenes[sceneName]['settings'];

                    if(ss && ss['prefabs']) {
                        if(ss['prefabs'].constructor === Array) {
                            // Push multiple prefabs.
                            Array.prototype.push.apply(prefabList, ss['prefabs']);
                        } else if(ss['prefabs'].constructor === String) {
                            // Push single prefab.
                            prefabList.push(ss['prefabs']);
                        }
                    }
                }
            }
            
            var assets = {};

            // Now collect the assets required by those prefabs.
            for(var i = 0; i < prefabList.length; i++) {
                var pf = ig.Prefabs[prefabList[i]];

                if(pf && pf['assets']){
                    ig.merge(assets, pf['assets']);
                }
            }

            return assets;
        };

        // Need to load required assets for prefabs from gamewheel lib.
        // Note that ig.config.gwMediaPath should only exist when loading 
        // from a template (not compiled). Normally we default to media/.
        ig.content.load(ig.Prefabs.collectAssets(), (ig.baked ? ig.config.mediaPath : ig.Prefabs.gwMediaPath) || 'media/');
    });


// lib/gamewheel/entities/realText.js
ig.baked=true;ig.module(
    'gamewheel.entities.realText'
).requires(
    'impact.entity',
    'gamewheel.core.localization'
).defines(function(){ "use strict";
        
        // NOTE: This class does drawing direct to canvas and does not take scaling into account.
        
        ig.global.EntityRealText = ig.Entity.extend({
            
            align: 'left',			// alignment - left, center, right, start, end
            alpha: 1,				// opacity: 0 - transparent, 1 - opaque
            baseline: 'top',		// baseline - top, hanging, middle, alphabetic, ideographic, bottom. Ideally this should not be changed.
            color: '#FFFFFF',  		// text color
            _font: '20px Garamond',	// font settings as used by css font. Must match specification: https://developer.mozilla.org/en-US/docs/Web/CSS/font
            _text: '',				// text to draw
            wrapWidth: null,        // size in pixels a line can be before wrapping occurs. If null, text will not be wrapped
            textRect: {x: 0, y: 0, width: 0, height: 0},
            _lineHeight: 20,        // vertical spacing between lines of text. calculated from font style.
            centerOnPosX: false,    // centers the entire block of text horizontally on the entity's pos.
            centerOnPosY: false,    // centers the entire block of text vertically on the entity's pos.
            
            _linesToRender: [],     // text wrapped into lines that will be rendered.
            _drawOffset: {x: 0, y: 0}, // offsets drawing used to center the text on its position
            
            _initComplete: false,
            
            
            init: function( x, y, settings ) {

                // Define property for text so we can track when text changes.
                Object.defineProperty(this,"text",{
                    get: this._getText,
                    set: this._setText
                });

                // Define property for font so we can track when it changes.
                Object.defineProperty(this,"font",{
                    get: this._getFont,
                    set: this._setFont
                });

                // TODO: Implement change property for align.
                
                if(settings['textKey']) {
                    // Fall back to text property if nothing provided in language.
                    settings.text = ig.localization.textForKey(settings['textKey']) || settings.text || this._text;
                }
                
                // Apply settings.
                this.parent( x, y, settings );

                this._initComplete = true;
                this.wrapText();
            },
            
            _getText: function() {
                return this._text;
            },
            
            _setText: function(value) {
                this._text = value;
                this.wrapText();
            },

            _getFont: function() {
                return this._font;
            },

            _setFont: function(value) {
                this._font = value;
                this.wrapText();
            },
            
            _configureContext: function(context) {
                if(context === ig.system.context) {
                    // Use cache to only update font if it's necessary as updating
                    // the font on the canvas every draw is costly
                    if( !ig.game._fontCache || ig.game._fontCache !== this._font ){
                        ig.game._fontCache = this._font;
                        context.font = this._font;
                    }
                } else {
                    context.font = this._font;
                }
                context.textAlign = this.align;
                context.textBaseline = this.baseline;
                context.fillStyle = this.color;
            },
            
            
            wrapText: function() {
                if(!this._initComplete) {
                    // We don't want to run this method more than once during setup.
                    return;
                }
                
                // Calculate and assign the line height to use.
                this._lineHeight = this._getFontHeight(this._font);
                
                // Setup the context that will be used for measurement. No drawing will occur.
                var ctx = ig.system.context;
                this._configureContext(ctx);
                
                // Reset lines of text to be drawn. 
                this._linesToRender = [];
                
                var totalWidth = 0;
                var lines = this._text.split("\n");
                
                // Loop through lines we got from splitting on line feed.
                for (var i = 0; i < lines.length; i++) {
                    
                    var line = null;

                    // Do we have to do further wrapping?
                    if(this.wrapWidth) {
                        
                        // Split into words so we can build lines that fit the wrapping width.
                        var words = lines[i].split(' ');

                        for (var n = 0; n < words.length; n++) {
                            // Build the line. Only add words between lines. Extra spaces in the line are retained.  
                            var testLine = (line !== null ? line + " " : "") + words[n];
                            var testWidth = ctx.measureText(testLine).width;

                            if(testWidth <= this.wrapWidth || !line) {
                                // Line length is ok or this is the first word of a new line. Try add more.
                                line = testLine;
                                totalWidth = Math.max(totalWidth, testWidth);
                            } else {
                                // Latest word makes the line too long. Push line up until this point.
                                this._linesToRender.push(line);
                                // Reset the line and step back a word in the loop.
                                line = null;
                                n--;
                            }
                        }
                        
                    } else {
                        // No wrapping, but measure the line to get the width.
                        line = lines[i];

                        // Still need to measure as we're want to know the width of the widest line.
                        totalWidth = Math.max(totalWidth, ctx.measureText(line).width);
                    }
                    
                    this._linesToRender.push(line);
                }
                
                // Set the entity size to the size of the block of text.
                this.size = {x: totalWidth, y: this._linesToRender.length * this._lineHeight};
                
                var multiplierX = 0;
                if(this.centerOnPosX) {
                    // Note that this doesn't take start/end settings into account
                    if(this.align == 'center') {
                        multiplierX = 0;
                    } else if(this.align == 'right') {
                        multiplierX = 0.5;
                    } else {
                        // start is the default for align. Assume left aligned system.
                        multiplierX = -0.5;
                    }
                }
                
                // Calculate drawing offset. Allows centering of the entire block of text.
                this._drawOffset = {
                    x: totalWidth * multiplierX,
                    y: this.centerOnPosY ? this._linesToRender.length * this._lineHeight * -0.5 : 0
                    // y: this.centerOnPosY ? (this._linesToRender.length * this._lineHeight) * -0.5 + (this._lineHeight / 2) : 0
                };

                // Note that this doesn't take start/end settings into account
                var areaOffset = 0;
                if(this.align == 'center') {
                    areaOffset = totalWidth * -0.5
                } else if(this.align == 'right') {
                    areaOffset = -totalWidth
                }
                
                // Calculate and set rect that text is drawn in.
                // Note that this is not fully functional and may be incorrect depending 
                // on start/end align and baseline settings   
                this.textRect = {
                    x: this.pos.x + this._drawOffset.x + areaOffset,
                    y: this.pos.y + this._drawOffset.y,
                    width: totalWidth,
                    height: this._linesToRender.length * this._lineHeight
                };
            },
            

            draw: function () {
                this.parent();
                this.drawText(ig.system.context);
            },

            
            drawText: function(context) {

                context.save(); // comment this out

                this._configureContext(context);

                if(this.alpha != 1) {
                    // Set global alpha.
                    context.globalAlpha = this.alpha;
                }

                var x = this.pos.x + this._drawOffset.x;
                var y = this.pos.y + this._drawOffset.y;

                context.font = this._font;

                for (var i = 0; i < this._linesToRender.length; i++) {
                    context.fillText(this._linesToRender[i], x, y);
                    y += this._lineHeight;
                }

                if(this.alpha != 1) {
                    // Restore the global alpha.
                    context.globalAlpha = 1;
                }

                context.restore(); // comment this out

            },
            
            // This method won't work if css is not supported (i.e Ejecta)
            // http://videlais.com/2014/03/16/the-many-and-varied-problems-with-measuring-font-height-for-html5-canvas/
            _getFontHeight: function(fontStyle)
            {
                // Try get result from cache.
                var result = ig.global.EntityRealText.FontHeightCache[fontStyle];

                if (!result)
                {
                    var body = document.getElementsByTagName('body')[0];
                    var dummy = document.createElement('div');

                    var dummyText = document.createTextNode('M');
                    dummy.appendChild(dummyText);
                    dummy.setAttribute('style', "font: " + fontStyle + ';position:absolute;top:0;left:0');
                    body.appendChild(dummy);
                    result = dummy.offsetHeight;

                    // Cache result.
                    ig.global.EntityRealText.FontHeightCache[fontStyle] = result;
                    body.removeChild(dummy);
                }

                return result;
            }
            
        });

        ig.global.EntityRealText.FontHeightCache = {};
        

    });




// lib/gamewheel/helpers/easing.js
ig.baked=true;ig.module(
    'gamewheel.helpers.easing'
).defines(function() { "use strict";

        /**
         * Static utilities for general use.
         **/
        if(!ig.util) {
            ig.util = {};
        }
        
        // simple linear tweening - no easing, no acceleration
        ig.util.linearTween = function (time, start, change, duration) {
            return change*time/duration + start;
        };

        // quadratic easing in - accelerating from zero velocity
        ig.util.easeInQuad = function (time, start, change, duration) {
            time /= duration;
            return change*time*time + start;
        };

        // quadratic easing out - decelerating to zero velocity
        ig.util.easeOutQuad = function (time, start, change, duration) {
            time /= duration;
            return -change * time*(time-2) + start;
        };

        // quadratic easing in/out - acceleration until halfway, then deceleration
        ig.util.easeInOutQuad = function (time, start, change, duration) {
            time /= duration/2;
            if (time < 1) return change/2*time*time + start;
            time--;
            return -change/2 * (time*(time-2) - 1) + start;
        };

        // cubic easing in - accelerating from zero velocity
        ig.util.easeInCubic = function (time, start, change, duration) {
            time /= duration;
            return change*time*time*time + start;
        };

        // cubic easing out - decelerating to zero velocity
        ig.util.easeOutCubic = function (time, start, change, duration) {
            time /= duration;
            time--;
            return change*(time*time*time + 1) + start;
        };

        // cubic easing in/out - acceleration until halfway, then deceleration
        ig.util.easeInOutCubic = function (time, start, change, duration) {
            time /= duration/2;
            if (time < 1) return change/2*time*time*time + start;
            time -= 2;
            return change/2*(time*time*time + 2) + start;
        };

        // quartic easing in - accelerating from zero velocity
        ig.util.easeInQuart = function (time, start, change, duration) {
            time /= duration;
            return change*time*time*time*time + start;
        };

        // quartic easing out - decelerating to zero velocity
        ig.util.easeOutQuart = function (time, start, change, duration) {
            time /= duration;
            time--;
            return -change * (time*time*time*time - 1) + start;
        };

        // quartic easing in/out - acceleration until halfway, then deceleration
        ig.util.easeInOutQuart = function (time, start, change, duration) {
            time /= duration/2;
            if (time < 1) return change/2*time*time*time*time + start;
            time -= 2;
            return -change/2 * (time*time*time*time - 2) + start;
        };

        // quintic easing in - accelerating from zero velocity
        ig.util.easeInQuint = function (time, start, change, duration) {
            time /= duration;
            return change*time*time*time*time*time + start;
        };

        // quintic easing out - decelerating to zero velocity
        ig.util.easeOutQuint = function (time, start, change, duration) {
            time /= duration;
            time--;
            return change*(time*time*time*time*time + 1) + start;
        };

        // quintic easing in/out - acceleration until halfway, then deceleration
        ig.util.easeInOutQuint = function (time, start, change, duration) {
            time /= duration/2;
            if (time < 1) return change/2*time*time*time*time*time + start;
            time -= 2;
            return change/2*(time*time*time*time*time + 2) + start;
        };

        // sinusoidal easing in - accelerating from zero velocity
        ig.util.easeInSine = function (time, start, change, duration) {
            return -change * Math.cos(time/duration * (Math.PI/2)) + change + start;
        };

        // sinusoidal easing out - decelerating to zero velocity
        ig.util.easeOutSine = function (time, start, change, duration) {
            return change * Math.sin(time/duration * (Math.PI/2)) + start;
        };

        // sinusoidal easing in/out - accelerating until halfway, then decelerating
        ig.util.easeInOutSine = function (time, start, change, duration) {
            return -change/2 * (Math.cos(Math.PI*time/duration) - 1) + start;
        };

        // exponential easing in - accelerating from zero velocity
        ig.util.easeInExpo = function (time, start, change, duration) {
            return change * Math.pow( 2, 10 * (time/duration - 1) ) + start;
        };

        // exponential easing out - decelerating to zero velocity
        ig.util.easeOutExpo = function (time, start, change, duration) {
            return change * ( -Math.pow( 2, -10 * time/duration ) + 1 ) + start;
        };

        // exponential easing in/out - accelerating until halfway, then decelerating
        ig.util.easeInOutExpo = function (time, start, change, duration) {
            time /= duration/2;
            if (time < 1) return change/2 * Math.pow( 2, 10 * (time - 1) ) + start;
            time--;
            return change/2 * ( -Math.pow( 2, -10 * time) + 2 ) + start;
        };

        // circular easing in - accelerating from zero velocity
        ig.util.easeInCirc = function (time, start, change, duration) {
            time /= duration;
            return -change * (Math.sqrt(1 - time*time) - 1) + start;
        };

        // circular easing out - decelerating to zero velocity
        ig.util.easeOutCirc = function (time, start, change, duration) {
            time /= duration;
            time--;
            return change * Math.sqrt(1 - time*time) + start;
        };

        // circular easing in/out - acceleration until halfway, then deceleration
        ig.util.easeInOutCirc = function (time, start, change, duration) {
            time /= duration/2;
            if (time < 1) return -change/2 * (Math.sqrt(1 - time*time) - 1) + start;
            time -= 2;
            return change/2 * (Math.sqrt(1 - time*time) + 1) + start;
        };

    });

// lib/gamewheel/core/activity.js
ig.baked=true;ig.module(
    'gamewheel.core.activity'
).requires(
    'impact.entity',
    'gamewheel.helpers.util',
    'gamewheel.helpers.easing',
    'impact.game'
).defines(function() {
        
        ig.activity = ig.activity || {};

        ig.Game.inject({

            _activities: [],

            // Adds an activity to the activity stack to be updated each frame.
            runActivity: function (entity, activity, name) {
                this._activities.push({
                    entity: entity,
                    activity: activity,
                    name: name
                })
            },

            // Removes all activities that match the given name and entity.
            removeActivity: function (entity, name) {
                // Loop through all running activities.
                for (var i = 0; i < this._activities.length; i++) {
                    var a = this._activities[i];

                    var match = false;
                    if (name && entity) {
                        // Both name and entity must match.
                        match = name === a.name && entity === a.entity;
                    } else {
                        // Only one of entity or name provided, match on provided property.
                        match = (name && a.name === name) || (entity && a.entity === entity);
                    }

                    if (match) {
                        this._activities.splice(i, 1);
                        i--;
                    }
                }
            },

            stopActivity : function( activity ) {
                this._activities.splice( this._activities.indexOf( activity ), 1 );
            },

            updateEntities: function () {
                this.parent();
                // Update activities.
                for (var i = 0; i < this._activities.length; i++) {
                    var a = this._activities[i];
                    if (a.activity.complete || a.entity && a.entity._killed) {
                        // Activity complete, or entity that activity is attached to is killed, remove it. 
                        this._activities.splice(i, 1);
                        i--;
                    } else {
                        a.activity.update();
                    }
                }
            }

        });
        
        
        ig.activity = {};
        
        // Calls provided callback when run. 
        ig.activity.runCallback = function (callback) {
            return {
                complete: false,
                callback: callback,

                reset: function () {
                    this.complete = false;
                },

                update: function () {
                    if (!this.complete) {
                        this.callback();
                        this.complete = true;
                    }
                }
            }
        };
        
        // Runs an activity a number of times. If playCount is null or 0, loops forever.
        ig.activity.repeat = function (activity, playCount) {
            return {
                complete: false,
                activity: activity,
                playCount: playCount, // Loops forever when 0 or undefined.
                loops: 0,

                reset: function () {
                    this.complete = false;
                    this.loops = 0;
                },

                update: function () {
                    if (!playCount || this.loops < this.playCount) {
                        this.activity.update();
                        if (this.activity.complete) {
                            this.activity.reset();
                            this.loops += 1;
                        }
                    }
                    this.complete = !(!playCount || this.loops < this.playCount);
                }
            }
        };
        
        // Runs multiple activities at the same time. Only completes once ALL activities are complete.
        ig.activity.group = function (activities) {
            return {
                complete: false,
                group: activities,

                update: function () {
                    var allComplete = true;
                    for (var i = 0; i < this.group.length; i++) {
                        this.group[i].update();
                        allComplete = allComplete && this.group[i].complete;
                    }
                    this.complete = allComplete;
                },

                reset: function () {
                    for (var i = 0; i < this.group.length; i++) {
                        this.group[i].reset();
                    }
                    this.complete = false;
                }
            }
        };

        // Returns a repeating group activity. Repeats forever when playCount is 0.
        ig.activity.repeatGroup = function(activities, playCount) {
            return ig.activity.repeat(ig.activity.group(activities), playCount);
        };

        // Runs multiple activities in one after another.
        ig.activity.sequence = function (activities) {
            return {
                complete: false,
                sequence: activities,
                sequencePos: 0,

                update: function () {
                    if (this.sequencePos < this.sequence.length) {
                        this.sequence[this.sequencePos].update();
                        if (this.sequence[this.sequencePos].complete) {
                            this.sequencePos++;
                        }
                    }
                    this.complete = this.sequencePos >= this.sequence.length;
                },

                reset: function () {
                    this.sequencePos = 0;
                    for (var i = 0; i < this.sequence.length; i++) {
                        this.sequence[i].reset();
                    }
                    this.complete = false;
                }
            };
        };

        // Returns a repeating sequence activity. Repeats forever when playCount is 0.
        ig.activity.repeatSequence = function(activities, playCount) {
            return ig.activity.repeat(ig.activity.sequence(activities), playCount);
        };

        // Waits for the specified amount of time before completing.
        ig.activity.pause = function (duration) {
            return {
                duration: duration,
                elapsed: 0,
                complete: duration <= 0,

                reset: function () {
                    this.elapsed = 0;
                    this.complete = duration <= 0;
                },

                update: function () {
                    this.elapsed = Math.min(this.elapsed + ig.system.tick, this.duration);
                    this.complete = (this.elapsed >= this.duration);
                }
            }
        };
        
        // Runs kill() on the specified entity.
        ig.activity.killEntity = function (entity) {
            return {
                complete: false,
                entity: entity,

                reset: function () {
                    this.complete = false;
                },

                update: function () {
                    if (!this.complete) {
                        this.entity.kill();
                        this.complete = true;
                    }
                }
            }
        };

        // Progressively sets currentAnim.alpha for <entity> to approach <alpha> over <duration> seconds.
        ig.activity.alphaTo = function (entity, alpha, duration, easing) {
            return ig.activity.changeTo(entity, 'currentAnim.alpha', alpha, duration, easing);
        };

        // Progressively updates currentAnim.angle for <entity> to change by <angle> over <duration> seconds.
        ig.activity.rotateBy = function (entity, amount, duration, easing) {
            return ig.activity.changeBy(entity, 'currentAnim.angle', amount, duration, easing);
        };

        // Progressively sets currentAnim.angle for <entity> to approach <angle> over <duration> seconds.
        ig.activity.rotateTo = function (entity, angle, duration, easing) {
            return ig.activity.changeTo(entity, 'currentAnim.angle', angle, duration, easing);
        };

        // Progressively sets pos.x and pos.y for <entity> to approach <x, y> over <duration seconds.
        ig.activity.moveTo = function (entity, x, y, duration, easing) {

            var moves = [];
            if (x != 0) {
                moves.push(ig.activity.moveToX(entity, x, duration, easing));
            }
            if (y != 0) {
                moves.push(ig.activity.moveToY(entity, y, duration, easing));
            }

            if (moves.length > 1) {
                return ig.activity.group(moves);
            } else if (moves.length == 1) {
                return moves[0];
            } else {
                return ig.activity.pause(duration);
            }
        };

        // Progressively sets pos.x for <entity> to approach <position> over <duration> seconds.
        ig.activity.moveToX = function (entity, position, duration, easing) {
            return ig.activity.changeTo(entity, 'pos.x', position, duration, easing);
        };

        // Progressively sets pos.y for <entity> to approach <position> over <duration> seconds.
        ig.activity.moveToY = function (entity, position, duration, easing) {
            return ig.activity.changeTo(entity, 'pos.y', position, duration, easing);
        };

        // Progressively updates pos.x and pos.y for <entity> to change by <x, y> over <duration> seconds.
        ig.activity.moveBy = function (entity, x, y, duration, easing) {

            var moves = [];
            if (x != 0) {
                moves.push(ig.activity.moveByX(entity, x, duration, easing));
            }
            if (y != 0) {
                moves.push(ig.activity.moveByY(entity, y, duration, easing));
            }

            if (moves.length > 1) {
                return ig.activity.group(moves);
            } else if (moves.length == 1) {
                return moves[0];
            } else {
                return ig.activity.pause(duration);
            }
        };

        // Progressively updates pos.x for <entity> to change by <amount> over <duration> seconds.
        ig.activity.moveByX = function (entity, amount, duration, easing) {
            return ig.activity.changeBy(entity, 'pos.x', amount, duration, easing);
        };

        // Progressively updates pos.y for <entity> to change by <amount> over <duration> seconds.
        ig.activity.moveByY = function (entity, amount, duration, easing) {
            return ig.activity.changeBy(entity, 'pos.y', amount, duration, easing);
        };

        // Progressively sets scale.x and scale.y for <entity> to approach <value> over <duration> seconds.
        ig.activity.scaleTo = function (entity, value, duration, easing) {
            return ig.activity.group([
                ig.activity.changeTo(entity, 'scale.x', value, duration, easing),
                ig.activity.changeTo(entity, 'scale.y', value, duration, easing)
            ]);
        };

        // Progressively sets scale.x for <entity> to approach <value> over <duration> seconds.
        ig.activity.scaleToX = function (entity, value, duration, easing) {
            return ig.activity.changeTo(entity, 'scale.x', value, duration, easing);
        };

        // Progressively sets scale.y for <entity> to approach <value> over <duration> seconds.
        ig.activity.scaleToY = function (entity, value, duration, easing) {
            return ig.activity.changeTo(entity, 'scale.y', value, duration, easing);
        };

        // Progressively sets scale.x and scale.y for <entity> to approach <value> over <duration> seconds.
        ig.activity.scaleBy = function (entity, value, duration, easing) {
            return ig.activity.group([
                ig.activity.changeBy(entity, 'scale.x', value, duration, easing),
                ig.activity.changeBy(entity, 'scale.y', value, duration, easing)
            ]);
        };

        // Progressively sets scale.x for <entity> to approach <value> over <duration> seconds.
        ig.activity.scaleByX = function (entity, value, duration, easing) {
            return ig.activity.changeBy('scale.x', value, duration, easing);
        };

        // Progressively sets scale.y for <entity> to approach <value> over <duration> seconds.
        ig.activity.scaleByY = function (entity, value, duration, easing) {
            return ig.activity.changeBy('scale.y', value, duration, easing);
        };

        ig.activity.delay = function(duration, activity) {
            return ig.activity.sequence([
                ig.activity.pause(duration),
                activity
            ]);
        };


            // Progressively sets a value on an object as defined by propPath.
        ig.activity.changeTo = function (object, propPath, value, duration, easing) {
            return {
                object: object,
                value: value,
                duration: duration,
                animator: undefined,
                started: false,
                complete: false,
                easing: easing,

                reset: function () {
                    this.started = false;
                    this.complete = false;
                },

                init: function () {
                    this.started = true;
                    var currentVal = ig.util.objectByString(this.object, propPath);
                    this.animator = ig.activity.valueChange(
                        currentVal, this.value - currentVal, this.duration, this.easing
                    )
                },

                update: function () {
                    if (!this.started) {
                        this.init();
                    }
                    this.animator.update();
                    ig.util.setObjectByString(this.object, propPath, this.animator.value);
                    this.complete = this.animator.complete;
                }
            }
        };

        // Progressively updates a value on an object as defined by propPath.
        ig.activity.changeBy = function (object, propPath, distance, duration, easing) {
            return {
                object: object,
                distance: distance,
                duration: duration,
                animator: undefined,
                easing: easing,

                started: false,
                complete: false,

                reset: function () {
                    this.started = false;
                    this.complete = false;
                },

                init: function () {
                    this.started = true;
                    this.animator = ig.activity.valueChange(0, this.distance, this.duration, this.easing)
                },

                update: function () {
                    if (!this.started) {
                        this.init();
                    }
                    var val = this.animator.value;
                    this.animator.update();
                    var currentVal = ig.util.objectByString(this.object, propPath);
                    ig.util.setObjectByString(this.object, propPath, currentVal + (this.animator.value - val));
                    this.complete = this.animator.complete;
                }
            }
        };

        // Changes value from <start> to <start + change> over duration seconds.
        ig.activity.valueChange = function (start, change, duration, easing) {
            return {
                elapsed: 0,
                start: start,
                change: change,
                duration: duration,
                easing: easing || ig.util.linearTween,
                value: start,
                complete: false,

                reset: function () {
                    this.value = this.start;
                    this.elapsed = 0;
                    this.complete = false;
                },

                update: function () {
                    this.elapsed = Math.min(this.elapsed + ig.system.tick, this.duration);
                    this.value = this.easing(this.elapsed, this.start, this.change, this.duration);
                    this.complete = (this.elapsed >= this.duration);
                }
            }
        };

    });

// lib/gamewheel/entities/leaderboardDisplay.js
ig.baked=true;ig.module(
    'gamewheel.entities.leaderboardDisplay'
).requires(
    'gamewheel.entities.realText',
    'gamewheel.helpers.util'
).defines(function() {
        
        ig.global.EntityLeaderboardDisplay = ig.Entity.extend({
            
            size: {x: 560, y: 320},
            
            font: "400 30px \"Open Sans\", sans-serif",
            fontColor: "#505050",
            fontCurrent: "400 30px \"Open Sans\", sans-serif",
            fontCurrentColor: "#FFFFFF",
            headerFont: "400 35px \"Open Sans\", sans-serif",
            headerFontColor: "#FFFFFF",

            strings: {
                error: "Unable to load leaderboard.",
                loading: "Loading...",
                rank: "Rank",
                score: "Score",
                player: "Name"
            },

            scoreDisplayString: "%S",
            scoreFixedLen: 0,
            scoresToShow: 5,
            
            leaderboardData: [],
            
            _messageLabel: undefined,
            _textFields: [],

            init: function(x, y, settings) {
                this.parent(x, y, settings);
                if(!settings['loadTestData']) {
                    this.loadData();
                } else {
                    if(settings['loadTestData'] instanceof Array) {
                        // Assume specific test data provided.
                        this.leaderboardData = settings['loadTestData'];
                    } else {
                        // Generate some random data to display.
                        var s = ig.util.randomInt(375, 425);
                        for (var i = 0; i < this.scoresToShow; i++) {
                            this.leaderboardData.push({
                                rank: i + 1,
                                score: s,
                                player: ['NinjaFive', 'Galaxy', 'Rayne', 'Hyjinx', 'Flux', 'KCValue', 'nSphelt'].random()
                            });
                            s -= ig.util.randomInt(1, 25);
                        }
                        this.leaderboardData.random().currentPlayer = true;
                    }
                    
                    this.loaded(true);
                }
            },
            
            checkSession: function(fn) {
                // Normalize callback
                fn = ig.apiUtil.normalizeFunction(fn);
                // Self-reference
                var self = this;

                // Check if already got session token
                if (ig.settings.tokens.api_runtime_session) return fn.call(this, true);

                // Force auth
                this.checkAuth(function(success) {

                    // Check if not success
                    if (!success) return fn.call(self, false);

                    // Create session
                    ig.apiUtil.createSession(fn);
                });
            },

            checkAuth: function(fn) {

                // Normalize callback
                fn = ig.apiUtil.normalizeFunction(fn);

                // Check if already got auth token
                if (ig.settings.tokens.api_runtime) return fn.call(this, true);

                // Authenticate
                ig.apiUtil.authenticate(fn);
            },

            loadData:function(){

                // Show loading message.
                this._messageLabel = this.genLabel(this.strings.loading, this.font, this.fontColor);
                this._messageLabel.align = 'center';
                this._messageLabel.pos = {
                    x: this.pos.x + this.size.x / 2,
                    y: this.pos.y + this.size.y / 2
                };

                this.checkSession(function(success){

                    if(!success) {
                        this.loaded(false);
                    }

                    var dataToSend = {
                        session_token: ig.settings.tokens.api_runtime_session,
                        scoreCount: this.scoresToShow
                    };

                    ig.apiUtil.xhr('/highscore', 'POST', null, dataToSend, function(status, resData) {
                        if (status !== 200 && status !== 201) {
                            this.loaded(false);
                        } else {
                            this.leaderboardData = resData;
                            this.loaded(true);
                        }
                    }.bind(this));

                }.bind(this));

            },

            loaded: function(success){

                if(this._messageLabel) {
                    this._messageLabel.kill();
                }

                if(success) {
                    this.displayScores();
                } else {
                    this._messageLabel = this.genLabel(this.strings.error, this.font, this.fontColor);
                    this._messageLabel.align = 'center';
                    this._messageLabel.pos = {
                        x: this.pos.x + this.size.x / 2,
                        y: this.pos.y + this.size.y / 2
                    };
                }
            },

            displayScores: function(){
                var rankText = '';
                var rankTextCurrent = '';
                var scoreText = '';
                var scoreTextCurrent = '';
                var playerText = '';
                var playerTextCurrent = '';

                for(var i = 0; i < this.scoresToShow; i++){
                    var entry = this.leaderboardData[i];
                    if(entry){
                        if(entry['currentPlayer']){
                            rankTextCurrent += entry.rank.toString() + '\n';
                            scoreTextCurrent += this.formatScore(entry.score) + '\n';
                            playerTextCurrent += entry.player + '\n';
                            rankText += '\n';
                            scoreText += '\n';
                            playerText += '\n';
                        } else {
                            rankText += entry.rank.toString() + '\n';
                            scoreText += this.formatScore(entry.score) + '\n';
                            playerText += entry.player + '\n';
                            rankTextCurrent += '\n';
                            scoreTextCurrent += '\n';
                            playerTextCurrent += '\n';
                        }
                    } else {
                        rankText += '...\n';
                        scoreText += '...\n';
                        playerText += '...\n';
                        rankTextCurrent += '\n';
                        scoreTextCurrent += '\n';
                        playerTextCurrent += '\n';
                    }
                }

                // Generate the headers.
                var rankHeader = this.genLabel(this.strings.rank, this.headerFont, this.headerFontColor);
                var scoreHeader = this.genLabel(this.strings.score, this.headerFont, this.headerFontColor);
                var playerHeader = this.genLabel(this.strings.player, this.headerFont, this.headerFontColor);

                // Generate the labels.
                var rankLabel = this.genLabel(rankText, this.font, this.fontColor);
                var scoreLabel = this.genLabel(scoreText, this.font, this.fontColor);
                var playerLabel = this.genLabel(playerText, this.font, this.fontColor);
                var rankCurrentLabel = this.genLabel(rankTextCurrent, this.fontCurrent, this.fontCurrentColor);
                var scoreCurrentLabel = this.genLabel(scoreTextCurrent, this.fontCurrent, this.fontCurrentColor);
                var playerCurrentLabel = this.genLabel(playerTextCurrent, this.fontCurrent, this.fontCurrentColor);

                // Set alignment for columns.
                rankHeader.align = 'center';
                rankLabel.align = 'center';
                rankCurrentLabel.align = 'center';
                scoreHeader.align = 'center';
                scoreLabel.align = 'right';
                scoreCurrentLabel.align = 'right';

                // Calculate spacing.
                var rankWidth = Math.max(rankHeader.size.x, rankLabel.size.x, rankCurrentLabel.size.x);
                var scoreWidth = Math.max(scoreHeader.size.x, scoreLabel.size.x, scoreCurrentLabel.size.x);
                var playerWidth = Math.max(playerHeader.size.x, playerLabel.size.x, playerCurrentLabel.size.x);
                var totalWidth = rankWidth + scoreWidth + playerWidth;
                var gap = Math.max((this.size.x - totalWidth) / 2, 15); // Fallback spacing of 15.

                // Column starting positions.
                var col1 = this.pos.x;
                var col2 = col1 + rankWidth + gap;
                var col3 = col2 + scoreWidth + gap;

                // Position headers.
                rankHeader.pos = {x: col1 + rankWidth / 2, y: this.pos.y};
                scoreHeader.pos = {x: col2 + scoreWidth / 2, y: rankHeader.pos.y};
                playerHeader.pos = {x: col3, y: scoreHeader.pos.y};

                // Position labels.
                rankLabel.pos = {x: rankHeader.pos.x, y: rankHeader.pos.y + rankHeader.size.y + 5};
                rankCurrentLabel.pos = rankLabel.pos;
                scoreLabel.pos = {x: col2 + scoreWidth, y: rankLabel.pos.y};
                scoreCurrentLabel.pos = scoreLabel.pos;
                playerLabel.pos = {x: col3, y: rankLabel.pos.y};
                playerCurrentLabel.pos = playerLabel.pos;
            },

            genLabel: function(text, font, color){
                var label = ig.game.spawnEntityWithSetup({
                    type: 'EntityRealText',
                    settings: {
                        text: text,
                        color: color,
                        font: font
                    }
                });
                this._textFields.push(label);
                return label;
            },

            formatScore: function(score){
                return this.scoreDisplayString.replace('%S', score.toFixed(this.scoreFixedLen));
            },
            
            kill: function(){
                this.parent();
                
                this._textFields.forEach(function(field){
                    field.kill();
                });
            }
            
        });

    });

// lib/gamewheel/entities/button.js
ig.baked=true;ig.module(
    'gamewheel.entities.button'
).requires(
    'impact.system',
    'impact.entity'
).defines(function () { "use strict";

        ig.ui = ig.ui || {};

        // TODO: Is this the best spot for this override?
        ig.System.inject({
            // When changing the game class, first unbind all input.
            setGame: function( gameClass ) {
                ig.input.unbindAll();
                if(ig.input.touchManager) {
                    ig.input.touchManager.clear();
                }
                this.parent(gameClass);
            }
        });

        ig.global.EntityButton = ig.Entity.extend({

            hidden: false,
            canBePressed : true,

            pressing: false,
            pressingLast: false,
            wasBeingPressed : false,

            triggerCallback: undefined,
            triggerAction: undefined,
            onPress: undefined,
            
            pressSound: undefined,
            pressedScale: {x: 0.93, y: 0.93},
            
            scale: {x: 1, y: 1},
            scalePoint: null,

            touchId: null,

            highlight : undefined,

            init: function (x, y, settings) {
                this.parent(x, y, settings);
                this.setAnim('normal');

                if(!ig.input.touchManager) {
                    ig.input.touchManager = new ig.ui.TouchManager();
                }

                ig.input.touchManager.buttons.push(this);

                if( this.highlight ) {
                    var activity = ig.game.activityScripts[this.highlight];
                    console.log("activity", activity);
                    if( activity ) {
                        ig.game.runActivity( this, activity(this), "button highlight");
                    }
                }
            },

            kill: function() {
                // Remove from the touch manager when killed.
                ig.input.touchManager.buttons.erase(this);
                this.parent();
            },

            // TODO: The reactions to events might need to be coupled to the update method so that buttons don't react if they're not updated.

            touchStart: function( id, x, y ) {
                if( this.canBePressed ) {
                    if(!this.hidden && this.locationInButton(x, y)) {
                        this.touchId = id;
                        this.pressing = true;
                        return true;
                    }
                }
                return false;
            },

            touchMove: function( id, x, y ) {
                if( this.canBePressed ) {
                    if (!this.hidden && this.touchId == id) {
                        this.pressing = this.locationInButton(x, y);
                    }
                }
            },

            touchEnd: function( id, x, y ) {
                if( this.canBePressed ) {
                    if (this.touchId == id) {
                        this.touchId = null;
                        this.pressing = false;

                        if (!this.hidden && this.locationInButton(x, y)) {
                            // Clicked the button.
                            if (this.triggerCallback) {
                                this.triggerCallback();
                            }

                            // Play sound if defined.
                            if (this.pressSound) {
                                ig.content.playSoundNamed(this.pressSound);
                            }

                            // Perform action if defined.
                            if (this.onPress) {
                                ig.actions.perform(this.onPress, this);
                            }
                        }
                    }
                }
            },

            locationInButton: function(x, y) {
                return x > this.pos.x &&
                       x < this.pos.x + this.size.x &&
                       y > this.pos.y &&
                       y < this.pos.y + this.size.y;
            },

            update: function () {
                //if( this.canBePressed )
                {
                    this.parent();

                    // Check for state change.
                    if (this.pressing != this.pressingLast) {
                        if (this.pressing) {
                            this.setAnim('pressing');
                        } else {
                            this.setAnim('normal');
                        }
                    }

                    this.wasBeingPressed = this.pressingLast;
                    this.pressingLast = this.pressing;
                }
            },


            draw: function() {
                // TODO: Scaling has been added as an extension but this class manages its own scaling.
                // TODO: This is simply to avoid forcing an import of the extension which may break other games. 
                
                if(!this.hidden && this.currentAnim) {

                    var sx = this.scale.x;
                    var sy = this.scale.y;

                    if(this.pressing) {
                        sx = sx * this.pressedScale.x;
                        sy = sy * this.pressedScale.y;
                    }

                    if(sx != 1 || sy != 1) {
                        sx = sx / ig.system.scale;
                        sy = sy / ig.system.scale;

                        if(!this.scalePoint) {
                            // Default to scale from the center.
                            this.scalePoint = {
                                x: this.currentAnim.sheet.width / 2,
                                y: this.currentAnim.sheet.height / 2
                            }
                        }

                        var ox = this.offset.x + (this.scalePoint.x * sx) - this.scalePoint.x;
                        var oy = this.offset.y + (this.scalePoint.y * sy) - this.scalePoint.y;
                        
                        // We do our own drawing so we can include the scaling effect.
                        var ctx = ig.system.context;
                        ctx.save();
                        ctx.translate(
                            ig.system.getDrawPos(this.pos.x.round() - ox - ig.game._rscreen.x),
                            ig.system.getDrawPos(this.pos.y.round() - oy - ig.game._rscreen.y)
                        );
                        ctx.scale(sx, sy);
                        this.currentAnim.draw(0, 0);
                        ctx.restore();
                    } else {
                        this.parent();
                    }
                }
            },
            
            /* TODO: We can simplify the draw method to this once backward compatibility with all games is confirmed.
               TODO: Remember to remove scale and scalePoint definitions at the top.
            draw: function() {
                if(!this.hidden && this.currentAnim) {
                    if(this.pressing) {
                        var sx = this.scale.x;
                        var sy = this.scale.y;

                        this.scale.x = sx * this.pressedScale.x;
                        this.scale.y = sy * this.pressedScale.y;

                        this.parent();

                        this.scale.x = sx;
                        this.scale.y = sy;
                    } else {
                        this.parent();
                    }
                }
            },
            */

            setAnim: function(name) {
                if(this.anims[name] && this.currentAnim !== this.anims[name]) {
                    this.currentAnim = this.anims[name];
                    return true;
                }
                return false;
            }

        });


        ig.ui.TouchManager = ig.Class.extend({
            buttons: [],
            controllers : [],

            init: function( ) {

                document.addEventListener('touchstart', this.touchStart.bind(this), false);
                document.addEventListener('touchmove', this.touchMove.bind(this), false);
                document.addEventListener('touchend', this.touchEnd.bind(this), false);
                document.addEventListener('touchcancel', this.touchEnd.bind(this), false);

                document.addEventListener('MSPointerDown', this.touchStart.bind(this), false);
                document.addEventListener('MSPointerUp', this.touchEnd.bind(this), false);
                document.addEventListener('MSPointerMove', this.touchMove.bind(this), false);

                document.addEventListener('mousedown', this.touchStart.bind(this), false);
                document.addEventListener('mouseup', this.touchEnd.bind(this), false);
                document.addEventListener('mousemove', this.touchMove.bind(this), false);

                document.body.style.msTouchAction = 'none';
            },

            clear: function() {
                this.buttons = [];
            },


            getTouches: function(ev) {
                var internalWidth = parseInt(ig.system.canvas.offsetWidth) || ig.system.realWidth;
                var scale = ig.system.scale * (internalWidth / ig.system.realWidth);

                var pos = {left: 0, top: 0};
                if( ig.system.canvas.getBoundingClientRect ) {
                    pos = ig.system.canvas.getBoundingClientRect();
                }

                var touches = [];

                if(ev.changedTouches) {
                    // Loop through all the touches.
                    for( var i = 0; i < ev.changedTouches.length; i++ ) {
                        var touch = ev.changedTouches[i];

                        touches.push({
                            id: touch.identifier,
                            x: (touch.clientX - pos.left) / scale,
                            y: (touch.clientY - pos.top) / scale
                        })
                    }
                } else {
                    touches.push({
                        id: 0,
                        x: (ev.clientX - pos.left) / scale,
                        y: (ev.clientY - pos.top) / scale
                    })
                }

                return touches;
            },


            touchStart: function(ev) {
                ev.preventDefault();

                // Ignore if mouse and not clicking left mouse button.
                if(!ev.changedTouches && ev.button !== 0) {
                    return;
                }

                // Get and loop through touches that have just started.
                var touches = this.getTouches(ev);
                for( var i = 0; i < touches.length; i++ ) {
                    // Loop through all buttons until. Break if we find one that responds to the event.
                    for( var b = this.buttons.length - 1; b >= 0; b-- ) {
                        if(this.buttons[b].touchStart(touches[i].id, touches[i].x, touches[i].y)) {
                            break;
                        }
                    }

                    // Loop through all controllers until. Break if we find one that responds to the event.
                    for( var c = this.controllers.length - 1; c >= 0; c-- ) {
                        if(this.controllers[c].touchStart(touches[i].id, touches[i].x, touches[i].y)) {
                            break;
                        }
                    }
                }
            },

            touchMove: function(ev) {
                ev.preventDefault();
                // Get and loop through touches that have just started.
                var touches = this.getTouches(ev);
                for( var i = 0; i < touches.length; i++ ) {
                    // Loop through all buttons until. Break if we find one that responds to the event.
                    for( var b = 0; b < this.buttons.length; b++ ) {
                        if(this.buttons[b].touchMove(touches[i].id, touches[i].x, touches[i].y)) {
                            break;
                        }
                    }

                    // Loop through all controllers until. Break if we find one that responds to the event.
                    for( var c = this.controllers.length - 1; c >= 0; c-- ) {
                        if(this.controllers[c].touchMove(touches[i].id, touches[i].x, touches[i].y)) {
                            break;
                        }
                    }
                }
            },

            touchEnd: function(ev) {
                ev.preventDefault();
                // Get and loop through touches that have ended.
                var touches = this.getTouches(ev);
                for( var i = 0; i < touches.length; i++ ) {
                    // Loop through all buttons until. Break if we find one that responds to the event.
                    for( var b = 0; b < this.buttons.length; b++ ) {
                        this.buttons[b].touchEnd( touches[i].id, touches[i].x, touches[i].y);
                    }

                    // Loop through all controllers until. Break if we find one that responds to the event.
                    for( var c = this.controllers.length - 1; c >= 0; c-- ) {
                        if(this.controllers[c].touchEnd(touches[i].id, touches[i].x, touches[i].y)) {
                            break;
                        }
                    }
                }
            }

        });


    });


// lib/gamewheel/scenes/baseTemplateScene.js
ig.baked=true;ig.module(
    'gamewheel.scenes.baseTemplateScene'
).requires(
    'gamewheel.scenes.basicScene',
    'gamewheel.prefabs.prefabCollection',
    'gamewheel.entities.realText',
    'gamewheel.core.analytics',
    'gamewheel.core.activity',
    'gamewheel.entities.leaderboardDisplay',
    'gamewheel.entities.button'
).defines(function(){ "use strict";

        ig.BaseTemplateScene = ig.BasicScene.extend({

            backgroundImage: null,
            
            headerHeight: 160,
            headerColor: null,
            headerImage: null,
            
            _headerMessage: undefined,
            headerMessageText: null,
            headerMessageLocation: null,
            headerFontSettings: {},
            headerMessageDefaults: {
                "gravityFactor" : 0,
                "color": "#505050",
                "align": "center",
                "font": "400 50px \"Open Sans\", sans-serif",
                "wrapWidth": 620,
                centerOnPosX: true,
                centerOnPosY: true
            },
            displayScoreInHeader: false,
            scoreDisplayFn: function(score){ return score; },

            footerHeight: 160,
            footerColor: null,
            footerImage: null,
            
            _message: undefined,
            messageText: null,
            messageLocation: null,
            messageFontSettings: {},
            messageDefaults: {
                "color": "#505050",
                "align": "center",
                "font": "700 80px \"Open Sans\", sans-serif",
                "wrapWidth": 620,
                centerOnPosX: true,
                centerOnPosY: true
            },
            
            _mainButton: undefined,
            mainButtonImage: null,
            mainButtonLocation: null,
            mainButtonAction: {action: "endScene", args: null},

            _ctaButton: undefined,
            ctaButtonImage: null,
            ctaButtonLocation: null,
            ctaButtonAction: {action: "endScene", args: null, analyticsEvent: ig.ANALYTICS_EVENT.PRESSED_CTA},
            
            cornerButtonMargin: 10,

            _helpButton: undefined,
            helpButtonImage: null,
            helpButtonAction: {action: 'goToScene', args: 'help', analyticsEvent: ig.ANALYTICS_EVENT.PRESSED_HELP},
            
            _replayButton: undefined,
            replayButtonImage: null,
            replayButtonLocation: null,
            replayButtonAction: {action: 'goToScene', args: 'game', analyticsEvent: ig.ANALYTICS_EVENT.PRESSED_REPLAY},

            _shareButton: undefined,
            shareButtonImage: null,
            shareButtonLocation: null,
            shareButtonAction: {"action": 'shareOnFacebook', "args": null, analyticsEvent: ig.ANALYTICS_EVENT.PRESSED_SHARE},

            _footerButton: undefined,
            footerButtonAction: null,

            // Activity settings
            autoInitButtonActivity: true,
            mainButtonHighlight: 'bubble', 
            ctaButtonHighlight: 'bubble',
            
            messageFadeIn: false,
            messageFadeTime: 2,
            
            prefabs: undefined,
            prefabSettings: undefined,
            
            useLeadForm: false,

            init: function () {

                this.parent();

                if(this.messageText !== null) {
                    var settings = ig.merge(this.messageDefaults, this.messageFontSettings);
                    settings.text = this.messageText;
                    
                    var messageSetup = {
                        type: ig.global.EntityRealText,
                        location: this.messageLocation || {x: 'center', y: this.headerHeight + 200},
                        settings: settings
                    };
                    this._message = this.spawnEntityWithSetup(messageSetup);
                    this.namedEntities[ "messageText" ] = this._message;
                    
                    if(this.messageFadeIn) {
                        this.runActivity(this._message,
                            ig.activity.changeTo(this._message, 'alpha', this._message.alpha, this.messageFadeTime, ig.util.linearTween));
                        this._message.alpha = 0;
                    }
                }

                if(this.headerMessageText !== null) {
                    settings = ig.merge(this.headerMessageDefaults, this.headerFontSettings);
                    settings.text = this.headerMessageText;

                    var headerSetup = {
                        type: ig.global.EntityRealText,
                        location: this.headerMessageLocation || {x: 'center', y: this.headerHeight / 2},
                        settings: settings
                    };
                    this._headerMessage = this.spawnEntityWithSetup(headerSetup);
                    this.namedEntities[ "headerMessage" ] = this._headerMessage;
                }
                
                if(this.mainButtonImage) {
                    var mainButtonSetup = {
                        type: ig.global.EntityButton,
                        image: this.mainButtonImage,
                        location: this.mainButtonLocation || {x: 'center', bottom: this.footerHeight + 100},
                        settings: {
                            onPress: this.mainButtonAction
                        }
                    };
                    this._mainButton = this.spawnEntityWithSetup(mainButtonSetup);

                    this.namedEntities[ "mainButton" ] = this._mainButton;
                }

                if(this.ctaButtonImage) {
                    var ctaButtonSetup = {
                        type: ig.global.EntityButton,
                        image: this.ctaButtonImage,
                        location: this.ctaButtonLocation || {x: 'center', bottom: this.footerHeight + 100},
                        settings: {
                            onPress: this.ctaButtonAction
                        }
                    };

                    this._ctaButton = this.spawnEntityWithSetup(ctaButtonSetup);
                    this.namedEntities[ "ctaButton" ] = this._ctaButton;
                }

                if(this.helpButtonImage) {
                    var helpButtonSetup = {
                        type: ig.global.EntityButton,
                        image: this.helpButtonImage,
                        location: {right: this.cornerButtonMargin, bottom: this.footerHeight + this.cornerButtonMargin},
                        settings: {
                            onPress: this.helpButtonAction
                        }
                    };

                    this._helpButton = this.spawnEntityWithSetup(helpButtonSetup);
                    this.namedEntities[ "helpButton" ] = this._helpButton;
                }

                if(this.shareButtonImage) {
                    var shareButtonSetup = {
                        type: ig.global.EntityButton,
                        image: this.shareButtonImage,
                        location: this.shareButtonLocation || {right: this.cornerButtonMargin, bottom: this.footerHeight + this.cornerButtonMargin},
                        settings: {
                            onPress: this.shareButtonAction
                        }
                    };

                    this._shareButton = this.spawnEntityWithSetup(shareButtonSetup);
                    this.namedEntities[ "shareButton" ] = this._shareButton;
                }
                
                if(this.footerButtonAction) {
                    // Default to raise footer pressed event for backward compatibility.
                    if(this.footerButtonAction.analyticsEvent === undefined) {
                        this.footerButtonAction.analyticsEvent = ig.ANALYTICS_EVENT.PRESSED_FOOTER;
                    }

                    var footerButtonSetup = {
                        type: ig.global.EntityButton,
                        location: {right: 0, bottom: 0},
                        settings: {
                            size: {x: ig.system.width, y: this.footerHeight},
                            onPress: this.footerButtonAction
                        }
                    };

                    this._footerButton = this.spawnEntityWithSetup(footerButtonSetup);
                    this.namedEntities[ "footerButton" ] = this._footerButton;
                }
                
                if(this.replayButtonImage) {
                    var replayButtonSetup = {
                        type: ig.global.EntityButton,
                        image: this.replayButtonImage,
                        location: this.replayButtonLocation || {left: this.cornerButtonMargin, bottom: this.footerHeight + this.cornerButtonMargin},
                        settings: {
                            onPress: this.replayButtonAction
                        }
                    };

                    this._replayButton = this.spawnEntityWithSetup(replayButtonSetup);
                    this.namedEntities[ "replayButton" ] = this._replayButton;
                }

                if(this.autoInitButtonActivity) {
                    this.initButtonActivity();
                }
                
                this.spawnPrefabs();
                
            },

            endScene: function() {
                // This is hack to make enabling of lead form as easy as setting a single flag.
                // We pass along the onEndScene def so that the wizard only has to update one setting
                // for what happens after the end scene and we can just set a flag to turn the form on and off.

                // Check if we should be using a lead form.
                if (this.useLeadForm) {
                    var form = ig.config['scenes']['form'];
                    if(form) {
                        // Form is defined in config. Switch to it.
                        form.settings = form.settings || {};
                        // If onSceneEnd is not defined for form then use definition from this scene.
                        form.settings.onSceneEnd = form.settings.onSceneEnd || this.onSceneEnd;
                        return ig.actions.goToScene('form');
                    }
                    // If form is not defined we'll just end the scene as normal.
                }

                // Fallback to default
                this.parent();
            },
            
            spawnPrefabs: function() {
                // prefabs can be either a string or an array of strings.
                if(this.prefabs) {
                    if(this.prefabs.constructor === Array) {
                        // Spawn multiple prefabs.                              
                        for(var i = 0; i < this.prefabs.length; i++) {
                            var s1 = this.prefabSettings && this.prefabSettings[this.prefabs[i]];
                            this.spawnPrefab(this.prefabs[i], s1);
                        }
                    } else if(this.prefabs.constructor === String) {
                        // Spawn single prefab.
                        var s2 = this.prefabSettings && this.prefabSettings[this.prefabs];
                        this.spawnPrefab(this.prefabs, s2);
                    }
                }
            },
            
            update: function () {
                this.parent();

                if(this.displayScoreInHeader && this._headerMessage && ig.sessionStats.currentSession) {
                    var s = this.scoreDisplayFn(ig.sessionStats.currentSession.score);
                    this._headerMessage.text = s.toString();
                }
            },
            
            drawEntities: function () {
                // Draw background
                if(this.backgroundImage) {
                    ig.content.imageNamed(this.backgroundImage).draw(0, this.headerHeight);
                }

                // TODO: How do we insert draw for extra entities in here? Can we implement some sort of layers?
                
                // Draw header
                if(this.headerColor) {
                    ig.system.context.fillStyle = this.headerColor;
                    ig.system.context.fillRect(0, 0, ig.system.width, this.headerHeight);
                }

                if(this.headerImage) {
                    ig.content.imageNamed(this.headerImage).draw(0, 0);
                }

                // Draw footer
                if(this.footerColor) {
                    ig.system.context.fillStyle = this.footerColor;
                    ig.system.context.fillRect(0, ig.system.height - this.footerHeight, ig.system.width, this.footerHeight);
                }

                if(this.footerImage) {
                    ig.content.imageNamed(this.footerImage).draw(0, ig.system.height - this.footerHeight);
                }

                this.parent();
            },
            
            draw: function () {
                
                this.parent();
            },

            initButtonActivity: function() {
                if(this._mainButton) {
                    var mainHighlight = this.activityScripts[this.mainButtonHighlight];
                    if(mainHighlight) {
                        ig.game.runActivity(this._mainButton, mainHighlight(this._mainButton), "mainButtonHighlight");
                    }
                }

                if(this._ctaButton) {
                    var ctaHighlight = this.activityScripts[this.ctaButtonHighlight];
                    if(ctaHighlight) {
                        ig.game.runActivity(this._ctaButton, ctaHighlight(this._ctaButton), "ctaButtonHighlight");
                    }

                    ig.game.runActivity(this._ctaButton,
                        this.activityScripts.fadeSlide(this._ctaButton, 0, -150, 0.3, ig.util.easeInOutCubic, 0.4),
                        "ctaButtonAppear");
                }

                if(this._shareButton) {
                    ig.game.runActivity(this._shareButton,
                        this.activityScripts.fadeSlide(this._shareButton, -100, 0, 0.5, ig.util.easeInOutCubic, 0.9),
                        "shareButtonAppear");
                }

                if(this._replayButton) {
                    ig.game.runActivity(this._replayButton,
                        this.activityScripts.fadeSlide(this._replayButton, 100, 0, 0.5, ig.util.easeInOutCubic, 0.9),
                        "replayButtonAppear");
                }

                if( this.initButtons && typeof this.initButtons === "function") {
                    (this.initButtons.bind(this))();
                }


            },

            activityScripts: {

                pulse: function (entity) {
                    return ig.activity.repeatSequence([
                        ig.activity.scaleTo(entity, 1.1, 1, ig.util.easeInOutQuart),
                        ig.activity.scaleTo(entity, 1, 1, ig.util.easeInOutQuart)
                    ]);
                },

                bubble: function (entity) {
                    var a = ig.activity;
                    var easing = ig.util.easeInOutQuad;

                    var bubbleDuration = 0.6;
                    var pauseDuration = 1;
                    var scaleChange = 0.1;

                    return a.repeatSequence([
                        a.group([
                            a.scaleToX(entity, 1 + scaleChange, bubbleDuration / 6, easing),
                            a.scaleToY(entity, 1 - scaleChange, bubbleDuration / 6, easing)
                        ]),
                        a.group([
                            a.scaleToX(entity, 1 - scaleChange, bubbleDuration / 3, easing),
                            a.scaleToY(entity, 1 + scaleChange, bubbleDuration / 3, easing)
                        ]),
                        a.group([
                            a.scaleToX(entity, 1 + scaleChange / 2, bubbleDuration / 6, easing),
                            a.scaleToY(entity, 1 - scaleChange / 2, bubbleDuration / 6, easing)
                        ]),
                        a.group([
                            a.scaleToX(entity, 1 - scaleChange / 2, bubbleDuration / 6, easing),
                            a.scaleToY(entity, 1 + scaleChange / 2, bubbleDuration / 6, easing)
                        ]),
                        a.group([
                            a.scaleToX(entity, 1, bubbleDuration / 6, easing),
                            a.scaleToY(entity, 1, bubbleDuration / 6, easing)
                        ]),
                        a.pause(pauseDuration)
                    ]);
                },

                fadeSlide: function (entity, x, y, duration, easing, delay) {

                    entity.currentAnim.alpha = 0;
                    entity.pos.x -= x;
                    entity.pos.y -= y;

                    var a = ig.activity;
                    var slide = a.group([
                        a.alphaTo(entity, 1, duration),
                        a.moveBy(entity, x, y, duration, easing)
                    ]);

                    if (delay) {
                        return a.sequence([a.pause(delay), slide]);
                    } else {
                        return slide;
                    }
                }
            }

        });

    });





// lib/games/catcherBK/entities/stackingBlock.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.stackingBlock'
).requires(
    'impact.entity',
    'gamewheel.helpers.easing'
).defines(function(){ "use strict";
        
        ig.global.EntityStackingBlock = ig.Entity.extend({

            // Settings for the bouncing behaviour
            bounceMin: {x: 100, y: -200},
            bounceMax: {x: 200, y: -100},

            // Set defaults.
            gravityFactor: 1,
            maxVel: {x:1000, y: 600},
            type: ig.Entity.TYPE.A, 
            checkAgainst: ig.Entity.TYPE.A,
            collides: ig.Entity.COLLIDES.NEVER,
            
            // Doubly, linked list of stacked blocks.
            blockAbove: undefined,
            blockBelow: undefined,
            
            score: 0,   
            
            init: function( x, y, settings ) {
                this.parent( x, y, settings );
            },
            
            blocksAbove: function() {
                var count = 0;
                var block = this.blockAbove;
                // Count how many blocks are stacked on top of this one.
                while(block) {
                    count++;
                    block = block.blockAbove;
                }
                return count;
            },
            
            stack: function(other) {
                // Stack this block on top of other.
                this.blockBelow = other;
                other.blockAbove = this;
                // Don't check for collisions.
                this.checkAgainst = ig.Entity.TYPE.NONE;
            },
            
            unstack: function() {
                if (this.blockBelow) {
                    // Detach from the block below.
                    this.blockBelow.blockAbove = null;
                    this.blockBelow = null;
                    
                    // Tell the block below to unstack.
                    if (this.blockAbove) {
                        this.blockAbove.unstack();
                    }
                    
                    // Bounce away in some random direction.
                    this.bounce(Math.floor(Math.random() * 2) == 0);
                }
            },
            
            check: function(other) {
                if(other instanceof ig.global.EntityStackingBlock) {
                    // Falling block has hit another block.
                    if (!other.blockAbove &&
                        this.pos.x + (this.size.x / 2) > other.pos.x &&
                        this.pos.x + (this.size.x / 2) < other.pos.x + other.size.x) {
                        this.stack(other);
                    } else {
                        this.bounce(this.pos.x < other.pos.x);
                    }
                }
            },
            
            bounce: function(left) {
                // We're done with this block. Make sure it doesn't interact with anything else.
                this.TYPE = ig.Entity.TYPE.NONE;
                this.checkAgainst = ig.Entity.TYPE.NONE;

                // Increase the gravity so it fall of screen faster.
                this.gravityFactor = 2;
                // Set the box moving.
                this.vel.y = (Math.random() * (this.bounceMax.y - this.bounceMin.y)) + this.bounceMin.y;
                this.vel.x = (Math.random() * (this.bounceMax.x - this.bounceMin.x)) + this.bounceMin.x;
                if(left && this.vel.x > 0) {
                    this.vel.x *= -1;
                }
            },
            
            update: function() {
                // Try move where we want to move.
                this.parent();
                
                // Update position of block based on the block it's stacked on.
                if(this.blockBelow) {

                    // Find the distance horizontally between this block and the block below. 
                    var distance = this.xDistanceTo(this.blockBelow);
                    // Set the velocity based on the distance so that we follow the the block below.
                    this.vel.x = -distance * 20;
                    
                    // Make sure the center of this block is within the edges of the block it's stacked on.
                    this.pos.x = this.pos.x.limit(
                        this.blockBelow.pos.x - (this.size.x / 2),
                        this.blockBelow.pos.x + this.blockBelow.size.x - (this.size.x / 2)
                    );
                    
                    // Make sure this is stacked vertically on top of the block below.
                    this.pos.y = this.blockBelow.pos.y - this.size.y;
                }

                // Update score for this block. We really only need to update the player block. 
                // Checking we have no block below gets us most of the way there.
                if(!this.blockBelow) {
                    this.score = this.blocksAbove();    
                }
                
                // Kill the block when it falls off the bottom of the screen.
                if(this.pos.y - this.offset.y > ig.system.height) {
                    this.kill();
                }
            },

            draw: function() {
                this.parent();
            },
            
            xDistanceTo: function( other ) {
                // Return the horizontal distance between the centers of two entities.
                return (this.pos.x + this.size.x/2) - (other.pos.x + other.size.x/2);
            }
            
        });


    });


// lib/games/catcherBK/entities/stackBreaker.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.stackBreaker'
).requires(
    'impact.entity',
    'games.catcherBK.entities.stackingBlock',
    'gamewheel.helpers.easing'
).defines(function(){ "use strict";


        ig.global.EntityStackBreaker = ig.Entity.extend({

            size: {x: 100, y: 20},
            
            gravityFactor: 1,
            
            maxVel: {x:1000, y: 600},
            bounceMin: {x: 500, y: -600},
            bounceMax: {x: 900, y: -300},

            type: ig.Entity.TYPE.B,
            checkAgainst: ig.Entity.TYPE.A,
            collides: ig.Entity.COLLIDES.NEVER,

            init: function( x, y, settings ) {
                this.parent( x, y, settings );
            },

            check: function(other) {
                // We only want to interact with block entities that are currently stacked.
                if(other instanceof ig.global.EntityStackingBlock) {
                    if (other.blockBelow) {
                        // We've hit a stacked block. Tell block to unstack.
                        other.unstack();

                        // Prevent this entity from interacting with anything else.
                        this.checkAgainst = ig.Entity.TYPE.NONE;
                        this.TYPE = ig.Entity.TYPE.NONE;

                        this.vel.y = (Math.random() * (this.bounceMax.y - this.bounceMin.y)) + this.bounceMin.y;
                        this.vel.x = (Math.random() * (this.bounceMax.x - this.bounceMin.x)) + this.bounceMin.x;
                        var goLeft = this.pos.x + (this.size.x / 2) < other.pos.x + (other.size.x / 2);
                        if(goLeft && this.vel.x > 0) {
                            this.vel.x *= -1;
                        }
                    }
                }
            },
            
            draw: function() {
                this.parent();
            },
            
            update: function() {
                // Try move where we want to move.
                this.parent();

                // Kill the entity when it falls off the bottom of the screen.
                if(this.pos.y - this.offset.y > ig.system.height) {
                    this.kill();
                }
            }

        });


    });


// lib/games/catcherBK/entities/spawnerEnhanced.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.spawnerEnhanced'
).requires(
        'impact.entity',
        'gamewheel.helpers.util'
    ).defines(function(){ "use strict";

        ig.SPAWN_MODE = {
            RANDOM: 0, // Any item can be picked at any time.
            SHUFFLED: 1, // Items are randomized, but none can be repeated until all have been picked.
            ORDERED: 2, // Items are picked in order.
            WEIGHTED: 3 // Items are picked at random but weight values affect the randomness.
        };

        ig.global.EntitySpawnerEnhanced = ig.Entity.extend({

            items: [], // Keys for items defined in config that are to be spawned.
            weights: [], // Weight values for item keys. Must have the same length as items.

            size: {x: 640, y: 0}, // Pos and size of this entity are used to determine the spawn location.

            initialRest: 1, // Time to wait before the first item is spawned.
            spawnRestMin: 0.5, // Minimum time to wait between spawns.
            spawnRestMax: 2, // Maximum time to wait between spawns.

            spawnMode: ig.SPAWN_MODE.RANDOM, // Randomization to use when spawning items.

            fixedToScreen: true, // When true, position will be relative to the screen edge, not the world.

            entitySettings: null, // Extra settings to be merged into each spawned entity. 

            timerEnabled: true, // Should items be time-spawned?

            followEntityNamed: null, // If set, spawned items will be positioned close to the named entity.  

            _spawnTimer: undefined,
            _spawnIndex: 0,

            zIndex: 1,
            forceSpawnQueue: [], // Keys for items that should be spawned in order ignoring any random settings. 

            init: function( x, y, settings ) {
                this.parent( x, y, settings );
                this._spawnTimer = new ig.Timer(this.initialRest);
            },

            update: function() {
                // We don't call parent, because this entity doesn't need to interact with anything.
                if(this.timerEnabled && this._spawnTimer.delta() >= 0){
                    this.spawnNextItem();
                    this.resetTimer();
                }
            },

            resetTimer: function() {
                var time = (Math.random() * (this.spawnRestMax - this.spawnRestMin)) + this.spawnRestMin;
                this._spawnTimer.set(time);
            },

            rest: function(duration) {
                // Pause timed spawning for a limited duration.
                this._spawnTimer.set(duration);
            },

            spawnNextItem: function () {
                var itemKey = null;
                var item = null;
                if(this.forceSpawnQueue.length > 0) {
                    itemKey = this.forceSpawnQueue.shift();

                    if(itemKey) {
                        item = this.spawnItemWithKey(itemKey);
                    }
                } else {
                    if(this.spawnMode == ig.SPAWN_MODE.RANDOM) {
                        // Random mode, just pick any item at random.
                        itemKey = this.items.random();
                    } else if(this.spawnMode == ig.SPAWN_MODE.WEIGHTED) {
                        var weight = 0;
                        // Sum weights.
                        for(var i = 0; i < this.weights.length; i++) {
                            weight += this.weights[i];
                        }
                        // Pick a random number within the weight range.
                        var rnd = Math.random() * weight;
                        // Reset the weight total.
                        weight = 0;
                        // Find the corresponding item.
                        for(i = 0; i < this.weights.length; i++) {
                            // Start adding up the weights again.
                            weight += this.weights[i];
                            // Check if we've reached the item we're looking for.
                            if(rnd < weight) {
                                itemKey = this.items[i];
                                break;
                            }
                        }
                    } else {
                        if(this._spawnIndex == 0 && this.spawnMode == ig.SPAWN_MODE.SHUFFLED) {
                            // Shuffle mode, and this is a new cycle. Shuffle the items.
                            ig.util.shuffleListInPlace(this.items);
                        }
                        // Pick the current item in the cycle.
                        itemKey = this.items[this._spawnIndex];
                        // Increment our spot in the cycle.
                        this._spawnIndex = (this._spawnIndex + 1) % this.items.length;
                    }

                    // If no valid key is found, nothing will be spawned this time.
                    if(itemKey) {
                        item = this.spawnItemWithKey(itemKey);
                    }
                }

                if(item) {
                    item.zIndex = this.zIndex;
                    ig.game.sortEntitiesDeferred();
                    return item;
                }
            },

            spawnItemWithKey: function( key ) {
                // Spawn the entity
                // var item = ig.game.spawnConfigEntity( key, this.entitySettings );
                var item = ig.game.spawnEntityWithSetup(  ig.config.entities[ key ].random()  );

                // Position the entity within the bounds of the spawner
                item.pos = this.getSpawnPosition( item.size );

                // Adjust for screen position.
                if(this.fixedToScreen) {
                    item.pos.x += ig.game.screen.x;
                    item.pos.y += ig.game.screen.y;
                }

                // Return the item.
                return item;
            },

            getSpawnPosition: function(itemSize){

                var location = null;

                // Check if we need to try spawn close to a named entity. 
                if(this.followEntityNamed) {
                    // Find the target entity
                    var entity = ig.game.getEntityByName(this.followEntityNamed);
                    // Make sure it exists.
                    if(entity) {
                        // Pick a point within the bounds of the entity.
                        var x = (Math.random() * entity.size.x) + entity.pos.x - (itemSize.x / 2);
                        var y = (Math.random() * entity.size.y) + entity.pos.y - (itemSize.y / 2);

                        // Set the location making sure we don't go outside the spawn area.
                        location = {
                            x: x.limit(this.pos.x, this.pos.x + this.size.x - itemSize.x),
                            y: y.limit(this.pos.y, this.pos.y + this.size.y - itemSize.y)
                        };
                    }
                }

                // If we have a location, return it. If not pick a random location. 
                return location || {
                        x: (Math.random() * Math.max(this.size.x - itemSize.x, 0)) + this.pos.x,
                        y: (Math.random() * this.size.y - itemSize.y) + this.pos.y
                    };
            },

            // We don't draw anything.
            draw: function() {}

        });


    });


// lib/games/catcherBK/entities/collectable.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.collectable'
).requires(
    'impact.entity',
    'gamewheel.helpers.easing'
).defines(function(){ "use strict";


        ig.global.EntityCollectable = ig.Entity.extend({
            
            // How many points this object is worth when collected.
            points: 1, 

            // Set with the key for a config entity to spawn that entity when this object is collected.
            spawnOnCollect: null,   

            // Object defaults to only triggering collect if it hits the top of another object.
            // Set this to true to allow collect from any side.
            unrestrictedCollect: true,

            // Set defaults
            gravityFactor: 1,
            maxVel: {x:1000, y: 700},
            type: ig.Entity.TYPE.B,
            checkAgainst: ig.Entity.TYPE.A,
            collides: ig.Entity.COLLIDES.NEVER,

            isCollectableType : true,
            
            
            init: function( x, y, settings ) {
                this.parent( x, y, settings );
            },

            check: function(other) {
                if(this.unrestrictedCollect ||
                    (this.last.y + this.size.y < other.last.y &&
                    this.pos.x + (this.size.x / 2) > other.pos.x &&
                    this.pos.x + (this.size.x / 2) < other.pos.x + other.size.x)) {
                    // Collect this item.
                    this.collect(other);

                    //if( this.isObstacle )
                    {
                        // SHOW KILL ANIM.
                        this.unrestrictedCollect = false;
                        this.gravityFactor = 0;
                        this.currentAnim = this.anims["kill"];
                        if( this.currentAnim ) {
                            this.currentAnim.rewind();
                        }

                        ig.game.runActivity(this,
                            ig.activity.sequence([
                                ig.activity.pause(1),
                                ig.activity.killEntity(this)
                            ])
                            , "delayed kill");
                    }
                    /*else {
                        this.kill();
                    }*/
                } else {
                    // Hit the side, bounce away from the object.
                    this.checkAgainst = ig.Entity.TYPE.NONE;
                   // this.vel.x = 10 * ((this.pos.x + (this.size.x / 2)) - (other.pos.x + (other.size.x / 2)));
                }
            },

            collect: function(collectedBy) {
                if(this.spawnOnCollect) {
                    var o = ig.game.spawnConfigEntity(this.spawnOnCollect);
                    // Center object on this one.
                    o.pos.x = this.pos.x + (this.size.x / 2) - (o.size.x / 2);
                    o.pos.y = this.pos.y + (this.size.y / 2) - (o.size.y / 2);
                }
                if(typeof collectedBy.receivePoints === "function") {
                    if(!collectedBy.isDead) {
                        collectedBy.receivePoints(this.points,  this);
                    }
                }

                this.collected( this );
            },

            draw: function() {
                this.parent();
            },

            update: function() {
                // Try move where we want to move.
                this.parent();

                // Kill the entity when it falls off the bottom of the screen.
                if(this.pos.y - this.offset.y > ig.game.screen.y + ig.system.height + 500) {
                    this.collected( this );
                    this.kill();
                }
            },

            collected : function( entity ) {

            },

            kill : function() {
                this.parent();
                if( this.arrow ) {
                    this.arrow.kill();
                }
            }

        });


    });


// lib/games/catcherBK/entities/pointMarker.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.pointMarker'
).requires(
    'gamewheel.entities.realText',
    'gamewheel.helpers.easing'
).defines(function(){

        ig.global.EntityPointMarker = ig.global.EntityRealText.extend({

            gravityFactor: 0,
            duration: 1.5, // Time that marker takes to move and fade out.
            movement: {x: 0, y: -50}, // Amount marker should move.

            negativeColor: undefined,
            showPlusSign: true,

            pointValue: undefined,

            alpha: 1,

            zIndex:30,

            init: function( x, y, settings ) {
                this.parent( x, y, settings );

                ig.game.sortEntitiesDeferred();
                
                if(this.pointValue !== undefined) {
                    this.displayPoints(this.pointValue);
                }

                ig.game.runActivity(this, ig.activity.changeTo(this, 'alpha', 0, this.duration));
                ig.game.runActivity(
                    this,
                    ig.activity.moveBy(this, this.movement.x, this.movement.y, this.duration, ig.util.easeOutCubic)
                );
                ig.game.runActivity(this, ig.activity.delay(this.duration, ig.activity.killEntity( this )));
            },

            displayPoints: function(value) {
                var sign = (value > 0 && this.showPlusSign) ? '+' : '';
                this.text = sign + value.toString();
                if(value < 0 && this.negativeColor) {
                    this.color = this.negativeColor;
                }
            }
        });

    });


// lib/games/catcherBK/entities/catcher.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.catcher'
).requires(
    'impact.entity',
    'gamewheel.helpers.easing',
    'games.catcherBK.entities.pointMarker',
    'gamewheel.core.activity'
).defines(function(){ "use strict";
        
        ig.global.EntityCatcher = ig.Entity.extend({

            gravityFactor: 1,
            maxVel: {x:1000, y: 600},
            
            type: ig.Entity.TYPE.A, 
            checkAgainst: ig.Entity.TYPE.NONE,
            collides: ig.Entity.COLLIDES.NEVER,
            
            score: 0,
            minScore: 0, // Score cannot go below this value. Set to null to have no minimum.
            
            maxTiltAngle: 0.2,
            maxTiltDistance: 100,  
            
            pointMarker: undefined, // Named config entity to spawn when earning points.

            collectedObstacles : 0,

            isDead : false,


            init: function (x, y, settings) {
                this.parent(x, y, settings);
            },

            update: function() {
                
                // Set angle based on how far the 
                var moved = (this.pos.x - this.last.x);
                this.currentAnim.angle = (moved / this.maxTiltDistance).limit(-this.maxTiltAngle, this.maxTiltAngle);
                
                // Try move where we want to move.
                this.parent();

            },

            receivePoints:function(points, from){

                // Show collection effect based on points.
                if(points > 0) {
                    ig.game.runActivity(this,
                        ig.activity.sequence([
                            ig.activity.scaleBy( this, 0.3, 0.1, ig.util.easeOutQuad ),
                            ig.activity.scaleTo( this, 1, 0.12, ig.util.easeInQuad )
                        ])
                    , "collect");
                } else {
                    ig.game.runActivity(this,
                        ig.activity.sequence([
                            ig.activity.scaleBy( this, -0.5, 0.1, ig.util.easeInQuad ),
                            ig.activity.scaleTo( this, 1, 0.1, ig.util.easeOutQuad )
                        ])
                    , "hitEffect");
                    this.collectedObstacles++;
                    ig.game.lostLife();
                }

                // Update the score.
                if(this.minScore !== undefined && this.minScore !== null) {
                    this.score = Math.max(this.minScore, this.score + points);
                }

                // Display point marker
                if(from && this.pointMarker) {
                    var x = from.pos.x + (from.size.x / 2);
                    var y = from.pos.y + (from.size.y / 2);
                    var marker = ig.game.spawnConfigEntity(this.pointMarker, {
                        pos: {x: x, y: y},
                        pointValue: points
                    });
                }

            }
            
        });

    });


// lib/games/catcherBK/entities/selfDestroying.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.selfDestroying'
).requires(
    'impact.entity'
).defines(function(){ "use strict";
        
        ig.global.EntitySelfDestroying = ig.Entity.extend({
            
            lifetime: undefined,
            
            update: function() {
                this.parent();
                
                if(this.lifetime) {
                    this.lifetime -= ig.system.tick;
                    if(this.lifetime <= 0) {
                        this.kill();
                    }
                }
            }

        });
    });/**
 * Optionally displays orientation, splash and help screens.
 */



// lib/gamewheel/systems/trackers.js
ig.baked=true;ig.module(
    'gamewheel.systems.trackers'
).defines(function(){ "use strict";



        ig.ITracker = ig.Class.extend({

            complete: function () {
                // Not supported.
                return null;
            },
            progress: function () {
                // Not supported.
                return null;
            },
            remaining: function () {
                // Not supported.
                return null;
            },
            percentage: function () {
                // Not supported.
                return null;
            }

        });

        ig.ValueTracker = ig.ITracker.extend({

            _value: 0,
            _target: null,

            init: function (target) {
                this._target = target || null;
                Object.defineProperty(this, "target",{
                    get: this._getTarget,
                    set: this._setTarget
                });
                Object.defineProperty(this, "value",{
                    get: this._getValue,
                    set: this._setValue
                });
            },

            inc: function () {
                this._value++;
            },

            dec: function () {
                this._value--;
            },

            _setValue: function (value) {
                this._value = value;
            },

            _getValue: function () {
                return this._value;
            },

            _setTarget: function (value) {
                this._target = value || null;
            },

            _getTarget: function () {
                return this._target;
            },

            complete: function () {
                return this._target ? this._value >= this._target : null;
            },

            progress: function () {
                return this._value;
            },

            remaining: function () {
                return this._target ? Math.max(0, this._target - this._value) : null;
            },

            percentage: function () {
                return this._target ? (this._value / this._target).limit(0, 1) : null;
            }
        });


        ig.TimeTracker = ig.ITracker.extend({

            _timer: null,

            init: function (duration, autoStart) {
                this._timer = new ig.Timer(duration);
                if(!autoStart) {
                    this._timer.pause();
                }
            },

            start: function () {
                this._timer.unpause();
            },

            pause: function () {
                this._timer.pause();
            },

            paused: function () {
                return !!this._timer.pausedAt;
            },

            unpause: function () {
                this._timer.unpause();
            },

            increaseDuration: function (seconds) {
                if(this._timer.target) {
                    this._timer.target += seconds;
                }
            },

            duration: function () {
                return this._timer.target || null;
            },

            complete: function () {
                return this._timer.target ? this._timer.delta() >= 0 : null;
            },

            progress: function () {
                var e = this._timer.target + this._timer.delta();
                return this._timer.target ? Math.min(this._timer.target, e) : e;
            },

            remaining: function () {
                return this._timer.target ? this._timer.target - this.progress() : null;
            },

            percentage: function () {
                return this._timer.target ? this.progress() / this._timer.target : null;
            }
        });


    });



// lib/gamewheel/scenes/gameScene.js
ig.baked=true;ig.module(
    'gamewheel.scenes.gameScene'
).requires(
        'gamewheel.scenes.baseTemplateScene',
        'gamewheel.systems.trackers'
    ).defines(function(){ "use strict";

        ig.GameStates = {
            Tutorial: 1,
            Intro: 0,
            Playing: 2,
            Ending: 3,
            Over: 4,
            User: 5,
            Paused: 6
        };


        ig.GameScene = ig.BaseTemplateScene.extend({

            gameState: null,

            // Set clear color to default to transparent.
            clearColor: null,

            // Default to exit on scene end.
            onSceneEnd: {
                action: 'exitGame'
            },

            _trackTime: true,
            _trackScore: true,
            _trackDistance: false,

            gameDuration: null,
            targetScore : null,
            targetDistance: null,

            _timers: {},

            session: {
                score    : null,
                time     : null,
                distance : null,
                died     : null
            },


            init: function () {
                this.parent();

                this.initSession();

            },


            initSession: function () {
                // Reset session object.
                this.session = {
                    score: this._trackScore ? new ig.ValueTracker(this.targetScore) : null,
                    time: this._trackTime ? new ig.TimeTracker(this.gameDuration) : null,
                    distance: this._trackDistance ? new ig.ValueTracker(this.targetDistance) : null,
                    died: null
                };
            },

            update: function () {
                this.parent();

                // Notify of session updates.
                // TODO: I think we'll need to do this outside of this function. Otherwise we'll likely be a frame behind.
                ig.output.sessionUpdate(this.session);


                if(this.gameState === ig.GameStates.Playing) {
                    if(this.gameEndPredicate()) {
                        this.changeState(ig.GameStates.Ending);
                    }
                }
            },

            // Convenient way to grab a timer when needed.
            // Creates/returns a timer object associated with the given name.
            getTimer: function (name) {
                if(!this._timers[name]) {
                    this._timers[name] = new ig.Timer();
                }
                return this._timers[name];
            },


            gameEndPredicate: function() {
                return (this.session.time && this.session.time.complete()) ||
                    (this.session.score && this.session.score.complete()) ||
                    (this.session.distance && this.session.distance.complete());
            },

            changeState: function (state) {
                if(this.gameState != state) {
                    if(this.gameState !== null) {
                        this.onExitState(this.gameState);
                        this.onExitStateDefault(this.gameState);
                    }
                    this.gameState = state;
                    this.onEnterState(state);
                    this.onEnterStateDefault(state);
                }
            },

            onEnterStateDefault: function (state) {
                if(state === ig.GameStates.Playing){
                    if(this.session.time) {
                        this.session.time.start();
                    }
                    ig.output.gameEvent('GameStarted');
                }
            },

            onExitStateDefault: function (state) {
                if (state === ig.GameStates.Playing) {
                    if(this.session.time) {
                        this.session.time.pause();
                    }
                    ig.output.gameEvent('GameEnded');
                }
            },

            onExitState: function (state) {},
            onEnterState: function (state) {}

        });


    });




// lib/games/catcherBK/entities/scrollingBackgroundSelfDestroying.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.scrollingBackgroundSelfDestroying'
).requires(
    'impact.entity'
).defines(function(){ "use strict";
        
        ig.global.EntityScrollingBackgroundSelfDestroying = ig.Entity.extend({
            
            terminationPositionY: 960,
            gravityFactor : 1,
            // moveSpeed : 1,

            isBackground : true,

            callback : function() {},

            init: function (x, y, settings) {
                this.parent(x, y, settings);
            },
            
            update: function() {
                this.parent();

                if(this.pos.y >= this.terminationPositionY) {
                    this.callback( this );
                }
            },

            setScroll : function( posY, gravity ) {
                this.pos.y = posY;
                this.maxVel.y = gravity;
                this.vel.y = gravity;
            }

        });
    });


// lib/games/catcherBK/entities/progressTimer.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.progressTimer'
).requires(
        'impact.entity',
        'gamewheel.entities.realText'
    ).defines(function(){ "use strict";

        ig.global.EntityProgressTimer = ig.global.EntityRealText.extend({

            timeLeft : 0,

            init: function (x, y, settings) {
                this.parent(x, y, settings);
            },

            update: function() {
                this.parent();
                this._setText( "" + this.timeLeft );
            },

            updateTime : function( time ) {
                this.timeLeft = this.format( time );
            },

            format : function( string ) {
                var sec_num = parseInt(string, 10); // don't forget the second param
                var hours   = Math.floor(sec_num / 3600);
                var minutes = Math.floor((sec_num - (hours * 3600)) / 60);
                var seconds = sec_num - (hours * 3600) - (minutes * 60);

                if (minutes < 10) {minutes = "0"+minutes;}
                if (seconds < 10) {seconds = "0"+seconds;}
                return minutes+':'+seconds;
            }

        });
    });


// lib/games/catcherBK/entities/obstacleSpawner.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.obstacleSpawner'
).requires(
    'impact.entity'
).defines(function(){ "use strict";
        
        ig.global.EntityObstacleSpawner = ig.Entity.extend({

            spawnRange : {
                startX : 0,
                endX : 640
            },

            yStep : 160 / 2,
            minDistance : 200,

            lastZoneUsed : -1,
            zones : [],

            lastSpawnedEntity : null,
            lastSpawnedAmount : -1,
            lastSpawnedRow : [],
            yMin : 320,
            yMax : 420,

            init: function (x, y, settings) {
                this.parent(x, y, settings);

                var xStep = (this.spawnRange.endX - this.spawnRange.startX) / 4;

                var start = this.spawnRange.startX;
                var end = start + xStep;
                this.zones.push({
                    start : start,
                    end : end,
                    center : (start + end) / 2
                });

                start = end;
                end = start + xStep;
                this.zones.push({
                    start : start,
                    end : end,
                    center : (start + end) / 2
                });

                start = end;
                end = start + xStep;
                this.zones.push({
                    start : start,
                    end : end,
                    center : (start + end) / 2
                });

                start = end;
                end = start + xStep;
                this.zones.push({
                    start : start,
                    end : end,
                    center : (start + end) / 2
                });
            },

            update: function() {
                this.parent();
            },

            spawn : function( entityDescription, yOffset ) {

                console.log( "Last zone", this.lastZoneUsed );

                var zone = this.lastZoneUsed;
                while(zone == this.lastZoneUsed) {
                    var zoneObject = this.zones.random();
                    zone = this.zones.indexOf( zoneObject );
                }

                var zoneObject = this.zones[ zone ];
                this.lastZoneUsed = zone;

                if( this.lastSpawnedEntity ) {
                    var dist = this.lastSpawnedEntity.pos.y;
                    yOffset = yOffset || dist < this.minDistance ? -this.minDistance : 0;
                }

                this.lastSpawnedEntity = ig.game.spawnEntityWithSetup(entityDescription);
                this.lastSpawnedEntity.pos.y += yOffset || 0;
                this.lastSpawnedEntity.pos.x = zoneObject.center - this.lastSpawnedEntity.currentAnim.sheet.width / 2;

                var grav = ig.game.gravity;
                this.lastSpawnedEntity.vel.y = grav;
                this.lastSpawnedEntity.maxVel.y = grav;

                console.log( "Spawned", this.lastSpawnedEntity );
            },

            noDuplicates : function( b ) {
                var a = b.concat();

                for( var i = 0; i < a.length; ++i ) {
                    for( var j = i + 1; j < a.length; ++j ) {
                        if( a[ i ] === a[ j ] ) {
                            a.splice( j--, 1 );
                        }
                    }
                }

                return a;
            },

            spawnRow : function(mainEntityDescription, fillerEntityDescription, isEndRow, showArrow) {

                this.lastSpawnedRow = [];

                // this function should spawn 2-3 items in a row ! and track the latestEntity
                var min = 1;
                var max = 3;
                var amount = Math.floor(Math.random() * (max - min + 1)) + min;

                var yOffset = Math.floor(Math.random() * (this.yMax - this.yMin + 1)) + this.yMin;
                if(this.lastSpawnedEntity && this.lastSpawnedEntity.pos.y <= 0) {
                    yOffset = this.lastSpawnedEntity.pos.y - yOffset;
                } else {
                    yOffset = -yOffset;
                }

                var usedZones = [];
                for( var i = 0; i < amount; ++i ) {

                    var zoneIndex = Math.floor( Math.random() * this.zones.length );
                    while( usedZones.indexOf( zoneIndex ) > -1 ) {
                        zoneIndex = Math.floor( Math.random() * this.zones.length );
                    }


                    var zoneObject = this.zones[ zoneIndex ];
                    usedZones.push(zoneIndex);

                    this.lastSpawnedEntity = ig.game.spawnEntityWithSetup(mainEntityDescription.random());
                    this.lastSpawnedEntity.pos.y = yOffset;
                    this.lastSpawnedEntity.pos.x = zoneObject.center - this.lastSpawnedEntity.currentAnim.sheet.width / 2;
                    var grav = ig.game.gravity;
                    this.lastSpawnedEntity.vel.y = grav;
                    this.lastSpawnedEntity.maxVel.y = grav;
                    this.lastSpawnedRow.push( this.lastSpawnedEntity );
                }


                this.lastSpawnedAmount = amount;

                var unusedZones = [];
                for( var i = 0; i < 4; i++ ) {
                    if( usedZones.indexOf( i ) <= -1 ) {
                        unusedZones.push( i );
                    }
                }

                // Spawn fillers!!!
                // var fillersAmount = unusedZones.length;
                var fillersAmount = Math.floor(Math.random() * unusedZones.length);
                if(fillersAmount == 0 && unusedZones.length > 1) {
                    fillersAmount = 1;
                }

                if(isEndRow) {
                    fillersAmount = unusedZones.length;
                }

                for( var i = 0; i < fillersAmount; ++i ) {
                    var zoneObject = this.zones[ unusedZones[ i ] ];

                    this.lastSpawnedEntity = ig.game.spawnEntityWithSetup(fillerEntityDescription.random());
                    this.lastSpawnedEntity.pos.y = yOffset;
                    this.lastSpawnedEntity.pos.x = zoneObject.center - this.lastSpawnedEntity.currentAnim.sheet.width / 2;
                    var grav = ig.game.gravity;
                    this.lastSpawnedEntity.vel.y = grav;
                    this.lastSpawnedEntity.maxVel.y = grav;

                    this.lastSpawnedRow.push( this.lastSpawnedEntity );

                    if(showArrow) {
                        var lastE = this.lastSpawnedEntity;

                        // spawn shitty arrow
                        var arrow = ig.game.spawnConfigEntity("arrowEntity", {
                            pos : {
                                x : lastE.pos.x,
                                y : lastE.pos.y + 128
                            }
                        });

                        this.lastSpawnedEntity.arrow = arrow;
                    }
                }

                this.lastSpawnedEntity.collected = function() {
                    ig.game._spawner.spawnRow(ig.config.entities.obstacles, ig.config.entities.items, false);
                };
            }

        });
    });

// lib/games/catcherBK/entities/triggerArea.js
ig.baked=true;ig.module(
    'games.catcherBK.entities.triggerArea'
).requires(
        'impact.entity'
    ).defines(function(){ "use strict";
        ig.global.EntityTriggerArea = ig.Entity.extend({
            size         : { x : 640, y : 256 },
            collides     : ig.Entity.COLLIDES.NEVER,
            type         : ig.Entity.TYPE.B,
            checkAgainst : ig.Entity.TYPE.A,

            entitiesInside : [],

            onEnter  : function( trigger, entity ) {},
            onLeave  : function( trigger, entity ) {},
            onInside : function( trigger, entity ) {},

            init: function (x, y, settings) {
                this.parent(x, y, settings);
            },

            update: function () {
                this.parent();

                for( var i = this.entitiesInside.length - 1; i >= 0; i-- ) {
                    var entity = this.entitiesInside[ i ];
                    var collide = this.touches( entity );
                    if( !collide ) {
                        this.entitiesInside.splice( this.entitiesInside.indexOf( entity ), 1 );
                        this.onLeave( this, entity );
                    }
                }
            },

            check: function( other ) {

                if( this.entitiesInside.indexOf( other ) <= -1 ) {
                    this.entitiesInside.push( other );
                    this.onEnter( this, other );
                } else {
                    this.onInside( this, other );
                }

            }

        })
    });



// lib/games/catcherBK/scenes/gameScene.js
ig.baked=true;ig.module(
    'games.catcherBK.scenes.gameScene'
).requires(
        'gamewheel.core.content',
        'games.catcherBK.entities.stackingBlock',
        'games.catcherBK.entities.stackBreaker',
        'games.catcherBK.entities.spawnerEnhanced',
        'games.catcherBK.entities.collectable',
        'games.catcherBK.entities.catcher',
        'games.catcherBK.entities.selfDestroying',
        'gamewheel.scenes.gameScene',
        'gamewheel.entities.progressBar',
        'games.catcherBK.entities.pointMarker',
        'games.catcherBK.entities.scrollingBackgroundSelfDestroying',
        'games.catcherBK.entities.progressTimer',
        'games.catcherBK.entities.obstacleSpawner',
        'games.catcherBK.entities.triggerArea'
        // ,'impact.debug.debug'
).defines(function() { "use strict";


        ig.StackerGameScene = ig.GameScene.extend({

            gravity: 100,

            playerLifes : 3,

            // Set time tracking.
            _trackTime: true,
            gameDuration : null,
            // Set score tracking.
            _trackScore: true,
            targetScore  : null,
            // Disable distance tracking.
            _trackDistance: false,

            collectableEntityKey: null,
            obstacleEntityKey: null,

            obstacleRatio: 0.5,

            spawnRestMin: 0.3,
            spawnRestMax: 0.7,
            spawnAbovePlayer: true,

            initialSpawnCollectables: 2,

            borderWidth: null,

            gameOverDelay: 2,

            _player: undefined,
            _spawner: undefined,

            _timer : undefined,

            frameTimer : new ig.Timer(),
            gravityChangeTimer : new ig.Timer( 15 ),
            _useAccel : false,
            accelMoveSpeedX : 650,
            accelMoveSpeedY : 500,
            lastAccel : {

                data : {
                    x : 0,
                    y : 0,
                    z : 0
                },

                delta : {
                    x : 0,
                    y : 0,
                    z : 0
                }
            },

            lifes : [],

            switchTime: 0,
            switched : false,
            endSwitch : false,
            grillOnTimer : 30,
            rowsAmount : 30,

            bgs : [],

            accelOn : false,
            playerDead : false,
            isDragging : false,

            init: function () {

                this.parent();

                this._useAccel = ig.config.global.accelerometerControls && ig.ua.mobile || false;
                // Bind accel.
                if( this._useAccel ) {
                    ig.input.initAccelerometer();
                    this.accelMoveSpeedX = 250;
                    this.accelMoveSpeedY = 250;
                }

                this._spawner = ig.game.spawnEntityWithSetup({
                    type: ig.global.EntityObstacleSpawner,
                    settings: {}
                });

                // Setup the base block that the user controls.
                this._player = this.spawnConfigEntity('player', {
                    gravityFactor: 0,
                    checkAgainst: ig.Entity.TYPE.NONE
                });

                this._player.size.x = 128;
                this._player.size.y = 198.0 / 6.0;

                // spawn life images
                var margin = 20;
                for( var i = 0; i < this.playerLifes; i++ ) {
                    var e = ig.game.spawnConfigEntity("life");
                    e.pos.x += i * e.animSheet.width + margin;
                    this.lifes.push( e );
                }


                // this.progressBar = ig.game.spawnConfigEntity("gameProgress");
                this.progressTimer = ig.game.spawnConfigEntity("gameProgressTimer");
                this.pointDisplay = ig.game.spawnConfigEntity("pointDisplay");
                this._timer = new ig.Timer();

                // Start the game.
                this.changeState(ig.GameStates.Playing);

                this.runDebugGame();

                var amount = 8;
                var imageHeight = 240;
                var start = (amount/2) * imageHeight;
                for( var i = 0; i < amount; ++i ) {
                    var y = -start + ( i * imageHeight );
                    var bg = ig.game.spawnEntityWithSetup({
                        type: ig.global.EntityScrollingBackgroundSelfDestroying,
                        image : "grill_off",
                        settings: {
                            pos : {
                                x : 0,
                                y : y
                            },
                            terminationPositionY : 960,
                            callback : function( entity ) {
                                // entity.setScroll( entity.pos.y - start - imageHeight, ig.game.gravity );
                                entity.setScroll( entity.pos.y - start * 2, ig.game.gravity );
                                if( ig.game.endSwitch && !entity.endSwitch ) {
                                    entity.endSwitch = true;
                                    entity.animSheet.image = ig.content.imageNamed("grill_end");
                                }
                            },
                            zIndex : -1
                        }
                    });

                    bg.endSwitch = false;
                    this.bgs.push( bg );
                }



                var grillOnTrigger = ig.game.spawnEntity( "EntityTriggerArea", 0, 0, {
                    size : { x : 640, y : 240 },
                    triggered : false,
                    checkAgainst : ig.Entity.TYPE.B,
                    type : ig.Entity.TYPE.B,
                    gravityFactor : 0,

                    onEnter : function( t, e ) {
                        if( !grillOnTrigger.triggered ) {
                            grillOnTrigger.triggered = true;
                            setTimeout(function(){
                                ig.game.switchTime = ig.game.grillOnTimer;
                            }, 750);
                            grillOnTrigger.kill();
                        }
                    }

                });

                ig.game.sortEntitiesDeferred();

                ig.analytics.submitEvent(ig.ANALYTICS_EVENT.GAME_STARTED);
            },

            runDebugGame : function() {
                for(var i = 0; i < this.rowsAmount; ++i) {
                    //this._spawner.spawnRow(ig.config.entities.obstacles.random(), ig.config.entities.items.random());
                    this._spawner.spawnRow(ig.config.entities.obstacles, ig.config.entities.items, i == this.rowsAmount - 1 || i < 3, i < 3);
                }

                var spawnFinishLine = false;
                if( spawnFinishLine ) {
                    // finishLine
                    var lastY = this._spawner.lastSpawnedEntity.pos.y;
                    for( var i = 0, l = this._spawner.lastSpawnedRow.length; i < l; ++i ) {
                        var entity = this._spawner.lastSpawnedRow[ i ];
                        entity.collected = function( e ) {
                            if( !ig.game.endSwitch ) {
                                var lowestY = 1000000000;
                                var lowestEntity = null;

                                for( var k in ig.game.bgs ) {
                                    var bg = ig.game.bgs[ k ];
                                    if( bg.pos.y < lowestY ) {
                                        lowestY = bg.pos.y;
                                        lowestEntity = bg;
                                    }
                                }
                                lowestEntity.animSheet.image = ig.content.imageNamed("grill_finish");
                                lowestEntity.pos.y = lowestY;

                                // spawn end trigger box here!
                                // trigger for navigating to the form, after we hit the finish line.
                                var endTrigger = ig.game.spawnEntity( "EntityTriggerArea", 0, lowestY, {

                                    size : { x : 640, y : 240 },
                                    triggered : false,

                                    onEnter : function( t, e ) {

                                    },

                                    onInside : function( t, e ) {

                                    },

                                    onLeave : function( t, e ) {
                                        if(!this.triggered) {
                                            setTimeout(function(){
                                                console.log( ig.sessionStats.currentSession );
                                                ig.game.changeState(ig.GameStates.Over);
                                            }, 2000);
                                            this.triggered = true;
                                        }
                                    }

                                });

                                endTrigger.gravityFactor = lowestEntity.gravityFactor;
                                endTrigger.vel.y = lowestEntity.vel.y;
                                endTrigger.maxVel.y = lowestEntity.vel.y;
                                endTrigger.isBackground = true;

                                // end
                                ig.game.endSwitch = true;
                                ig.game._spawner.lastSpawnedRow = [];
                            }
                        };
                    }

                }

                ig.game.sortEntitiesDeferred();
            },

            moveEntities : function() {

            },

            changeGravity : function(grav) {
                this.gravity = grav;
                for( var e in this.entities ) {
                    var entity = this.entities[ e ];
                    if(entity.isCollectableType || entity.isBackground) {
                        entity.vel.y = grav;
                        entity.maxVel.y = grav;
                    }
                }
            },

            lostLife : function () {
                this.playerLifes--;

                var e = this.lifes.pop();

                if( e ) {
                    e.kill();
                }
            },

            bindInput: function () {

                // Bind key controls.
                if(!this._useAccel){
                    ig.input.bind(ig.KEY.LEFT_ARROW, 'left');
                    ig.input.bind(ig.KEY.RIGHT_ARROW, 'right');
                    ig.input.bind(ig.KEY.UP_ARROW, 'up');
                    ig.input.bind(ig.KEY.DOWN_ARROW, 'down');
                    ig.input.bind(ig.KEY.MOUSE1, 'click');
                }

            },

            update: function () {
                // Every 30 seconds increment gravity by 50

                if( this.gravityChangeTimer.delta() >= 0 ) {
                    this.gravityChangeTimer.reset();
                    this.changeGravity( this.gravity + 50 );
                    this.switchTime += 15;
                }

                if( this.switchTime >= this.grillOnTimer && !this.switched ) {
                    this.switched = true;
                    for( var i = 0; i < this.bgs.length; i++ ) {
                        var bg = this.bgs[ i ];
                        if( !bg.endSwitch ) {

                            ig.game.namedEntities["staticBackground"].currentAnim = ig.game.namedEntities["staticBackground"].anims["on"];
                            if( ig.game.namedEntities["staticBackground"].currentAnim ) {
                                ig.game.namedEntities["staticBackground"].currentAnim.rewind();
                            }
                        }
                    }
                }

                if(this.playerLifes <= 0 && !this.playerDead)
                {
                    this._player.isDead = true;
                    this.playerDead = true;
                    this.playerDied();
                }


                // Update all entities
                this.parent();
                var dt = this.frameTimer.delta();

                if( this._useAccel ) {

                    this.lastAccel.delta = {
                       x : ig.input.accel.x - this.lastAccel.data.x,
                       y : ig.input.accel.y - this.lastAccel.data.y,
                       z : ig.input.accel.z - this.lastAccel.data.z
                    };

                    this.lastAccel.data = ig.input.accel;
                }

                if(this.playerDead) {
                    this.accelOn = false;
                }

                if( !this.accelOn ) {
                    this.lastAccel.delta = {
                        x : 0,
                        y : 0,
                        z : 0
                    };

                    this.lastAccel.data = {
                        x : 0,
                        y : 0,
                        z : 0
                    };

                    this.accelOn = true;
                }

                // Update the score with the number of stacked items.
                this.session.score.value = this._player.score;
                ig.sessionStats.currentSession.lifes = this.playerLifes;
                ig.sessionStats.currentSession.score = this._player.score;
                this.pointDisplay._setText( this.session.score.value + "" );

                // Update player movement. Start by resetting player velocity.
                this._player.vel.x = 0;

                // Move the stack base based on input from the user.
                if(!this._useAccel) {

                    if(!ig.game.playerDead) {
                        if(ig.input.state('click')) {

                            if(ig.game.locationInEntity(ig.input.mouse.x, ig.input.mouse.y, this._player)) {
                                ig.game.isDragging = true;
                            }

                            if(ig.game.isDragging) {
                                this._player.pos.x = ig.input.mouse.x - (this._player.size.x / 2);
                                this._player.pos.y = ig.input.mouse.y - (this._player.size.y / 2);
                                if(this._player.pos.y >= ig.system.height - 160) {
                                    this._player.pos.y = ig.system.height - 160;
                                }

                                this._player.pos.x = Math.min(ig.system.width - (this._player.size.x), this._player.pos.x);
                                this._player.pos.x = Math.max(0, this._player.pos.x);
                            }

                        } else if(ig.input.state('right') && this._player.pos.x < ig.system.width - (this._player.size.x / 2)){
                            // Keyboard input. Move player so long as right is held.
                            this._player.pos.x += dt * this.accelMoveSpeedX;
                            this._player.pos.x = Math.min(ig.system.width - (this._player.size.x), this._player.pos.x);
                        } else if(ig.input.state('left')  && this._player.pos.x > 0 - (this._player.size.x / 2)){
                            // Keyboard input. Move player so long as left is held.
                            this._player.pos.x -= dt * this.accelMoveSpeedX;
                            this._player.pos.x = Math.max(0, this._player.pos.x);
                        } else if(ig.input.state('up')) {
                            this._player.pos.y -= dt * this.accelMoveSpeedY;
                        } else if(ig.input.state('down')) {
                            this._player.pos.y += dt * this.accelMoveSpeedY;
                        }
                    }

                    if(ig.input.released('click')) {
                        ig.game.isDragging = false;
                    }
                } else {
                    // query accel
                    var movementModifierX = dt * this.accelMoveSpeedX;
                    var movementModifierY = dt * this.accelMoveSpeedY;

                    var accelX = this.lastAccel.data.x * (ig.ua.iOS ? 1 : -1);
                    var accelY = this.lastAccel.data.y * (ig.ua.iOS ? 1 : -1);


                    if( this._player.pos.x > 0 && accelX < 0) {
                        this._player.pos.x += accelX * movementModifierX;
                    } else if ( (this._player.pos.x < (ig.system.width - this._player.size.x)) && accelX > 0 ) {
                        this._player.pos.x += accelX * movementModifierX;
                    }

                    if( this._player.pos.y > 160 && accelY > 0 ) { // Header height
                        this._player.pos.y -= accelY * movementModifierY;
                    } else if( this._player.pos.y < ig.system.height - 160 - this._player.size.y && accelY < 0 ) { // 160 = footer height
                        this._player.pos.y -= accelY * movementModifierY;
                    }
                }

                var time = this._timer.delta();

                this.progressTimer.updateTime( Math.abs( time ) );


                if(this.gameState === ig.GameStates.Ending) {
                    if(this.getTimer('ending').delta() >= 0) {
                        this.changeState(ig.GameStates.Over);
                    }
                }

                this.frameTimer.reset();

            },

            onEnterState: function (state) {
                if(state === ig.GameStates.Ending) {
                    // this._spawner.timerEnabled = false;
                    this.getTimer('ending').set(this.gameOverDelay);
                }

                if(state === ig.GameStates.Over) {
                    // ig.output.over({ session : ig.sessionStats.currentSession });
                    if( typeof ig.config.global.gameover === 'function' ) {
                        ig.config.global.gameover( ig.sessionStats.currentSession );
                    }

                    // We need to do this in the config.
                    // this.endScene();
                }
            },

            onExitState: function (state) {

            },

            playerDied : function() {
                // Display image
                var gameOverImage = ig.game.spawnConfigEntity("gameOverDisplay");

                // disable gravity
                this.changeGravity(0);
                this.gravityChangeTimer.reset();

                // display movement
                this.accelOn = false;

                // fire time out
                setTimeout(function(){
                    ig.game.changeState(ig.GameStates.Over);
                }, 2000);
            },
            locationInEntity: function(x, y, entity) {
                return x > entity.pos.x &&
                    x < entity.pos.x + entity.size.x &&
                    y > entity.pos.y &&
                    y < entity.pos.y + entity.size.y;
            }

        });

    });


// lib/gamewheel/entities/tiledScrollingBackground.js
ig.baked=true;ig.module(
    'gamewheel.entities.tiledScrollingBackground',
    'games.stacker.entities.scrollingBackgroundSelfDestroying'
).requires(
    'impact.entity'
).defines(function(){ "use strict";
        
        ig.global.EntityTiledScrollingBackground = ig.Entity.extend({

            images : undefined,
            gravityFactor: 0,
            zIndex : -100,

            startPosition : {
                y : 0
            },

            terminationPosition : {
                y : 640
            },

            areaToCover : 640,

            fillArea : true,

            scrollingEntities : [],


            init: function (x, y, settings) {
                this.parent(x, y, settings);
                this.createEntities( -this.areaToCover );
                this.createEntities( 0 );
                this.yStart = this.startPosition.y - this.areaToCover;
            },

            createEntities : function( offset ) {
                if( this.fillArea ) {

                    var areaHeight = this.areaToCover;
                    var areaCovered = 0;

                    while( areaCovered < areaHeight ) {
                        var imageName = this.images.random();
                        var image = ig.content.imageNamed( imageName );
                        var _self = this;
                        var yPos = this.startPosition.y + areaCovered + offset;
                        var pSetup = {
                            type: ig.global.EntityScrollingBackgroundSelfDestroying,
                            //location: { x: 0, y: 0},
                            image : imageName,
                            settings: {
                                moveSpeed : 3.0,
                                pos : { x: 0, y: yPos },
                                zIndex: this.zIndex,
                                terminationPositionY : 640,
                                callback : function( entity ) {
                                    console.log( "killed" );
                                    // entity.kill();
                                    // entity.pos.y = _self.scrollingEntities[ 0 ].pos.y - image.height + 2;
                                    entity.pos.y = _self.yStart;
                                    _self.scrollingEntities.splice( _self.scrollingEntities.indexOf( entity ), 1 );
                                    _self.scrollingEntities.unshift( entity );
                                }
                            }
                        };

                        var imageEntity = ig.game.spawnEntityWithSetup( pSetup );
                        areaCovered += image.height;
                        this.scrollingEntities.push( imageEntity );
                    }

                }
            },

            switchImage : function() {
                // if( this.images && this.images.length > 0 ) {
                //     var image = ig.content.imageNamed( this.images.random() );
                //     this.image = image;
                //     this.size.x = image.width;
                //     this.size.y = image.height;
                // }
            },

            draw: function () {
                // if( this.image ) {
                //     // Stretch image to fill screen.
                //     ig.system.context.drawImage(this.image.data,
                //         this.pos.x, this.pos.y,
                //         this.size.x, this.size.y);
                // }
            },

            update : function() {

            }
        });
    });

// lib/gamewheel/entities/imageDisplay.js
ig.baked=true;ig.module(
    'gamewheel.entities.imageDisplay'
).requires(
        'impact.entity'
    ).defines(function() { "use strict";

        ig.global.EntityImageDisplay = ig.Entity.extend({

            scaleToFitEntitySize : false,

            init: function (x, y, settings) {
                this.parent(x, y, settings);
            }

        });

    });

// lib/games/catcherBK/main.js
ig.baked=true;ig.module(
    'games.catcherBK.main'
).requires(
    'impact.system',
    'gamewheel.core.gamewheel',
    'gamewheel.core.gwLoader',
    'gamewheel.entities.progressBar',
    'gamewheel.scenes.baseTemplateScene',
    'games.catcherBK.scenes.gameScene',
    'gamewheel.entities.tiledScrollingBackground',
    'gamewheel.entities.imageDisplay',
    'games.catcherBK.entities.progressTimer'
).defines(function(){

});
